type controle
	private:
		key_state as ubyte ptr
		button_state as ubyte
		
		tecla (0 to MAX_TECLAS - 1) as integer
		last_tecla (0 to MAX_TECLAS - 1) as integer
		
	public:
		mouse as coord3d
	
		joy_x as single
		joy_y as single
	
		const as integer DOWN = 0
		const as integer PRESSED = 1
		const as integer K_UP = 0
		const as integer K_DOWN = 1
		const as integer K_LEFT = 2
		const as integer K_RIGHT = 3
		const as integer K_MARCHAUP = 4
		const as integer K_MARCHADOWN = 5
		const as integer K_NITRO = 6
		const as integer K_ESCOLHE = 7
		const as integer K_RETORNA = 8
		const as integer K_PAUSA = 9
		const as integer K_CAMUP = 10
		const as integer K_CAMDOWN = 11
		const as integer K_CARUP = 12
		const as integer K_CARDOWN = 13
		const as integer K_HUD = 14
		const as integer K_LOD = 15
		const as integer K_MIRROR = 16
		const as integer K_REPLAY = 17
		const as integer K_MOD = 18
		const as integer K_MARCHA1 = 19
		const as integer K_MARCHA2 = 20
		const as integer K_MARCHA3 = 21
		const as integer K_MARCHA4 = 22
			
		declare function confereTecla (i as integer, estado as integer) as _bool_
		declare function confereBotao (i as integer, estado as integer) as _bool_
		declare sub atualiza
		declare sub esperaTecla
end type

dim shared kev as controle

'-----------------------------------------------------------------------
function controle.confereTecla (i as integer, estado as integer) as _bool_
	if (estado = DOWN) or (estado = PRESSED and last_tecla(i) = false) then
		return tecla(i)
	else
		return false
	end if
end function

function controle.confereBotao (i as integer, estado as integer) as _bool_
	if button_state and SDL_BUTTON(i) then
		if estado = PRESSED then
			'espera soltar
			while button_state and SDL_BUTTON(i)
				atualiza
			wend
		end if
		
		return true
	else
		return false
	end if
end function

sub controle.atualiza
	dim as integer i, botoes
	
	'copia estado anterior
	for i = 0 to MAX_TECLAS - 1
		last_tecla(i) = tecla(i)
	next
	
	'atualiza key state
	SDL_PumpEvents
	key_state = SDL_GetKeyState(NULL)
	
	'atualiza mouse
	dim as integer mouse_x, mouse_y	
	button_state = SDL_GetMouseState(@mouse_x, @mouse_y)
	
	mouse.x = mouse_x
	mouse.y = mouse_y
	
	'atualiza joystick fb
	if JOYSTICK_ON then
		if not USE_Y_AXIS then
			getjoystick 0, botoes, joy_x, 
		else
			getjoystick 0, botoes, joy_x, joy_y
			
			'inverte eixo y / saturao
			joy_y = edge(joy_y * -1.2, -1, 1)
		end if
		
		'saturao e deadzone eixo x		
		joy_x = edge(joy_x * 1.3, -1, 1)
		if abs(joy_x) < 0.025 then joy_x = 0
	else
		botoes = 0
	end if
		
	'mapeia sdl para vetor interno
	if USE_Y_AXIS then
		tecla(K_UP)			= key_state[SDLK_UP] or (joy_y = 1)
		tecla(K_DOWN)		= key_state[SDLK_DOWN] or (joy_y = -1)
		tecla(K_MARCHAUP)	= key_state[SDLK_A] or (botoes and (1 shl 0)) or (botoes and (1 shl 5))
		tecla(K_MARCHADOWN)	= key_state[SDLK_Z] or (botoes and (1 shl 1)) or (botoes and (1 shl 4))
	else
		tecla(K_UP)			= key_state[SDLK_UP] or (botoes and (1 shl 0))
		tecla(K_DOWN)		= key_state[SDLK_DOWN] or (botoes and (1 shl 1))
		tecla(K_MARCHAUP)	= key_state[SDLK_A] or (botoes and (1 shl 5))
		tecla(K_MARCHADOWN)	= key_state[SDLK_Z] or (botoes and (1 shl 4))
	end if
	
	tecla(K_LEFT)		= key_state[SDLK_LEFT] 
	tecla(K_RIGHT)		= key_state[SDLK_RIGHT]
	tecla(K_NITRO)		= key_state[SDLK_X]
	tecla(K_ESCOLHE)	= key_state[SDLK_RETURN] or (botoes and (1 shl 9))
	tecla(K_RETORNA)	= key_state[SDLK_ESCAPE] or (botoes and (1 shl 8))
	tecla(K_PAUSA)		= key_state[SDLK_P] or key_state[SDLK_PAUSE]
	tecla(K_CAMUP)		= key_state[SDLK_PAGEUP] or (botoes and (1 shl 7))
	tecla(K_CAMDOWN)	= key_state[SDLK_PAGEDOWN] or (botoes and (1 shl 6))
	tecla(K_CARUP)		= key_state[SDLK_HOME]
	tecla(K_CARDOWN)	= key_state[SDLK_END]
	tecla(K_HUD)		= key_state[SDLK_F1]
	tecla(K_MIRROR)		= key_state[SDLK_F2]
	tecla(K_LOD)		= key_state[SDLK_F3]
	tecla(K_REPLAY)		= key_state[SDLK_F4] or key_state[SDLK_R]
	tecla(K_MOD)		= key_state[SDLK_LSHIFT] or key_state[SDLK_RSHIFT]
	
	'alternativas de navegao com joystick
	if JOYSTICK_ON then
		tecla(K_PAUSA) = tecla(K_ESCOLHE)
		tecla(K_LEFT) = tecla(K_LEFT) or (joy_x = -1)
		tecla(K_RIGHT) = tecla(K_RIGHT) or (joy_x = 1)
	end if
	
	'simula analgico com teclado
	if not JOYSTICK_ON then
		if tecla(K_LEFT) then
			joy_x = edge(joy_x - (0.05 * fps.getFator), -1, 1)
		elseif tecla(K_RIGHT) then
			joy_x = edge(joy_x + (0.05 * fps.getFator), -1, 1)
		else
			if joy_x > 0 then joy_x = edge(joy_x - (0.10 * fps.getFator), 0, 1)
			if joy_x < 0 then joy_x = edge(joy_x + (0.10 * fps.getFator), -1, 0)
		end if
	end if
		
	if not JOYSTICK_ON or (JOYSTICK_ON and not USE_Y_AXIS) then
		if tecla(K_DOWN) then
			joy_y = edge(joy_y - (0.1 * fps.getFator), -1, 0)
		elseif tecla(K_UP) then
			joy_y = edge(joy_y + (0.1 * fps.getFator), 0, 1)
		else
			if joy_y > 0 then joy_y = edge(joy_y - (0.1 * fps.getFator), 0, 1)
			if joy_y < 0 then joy_y = edge(joy_y + (0.1 * fps.getFator), -1, 0)
		end if
	end if
end sub

sub controle.esperaTecla
	do
		'whatever
	loop until confereTecla(K_ESCOLHE, PRESSED) or confereTecla(K_RETORNA, PRESSED)
end sub
