'::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
' Este arquivo define o formato de modelo 3D e contm a geometria dos veculos.
'Funes grficas so prototipadas. Alguma geometria tem que ser gerada on the fly
'com os grficos, como as rodas e o modelo do piloto.
'::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

type triangulo
	public:
		v (0 to 2) as coord3d
		tx (0 to 2) as single
		ty (0 to 2) as single
		cor as rgbval
		normal as coord3d
		
		declare sub novo
		declare sub calculaNormal
		declare sub pinta
		declare sub pinta (cor_default as rgbval)
end type

type triangleList
	private:
		tri (0 to MAX_TRIANGULOS - 1) as triangulo
		cont as integer
		
	public:
		tex as textura
		texturizado as _bool_
		debug as _bool_
		
		declare sub nova
		declare sub add (tri as triangulo)
		declare sub addTri (v0 as coord3d, v1 as coord3d, v2 as coord3d, cor as rgbval)
		declare sub add2Tri (v0 as coord3d, v1 as coord3d, v2 as coord3d, cor as rgbval)
		declare sub addQuad (v0 as coord3d, v1 as coord3d, cor as rgbval)
		declare sub addQuadTex (v0 as coord3d, v1 as coord3d, v2 as coord3d, v3 as coord3d, _
								tx0 as single, tx1 as single, ty0 as single, ty1 as single)		
		declare sub addQuadTex (v0 as coord3d, v1 as coord3d, _
								tx0 as single, tx1 as single, ty0 as single, ty1 as single)
		declare sub add2QuadTex (v0 as coord3d, v1 as coord3d, v2 as coord3d, v3 as coord3d, _
								tx0 as single, tx1 as single, ty0 as single, ty1 as single)
		
		declare sub pinta (cor_default as rgbval)
end type

type modelo
	private:
		casco as triangleList		'plano
		decais as triangleList		'texturas sombreadas
		painel as triangleList		'texturas sem sombra
		farois as triangleList		'texturas acesas
		lanternas as triangleList
		
		volante as coord3d
		velocimetro as coord3d
		eixo (0 to 1) as double		'coordenada em y
		
		const as double TALA = 0.12	'dimenses de roda
		const as double RAIO = 0.41
		
		declare sub pintaRoda (anim as double)
		
	public:
		largura as double			'medidas do centro aos extremos
		comprimento as double		'porque  s o que nos interessa
		altura as double
		
		simples as _bool_ 			'veculo decorativo: no faz sombra, rodas, luz
		
		declare sub cria (id as string)
		declare sub destroi
		
		declare sub pintaSombra (ang as double, coord as coord3d)
		declare sub pintaBase (ang as double, anim as double)
		declare sub pintaCasco (cor_default as rgbval)
		declare sub pintaPiloto (ang as double, angvel as double, coord as coord3d)
		declare sub pintaLuz
end type

'-----------------------------------------------------------------------
sub triangulo.novo
	dim i as integer
	
	for i = 0 to 2
		v(i).nova
		tx(i) = 0
		ty(i) = 0
	next
	
	cor.r = 1
	cor.g = 1
	cor.b = 1
	
	normal.x = 1
	normal.y = 1
	normal.z = 1
end sub

sub triangulo.calculaNormal
	dim as coord3d m, n
	m.x = v(0).x - v(1).x
	m.y = v(0).y - v(1).y	
	m.z = v(0).z - v(1).z
	n.x = v(1).x - v(2).x
	n.y = v(1).y - v(2).y
	n.z = v(1).z - v(2).z
			
	normal.x = (m.y * n.z) - (m.z * n.y)
	normal.y = (m.z * n.x) - (m.x * n.z)
	normal.z = (m.x * n.y) - (m.y * n.x)
end sub

'-----------------------------------------------------------------------
sub triangleList.nova
	cont = -1
	texturizado = false
	debug = true
end sub

sub triangleList.add (tri as triangulo)
	if cont < (MAX_TRIANGULOS - 1) then
		cont = cont + 1
		this.tri(cont) = tri
		this.tri(cont).calculaNormal
		
		if debug then print ".";
	end if
end sub

sub triangleList.addTri (v0 as coord3d, v1 as coord3d, v2 as coord3d, cor as rgbval)
	dim tmp as triangulo
	tmp.novo
	
	tmp.v(0) = v0
	tmp.v(1) = v1
	tmp.v(2) = v2
	tmp.cor = cor
	
	tmp.calculaNormal
	add tmp	
end sub

sub triangleList.add2Tri (v0 as coord3d, v1 as coord3d, v2 as coord3d, cor as rgbval)
	addTri v0, v1, v2, cor
	
	dim as coord3d v3, v4, v5
	v3 = v0
	v3.x = v0.x * -1
	v4 = v1
	v4.x = v1.x * -1
	v5 = v2
	v5.x = v2.x * -1
	
	addTri v5, v4, v3, cor
end sub

sub triangleList.addQuad (v0 as coord3d, v1 as coord3d, cor as rgbval)
	dim as coord3d v2, v3
	
	v2 = v0
	v2.x = v2.x * -1
	
	v3 = v1
	v3.x = v3.x * -1
	
	addTri v0, v2, v1, cor
	addTri v2, v3, v1, cor
end sub

sub triangleList.addQuadTex (v0 as coord3d, v1 as coord3d, v2 as coord3d, v3 as coord3d, _
							tx0 as single, tx1 as single, ty0 as single, ty1 as single)
	dim tmp as triangulo
	tmp.novo
	
	tmp.v(0) = v0
	tmp.tx(0) = tx0
	tmp.ty(0) = ty0
	tmp.v(1) = v1
	tmp.tx(1) = tx0
	tmp.ty(1) = ty1
	tmp.v(2) = v2
	tmp.tx(2) = tx1
	tmp.ty(2) = ty1
	add tmp
	
	tmp.v(0) = v0
	tmp.tx(0) = tx0
	tmp.ty(0) = ty0
	tmp.v(1) = v2
	tmp.tx(1) = tx1
	tmp.ty(1) = ty1
	tmp.v(2) = v3
	tmp.tx(2) = tx1
	tmp.ty(2) = ty0
	add tmp
end sub

sub triangleList.addQuadTex (v0 as coord3d, v1 as coord3d, _
				tx0 as single, tx1 as single, ty0 as single, ty1 as single)
	dim c (0 to 3) as coord3d
	c(0) = v0
	c(1) = v1 
	c(2) = v1
	c(3) = v0
	c(0).x = c(0).x * -1
	c(1).x = c(1).x * -1
	
	addQuadTex c(0), c(1), c(2), c(3), tx0, tx1, ty0, ty1
end sub

sub triangleList.add2QuadTex (v0 as coord3d, v1 as coord3d, v2 as coord3d, v3 as coord3d, _
							tx0 as single, tx1 as single, ty0 as single, ty1 as single)
	dim as coord3d v4, v5, v6, v7
	v4 = v3
	v4.x = v4.x * -1	
	v5 = v2
	v5.x = v5.x * -1
	v6 = v1
	v6.x = v6.x * -1	
	v7 = v0
	v7.x = v7.x * -1
	
	addQuadTex v0, v1, v2, v3, tx0, tx1, ty0, ty1
	addQuadTex v4, v5, v6, v7, tx1, tx0, ty0, ty1
end sub

'-----------------------------------------------------------------------
sub modelo.cria (id as string)
	'cria listas
	casco.nova
	painel.nova
	decais.nova
	farois.nova
	lanternas.nova
	
	select case id
		case "gol"
			'define variveis
			largura = 1.25
			comprimento = 3
			altura = 2
			simples = false
		
			'carrega texturas
			painel.tex.carrega "gfx/painel.bmp", false, true
			painel.texturizado = true
			
			farois.tex.carrega "gfx/farol.bmp", true, true
			farois.texturizado = true
			
			lanternas.tex.carrega "gfx/lanterna.bmp", true, true
			lanternas.texturizado = true
			
			'cria geometria
			'refira a modelo.png
			dim nx (1 to 5) as single
			dim ny (1 to 22) as single
			dim nz (1 to 13) as single
			
			nx(1) = 1
			nx(2) = 0.9
			nx(3) = 0.65
			nx(4) = 0.55
			nx(5) = 1.5
			
			ny(1) = 0.010
			ny(2) = 0.025
			ny(3) = 0.045
			ny(4) = 0.085
			ny(5) = 0.105
			ny(6) = 0.130
			ny(7) = 0.155
			ny(8) = 0.180
			ny(9) = 0.210
			ny(10) = 0.340
			ny(11) = 0.375
			ny(12) = 0.535
			ny(13) = 0.545
			ny(14) = 0.570
			ny(15) = 0.660
			ny(16) = 0.710
			ny(17) = 0.760
			ny(18) = 0.800
			ny(19) = 0.850
			ny(20) = 0.885
			ny(21) = 0.960
			ny(22) = 0.990
			
			nz(1) = 0.170
			nz(2) = 0.240
			nz(3) = 0.330
			nz(4) = 0.435
			nz(5) = 0.500
			nz(6) = 0.560
			nz(7) = 0.620
			nz(8) = 0.670
			nz(9) = 0.675
			nz(10) = 0.705
			nz(11) = 0.720
			nz(12) = 0.900
			nz(13) = 0.960
			
			'converte medidas de 0-1 para (-x)-x
			dim i as integer
			for i = 1 to 22
				ny(i) = ((ny(i) * 2) - 1) * comprimento
			next
			
			'faz pr-scaling
			for i = 1 to 4
				nx(i) = nx(i) * largura
			next			
			
			for i = 1 to 13
				nz(i) = nz(i) * altura
			next i
			
			'::: VRTICES :::
			dim v (1 to 51) as coord3d
			
			v(1).x = 	nx(1): 	v(1).y = 	ny(6): 	v(1).z = 	nz(1)
			v(2).x = 	nx(1): 	v(2).y = 	ny(18):	v(2).z = 	nz(1)
			v(3).x = 	nx(1): 	v(3).y = 	ny(2): 	v(3).z = 	nz(5)
			v(4).x = 	nx(1): 	v(4).y = 	ny(1): 	v(4).z = 	nz(4)
			v(5).x = 	nx(1): 	v(5).y = 	ny(2): 	v(5).z = 	nz(2)
			v(6).x = 	nx(1): 	v(6).y = 	ny(3): 	v(6).z = 	nz(2)
			v(7).x = 	nx(1): 	v(7).y = 	ny(3): 	v(7).z = 	nz(3)
			v(8).x = 	nx(1): 	v(8).y = 	ny(4): 	v(8).z = 	nz(4)
			v(9).x = 	nx(1): 	v(9).y = 	ny(8): 	v(9).z = 	nz(4)
			v(10).x = 	nx(1): 	v(10).y = 	ny(9): 	v(10).z = 	nz(3)
			
			v(11).x = 	nx(1): 	v(11).y = 	ny(9): 	v(11).z = 	nz(1)
			v(12).x = 	nx(1): 	v(12).y = 	ny(16):	v(12).z = 	nz(1)
			v(13).x = 	nx(1): 	v(13).y = 	ny(16):	v(13).z = 	nz(3)
			v(14).x = 	nx(1): 	v(14).y = 	ny(17):	v(14).z = 	nz(4)
			v(15).x = 	nx(1): 	v(15).y = 	ny(19):	v(15).z = 	nz(4)
			v(16).x = 	nx(1): 	v(16).y = 	ny(20):	v(16).z = 	nz(3)
			v(17).x = 	nx(1): 	v(17).y = 	ny(20):	v(17).z = 	nz(1)
			v(18).x = 	nx(1): 	v(18).y = 	ny(21):	v(18).z = 	nz(1)
			v(19).x = 	nx(1): 	v(19).y = 	ny(22):	v(19).z = 	nz(3)
			v(20).x = 	nx(1): 	v(20).y = 	ny(22):	v(20).z = 	nz(4)
			
			v(21).x = 	nx(2): 	v(21).y = 	ny(21):	v(21).z = 	nz(6)
			v(22).x = 	nx(2): 	v(22).y = 	ny(16):	v(22).z = 	nz(9)
			v(23).x = 	nx(3): 	v(23).y = 	ny(14):	v(23).z = 	nz(12)
			v(24).x = 	nx(4): 	v(24).y = 	ny(13):	v(24).z = 	nz(13)
			v(25).x = 	nx(4): 	v(25).y = 	ny(8): 	v(25).z = 	nz(13)
			v(26).x = 	nx(3): 	v(26).y = 	ny(7): 	v(26).z = 	nz(12)
			v(27).x = 	nx(2): 	v(27).y = 	ny(3): 	v(27).z = 	nz(11)
			v(28).x = 	nx(2): 	v(28).y = 	ny(5): 	v(28).z = 	nz(10)
			v(29).x = 	nx(2): 	v(29).y = 	ny(10):	v(29).z = 	nz(9)
			v(30).x = 	nx(3): 	v(30).y = 	ny(10):	v(30).z = 	nz(12)
			
			v(31).x = 	nx(3): 	v(31).y = 	ny(9): 	v(31).z = 	nz(12)
			v(32).x = 	nx(2): 	v(32).y = 	ny(11):	v(32).z = 	nz(8)
			v(33).x = 	nx(2): 	v(33).y = 	ny(15):	v(33).z = 	nz(7)
			v(34).x = 	nx(3): 	v(34).y = 	ny(12):	v(34).z = 	nz(12)
			v(35).x = 	nx(3): 	v(35).y = 	ny(11):	v(35).z = 	nz(12)
			
			'little fixes
			v(27).x =	v(27).x - 0.04
			for i = 20 to 33
				select case i
					case 20, 21, 22, 28, 29, 32, 33
						dim tmp as coord3d
						tmp = interpola(v(9), v(27), (v(i).z - nz(4)) * nz(11))
						
						v(i).x = tmp.x
				end select
			next i	
			
			v(27).x =	v(27).x + 0.02	'no sei por que isso acontece... erro de preciso?	
			
			'especial para painel
			v(36).x = nx(1) - 0.01: 	v(36).y = ny(15):			v(36).z = 	nz(4)
			v(37).x = nx(1): 			v(37).y = ny(15):			v(37).z = 	nz(1)	
			
			'especial para decais
			v(38).x = v(33).x + 0.05:	v(38).y = v(33).y - 0.3: 	v(38).z = v(33).z + 0.2
			v(39).x = v(33).x:			v(39).y = v(38).y:			v(39).z = v(33).z
			
			v(40).x = nx(5):			v(40).y = v(33).y - 0.1:	v(40).z = v(33).z
			v(41).x = nx(5):			v(41).y = v(38).y:			v(41).z = v(38).z - 0.04
			v(42).x = nx(5):			v(42).y = v(39).y:			v(42).z = v(39).z
			
			'especial para spoiler
			v(43).x = v(25).x - 0.05:	v(43).y = v(26).y:			v(43).z = v(25).z
			
			'especial para faris e lanternas
			v(44).x = v(20).x - 0.6:	v(44).y = v(20).y:			v(44).z = v(20).z
			v(45).x = v(44).x + 0.1:	v(45).y = v(21).y:			v(45).z = v(21).z
			
			v(46).x = (v(3).x * 0.6) + (v(27).x * 0.4)
			v(46).y = (v(3).y * 0.6) + (v(27).y * 0.4)
			v(46).z = (v(3).z * 0.6) + (v(27).z * 0.4)
			v(47).x = v(46).x - 0.32:	v(47).y = v(46).y:			v(47).z = v(46).z
			v(48).x = v(47).x:			v(48).y = v(27).y:			v(48).z = v(27).z
			
			v(49).x = (v(26).x * 0.1) + (v(27).x * 0.9)
			v(49).y = (v(26).y * 0.1) + (v(27).y * 0.9)
			v(49).z = (v(26).z * 0.1) + (v(27).z * 0.9)
			
			'especial para entradas de ar
			v(50).x = v(45).x:			v(50).y = v(18).y:			v(50).z = v(18).z
			v(51).x = v(44).x:			v(51).y = v(19).y:			v(51).z = v(19).z
			
			'::: TRINGULOS :::
			dim cor as rgbval
			cor.r = 1
			cor.g = 0
			cor.b = 1
			
			'lower strip
			casco.add2Tri v(3), v(4), v(8), cor
			casco.add2Tri v(4), v(7), v(8), cor
			casco.add2Tri v(4), v(5), v(7), cor
			casco.add2Tri v(7), v(5), v(6), cor
			casco.add2Tri v(9), v(13), v(14), cor
			casco.add2Tri v(9), v(10), v(13), cor
			casco.add2Tri v(10), v(12), v(13), cor			
			casco.add2Tri v(10), v(11), v(12), cor
			casco.add2Tri v(15), v(16), v(19), cor
			casco.add2Tri v(15), v(19), v(20), cor
			casco.add2Tri v(16), v(17), v(18), cor
			casco.add2Tri v(16), v(18), v(19), cor
			
			'middle strip
			casco.add2Tri v(21), v(15), v(20), cor
			casco.add2Tri v(27), v(3), v(8), cor
			casco.add2Tri v(27), v(8), v(28), cor
			casco.add2Tri v(28), v(8), v(9), cor
			casco.add2Tri v(28), v(9), v(29), cor
			casco.add2Tri v(29), v(9), v(32), cor
			casco.add2Tri v(32), v(9), v(14), cor
			casco.add2Tri v(32), v(14), v(33), cor
			casco.add2Tri v(22), v(33), v(14), cor
			casco.add2Tri v(22), v(14), v(15), cor
			casco.add2Tri v(22), v(15), v(21), cor
			
			'upper strip
			casco.add2Tri v(26), v(25), v(43), cor
			casco.add2Tri v(26), v(27), v(28), cor
			casco.add2Tri v(26), v(28), v(31), cor
			casco.add2Tri v(25), v(26), v(31), cor
			casco.add2Tri v(25), v(31), v(30), cor
			casco.add2Tri v(25), v(30), v(35), cor
			casco.add2Tri v(25), v(35), v(24), cor
			casco.add2Tri v(24), v(35), v(34), cor
			casco.add2Tri v(24), v(34), v(23), cor
			casco.add2Tri v(23), v(34), v(33), cor
			casco.add2Tri v(23), v(33), v(22), cor
			casco.add2Tri v(30), v(32), v(35), cor
			casco.add2Tri v(30), v(29), v(32), cor
			casco.add2Tri v(48), v(27), v(49), cor
						
			'frente
			casco.addQuad v(27), v(3), cor
			casco.addQuad v(3), v(4), cor
			casco.addQuad v(4), v(5), cor
			'casco.addQuad v(18), v(19), cor
			casco.add2Tri v(19), v(18), v(50), cor
			casco.add2Tri v(19), v(50), v(51), cor
			casco.addQuad v(19), v(20), cor
			'casco.addQuad v(20), v(21), cor
			casco.addQuad v(44), v(45), cor
			casco.addQuad v(21), v(22), cor
			casco.addQuad v(23), v(24), cor
			casco.addQuad v(24), v(25), cor
			casco.addQuad v(25), v(26), cor
			casco.addQuad v(25), v(43), cor
			
			'decais
			casco.add2Tri v(33), v(39), v(38), cor
			casco.add2Tri v(40), v(41), v(42), cor
			casco.add2Tri v(38), v(41), v(33), cor
			casco.add2Tri v(41), v(40), v(33), cor
			
			cor.r = 0.75
			cor.g = 0.75
			cor.b = 0.75
			casco.add2Tri v(38), v(39), v(41), cor
			casco.add2Tri v(41), v(39), v(42), cor
			
			'interior
			cor.r = 0.3
			cor.g = 0.3
			cor.b = 0.3
			casco.addQuad v(12), v(11), cor
			casco.addQuad v(11), v(10), cor
			casco.addQuad v(10), v(9), cor
			casco.addQuad v(9), v(28), cor
			casco.addQuad v(28), v(27), cor
			
			cor.r = 0
			cor.g = 0
			cor.b = 0
			casco.add2Tri v(33), v(37), v(36), cor
			casco.addQuad v(13), v(22), cor
			casco.addQuad v(12), v(13), cor
			
			'entrada de ar
			casco.addQuad v(51), v(50), cor
						
			'texturizados
			dim tmp as triangulo
			tmp.novo
			
			tmp.tx(0) = 0.05
			tmp.tx(1) = 0.01
			tmp.tx(2) = 0.5
			tmp.ty(0) = 0
			tmp.ty(1) = 0.06
			tmp.ty(2) = 0.06
			tmp.v(0) = v(22)
			tmp.v(0).x = tmp.v(0).x * -1
			tmp.v(1) = v(33)
			tmp.v(1).x = tmp.v(1).x * -1
			tmp.v(2) = v(33)
			tmp.v(2).x = 0
			painel.add tmp

			tmp.tx(0) = 0.05
			tmp.tx(1) = 0.5
			tmp.tx(2) = 0.95
			tmp.ty(0) = 0
			tmp.ty(1) = 0.06
			tmp.ty(2) = 0
			tmp.v(0) = v(22)
			tmp.v(0).x = tmp.v(0).x * -1
			tmp.v(1) = v(33)
			tmp.v(1).x = 0
			tmp.v(2) = v(22)
			painel.add tmp

			tmp.tx(0) = 0.95
			tmp.tx(1) = 0.5
			tmp.tx(2) = 0.99
			tmp.ty(0) = 0
			tmp.ty(1) = 0.06
			tmp.ty(2) = 0.06
			tmp.v(0) = v(22)
			tmp.v(1) = v(33)
			tmp.v(1).x = 0
			tmp.v(2) = v(33)
			painel.add tmp
			
			painel.addQuadTex v(33), v(37), 0.01, 0.99, 0.06, 0.495	
					
			farois.add2QuadTex v(21), v(20), v(44), v(45), 0, 1, 0, 1
			
			'lanternas
			dim as integer res = 2 'fica bonito com 1, mas no combina com visual low-poly
			for i = 0 to (16 - res) step res
				dim as coord3d centro, v1, v2
				centro = interpola(v(27), v(47), 0.7)
				centro.x = interpola(v(27), v(47), 0.5).x
				
				'1
				tmp.v(0) = centro
				tmp.v(1).x = interpola(v(27), v(48), 0.5 + (tabs.cosseno(i) * 0.6)).x
				tmp.v(1).y = interpola(v(27), v(46), 0.7 + (tabs.seno(i) * 0.5)).y
				tmp.v(1).z = interpola(v(27), v(46), 0.7 + (tabs.seno(i) * 0.5)).z
				tmp.v(2).x = interpola(v(27), v(48), 0.5 + (tabs.cosseno(i + res) * 0.6)).x
				tmp.v(2).y = interpola(v(27), v(46), 0.7 + (tabs.seno(i + res) * 0.5)).y
				tmp.v(2).z = interpola(v(27), v(46), 0.7 + (tabs.seno(i + res) * 0.5)).z
				
				tmp.tx(0) = 0.5: tmp.ty(0) = 0.5
				tmp.tx(1) = 0.5 + (tabs.cosseno(i) * 0.5): tmp.ty(1) = 0.5 + (tabs.seno(i) * 0.5)
				tmp.tx(2) = 0.5 + (tabs.cosseno(i + res) * 0.5): tmp.ty(2) = 0.5 + (tabs.seno(i + res) * 0.5)
				
				lanternas.add tmp
				
				'2
				dim inverso as triangulo
				inverso = tmp
				inverso.v(0) = tmp.v(2)
				inverso.v(0).x = tmp.v(2).x * -1
				inverso.v(1) = tmp.v(1)
				inverso.v(1).x = tmp.v(1).x * -1
				inverso.v(2) = tmp.v(0)
				inverso.v(2).x = tmp.v(0).x * -1
				
				inverso.tx(0) = tmp.tx(2) * -1: inverso.ty(0) = tmp.ty(2)
				inverso.tx(1) = tmp.tx(1) * -1
				inverso.tx(2) = tmp.tx(0) * -1: inverso.ty(2) = tmp.ty(0)
				
				lanternas.add inverso
			next i
						
			'::: EIXOS :::
			eixo(0) = v(1).y
			eixo(1) = v(2).y
			
			volante.x = -0.52
			volante.y = ny(15) - 0.1
			volante.z = nz(5) + 0.01

			velocimetro.x = volante.x - 0.002
			velocimetro.y = volante.y + 0.08
			velocimetro.z = volante.z + 0.160
				
		case "crane"
			'define variveis
			largura = 1.75
			comprimento = 3.5
			altura = 3.5
			simples = true
			
			'carrega imagens
			decais.tex.carrega "gfx/guindaste.bmp", false, true
			decais.texturizado = true
			
			'cria geometria
			dim nx (1 to 3) as single
			dim ny (1 to 8) as single
			dim nz (1 to 8) as single
			
			nx(1) = 1
			nx(2) = 0.4
			nx(3) = 0.1
			
			ny(1) = 0
			ny(2) = 0.1
			ny(3) = 0.15
			ny(4) = 0.25
			ny(5) = 0.55
			ny(6) = 0.7
			ny(7) = 0.8
			ny(8) = 1
			
			nz(1) = 0
			nz(2) = 0.28
			nz(3) = 0.55
			nz(4) = 0.7
			nz(5) = 1
			nz(6) = 2
			nz(7) = 4.6
			nz(8) = 5
			
			'converte medidas de 0-1 para (-x)-x
			dim i as integer
			for i = 1 to 8
				ny(i) = (((1 - ny(i)) * 2) - 1) * comprimento
			next
			
			'faz pr-scaling
			for i = 1 to 3
				nx(i) = nx(i) * largura
			next			
			
			for i = 1 to 8
				nz(i) = nz(i) * altura
			next i
			
			'::: VRTICES :::
			dim v (1 to 21) as coord3d
			
			v(1).x = 	nx(1): 	v(1).y = 	ny(2): 	v(1).z = 	nz(4)
			v(2).x = 	nx(1): 	v(2).y = 	ny(1): 	v(2).z = 	nz(3)
			v(3).x = 	nx(1): 	v(3).y = 	ny(1): 	v(3).z = 	nz(2)
			v(4).x = 	nx(1): 	v(4).y = 	ny(8): 	v(4).z = 	nz(2)
			v(5).x = 	nx(1): 	v(5).y = 	ny(8): 	v(5).z = 	nz(4)
			v(6).x = 	nx(1): 	v(6).y = 	ny(1): 	v(6).z = 	nz(1)
			v(7).x = 	nx(1): 	v(7).y = 	ny(8): 	v(7).z = 	nz(1)
			
			v(8).x = 	nx(2): 	v(8).y = 	ny(4): 	v(8).z = 	nz(5)
			v(9).x = 	nx(2): 	v(9).y = 	ny(3): 	v(9).z = 	nz(4)
			v(10).x = 	nx(2): 	v(10).y = 	ny(5): 	v(10).z = 	nz(4)
			v(11).x = 	nx(2): 	v(11).y = 	ny(5): 	v(11).z = 	nz(5)
			
			v(12).x = 	nx(3): 	v(12).y = 	ny(6): 	v(12).z = 	nz(6)
			v(13).x = 	nx(3): 	v(13).y = 	ny(6): 	v(13).z = 	nz(2)
			v(14).x = 	nx(3): 	v(14).y = 	ny(7): 	v(14).z = 	nz(2)
			v(15).x = 	nx(3): 	v(15).y = 	ny(7): 	v(15).z = 	nz(6)
			v(16).x = 	nx(3): 	v(16).y = 	ny(6): 	v(16).z = 	nz(7)	
			v(17).x = 	nx(3): 	v(17).y = 	ny(7): 	v(17).z = 	nz(7)
			
			v(18).x =	nx(2):	v(18).y =	ny(6) + 1:	v(18).z =	nz(8)
			v(19).x =	nx(2):	v(19).y =	ny(6) + 1:	v(19).z =	nz(7)
			v(20).x =	nx(2):	v(20).y =	ny(7) - 1:	v(20).z =	nz(7)
			v(21).x =	nx(2):	v(21).y =	ny(7) - 1:	v(21).z =	nz(8)
			
			'inclinao do brao
			v(12).y = v(12).y - 1
			v(15).y = v(15).y - 1
			
			for i = 16 to 21
				v(i).y = v(i).y + 6
			next i
			
			'deslocamento da cesta
			for i = 18 to 21
				v(i).y = v(i).y + (1 * largura)
				v(i).z = v(i).z - (0.3 * altura)
			next i
									
			'::: TRINGULOS :::
			dim cor as rgbval
			cor.r = 1
			cor.g = 0.9
			cor.b = 0
			
			'lateral
			casco.add2Tri v(3), v(2), v(1), cor
			casco.add2Tri v(5), v(3), v(1), cor
			casco.add2Tri v(4), v(3), v(5), cor
			
			'cabine
			casco.add2Tri v(10), v(9), v(8), cor
			casco.add2Tri v(11), v(10), v(8), cor
			casco.addQuad v(11), v(10), cor
			casco.addQuad v(8), v(11), cor
			
			'corpo
			casco.addQuad v(2), v(1), cor
			casco.addQuad v(1), v(5), cor
			casco.addQuad v(5), v(4), cor
			
			'brao
			casco.add2Tri v(15), v(13), v(12), cor
			casco.add2Tri v(14), v(13), v(15), cor
			casco.addQuad v(13), v(12), cor	
			casco.addQuad v(15), v(14), cor

			casco.add2Tri v(17), v(12), v(16), cor
			casco.add2Tri v(15), v(12), v(17), cor
			casco.addQuad v(17), v(15), cor
			casco.addQuad v(12), v(16), cor
			
			'cesta
			cor.r = 0.3
			cor.g = 0.25
			cor.b = 0.2
			casco.add2Tri v(18), v(19), v(20), cor
			casco.add2Tri v(18), v(20), v(21), cor
			casco.addQuad v(19), v(18), cor
			casco.addQuad v(20), v(21), cor
			casco.addQuad v(20), v(19), cor
						
			'texturizados
			decais.addQuadTex v(3), v(2), 0.01, 0.49, 0.74, 0.6
			decais.addQuadTex v(9), v(8), 0.01, 0.49, 0.49, 0.01
			
			decais.addQuadTex v(6), v(3), 0.51, 0.99, 0.76, 0.51
			decais.addQuadTex v(7), v(4), 0.51, 0.99, 0.76, 0.51
			decais.add2QuadTex v(4), v(7), v(6), v(3), 0, 1.5, 0.76, 0.99
			
		case "van"
			'define variveis
			largura = 2
			comprimento = 4.5
			altura = 4
			simples = true
			
			'carrega imagens
			decais.tex.carrega "gfx/van.bmp", false, true
			decais.texturizado = true
			
			'cria geometria
			dim nx (1 to 1) as single
			dim ny (1 to 3) as single
			dim nz (1 to 2) as single
			
			nx(1) = 1
			
			ny(1) = 0
			ny(2) = 0.1
			ny(3) = 1
			
			nz(1) = 0
			nz(2) = 1
			
			'converte medidas de 0-1 para (-x)-x
			dim i as integer
			for i = 1 to 3
				ny(i) = ((ny(i) * 2) - 1) * comprimento
			next
			
			'faz pr-scaling
			for i = 1 to 1
				nx(i) = nx(i) * largura
			next			
			
			for i = 1 to 2
				nz(i) = nz(i) * altura
			next i
			
			'::: VRTICES :::
			dim v (1 to 4) as coord3d
			
			v(1).x = 	nx(1): 	v(1).y = 	ny(2): 	v(1).z = 	nz(2)
			v(2).x = 	nx(1): 	v(2).y = 	ny(1): 	v(2).z = 	nz(1)
			v(3).x = 	nx(1): 	v(3).y = 	ny(3): 	v(3).z = 	nz(1)
			v(4).x = 	nx(1): 	v(4).y = 	ny(3): 	v(4).z = 	nz(2)
			
			'::: TRINGULOS :::
			dim cor as rgbval
			cor.r = 1
			cor.g = 1
			cor.b = 1
			
			casco.addQuad v(4), v(1), cor
			
			decais.add2QuadTex v(1), v(2), v(3), v(4), 0.01, 0.99, 0.51, 0.99
			decais.addQuadTex v(3), v(4), 0.51, 0.99, 0.49, 0.01
			decais.addQuadTex v(1), v(2), 0.01, 0.49, 0.01, 0.49
			
		case "balao"
			'define variveis
			largura = 8
			comprimento = 8
			altura = 18
			simples = true
			
			'gera geometria do balo
			dim as integer i, j
			dim as integer xres = 1
			dim as integer zres = 4
			
			for i = 0 to 16 - xres step xres
				for j = 0 to 16 - zres step zres
					dim as double x1, y1, z1, x2, y2, z2
					x1 = tabs.cosseno(i) * largura
					y1 = tabs.seno(i) * largura
					z1 = ((j / 8) * altura) ^ 0.8
					
					x2 = tabs.cosseno(i + xres) * largura
					y2 = tabs.seno(i + xres) * largura
					z2 = (((j + zres) / 8) * altura) ^ 0.8
				
					dim as double xmod1 = 0
					dim as double xmod2 = 0
					
					if j = 0 then
						xmod1 = 0.4
						xmod2 = 0.2
					elseif j = 4 then
						xmod1 = 0.2
						xmod2 = 0
					end if					
				
					dim v (1 to 4) as coord3d
					v(1).x = x1: v(1).y = y1: v(1).z = z1
					v(2).x = x1: v(2).y = y1: v(2).z = z2
					v(3).x = x2: v(3).y = y2: v(3).z = z2
					v(4).x = x2: v(4).y = y2: v(4).z = z1
										
					v(1).x = v(1).x * (tabs.seno(j / 2) + xmod1)
					v(1).y = v(1).y * (tabs.seno(j / 2) + xmod1)
					v(4).x = v(4).x * (tabs.seno(j / 2) + xmod1)
					v(4).y = v(4).y * (tabs.seno(j / 2) + xmod1)
					
					v(2).x = v(2).x * (tabs.seno((j + zres) / 2) + xmod2)
					v(2).y = v(2).y * (tabs.seno((j + zres) / 2) + xmod2)
					v(3).x = v(3).x * (tabs.seno((j + zres) / 2) + xmod2)
					v(3).y = v(3).y * (tabs.seno((j + zres) / 2) + xmod2)
					
					dim cor as rgbval
					cor = palete(i mod MAX_CORES) 'palete((int(i + (j / zres)) mod 16) mod MAX_CORES)
					
					casco.addTri v(3), v(2), v(1), cor
					casco.addTri v(4), v(3), v(1), cor
				next j
			next i
			
			'detalhes
			dim cor as rgbval
						
			dim v (1 to 8) as coord3d
			v(1).x = 2: v(1).y = -2: v(1).z = -5
			v(2).x = 2: v(2).y = -2: v(2).z = -8
			v(3).x = 2: v(3).y = 2: v(3).z = -8
			v(4).x = 2: v(4).y = 2: v(4).z = -5
			
			v(5) = v(1)
			v(6) = v(1)
			v(7) = v(4)
			v(8) = v(4)
			
			for i = 5 to 8
				v(i).z = 8
			next i
			v(6).y = 0
			v(7).y = 0
			
			'cesta
			cor.r = 0.5
			cor.g = 0.3
			cor.b = 0.2
			
			decais.add2Tri v(1), v(2), v(3), cor
			decais.add2Tri v(1), v(3), v(4), cor
			decais.addQuad v(1), v(2), cor
			decais.addQuad v(2), v(3), cor
			decais.addQuad v(3), v(4), cor
			
			'cordas
			cor.r = 0
			cor.g = 0
			cor.b = 0
			
			decais.add2Tri v(5), v(1), v(6), cor
			decais.add2Tri v(7), v(4), v(8), cor
			
		case "casa"
			'define variveis
			largura = 5
			comprimento = 3
			altura = 7
			simples = true
			
			'carrega imagens
			decais.tex.carrega "gfx/casa.bmp", false, true
			decais.texturizado = true
			
			'gera geometria
			dim v (1 to 5) as coord3d
			
			v(1).y = comprimento * -1:	v(1).x = largura:		v(1).z = altura * 0.6
			v(2).y = comprimento * -1:	v(2).x = largura:		v(2).z = 0
			v(3).y = comprimento:		v(3).x = largura:		v(3).z = 0
			v(4).y = comprimento:		v(4).x = largura:		v(4).z = altura * 0.6
			v(5).y = 0:					v(5).x = largura:		v(5).z = altura
			
			'paredes
			decais.addQuadTex v(1), v(2), 0, 1, 0.5, 1
			decais.addQuadTex v(3), v(4), 0, 1, 1, 0.5
			decais.add2QuadTex v(1), v(2), v(3), v(4), 0.375, 1, 0.5, 1
			
			'telhado
			decais.addQuadTex v(5), v(1), 0, 1, 0, 0.5
			decais.addQuadTex v(4), v(5), 0, 1, 0.5, 0
			
			dim tmp as triangulo
			tmp.novo
			
			tmp.v(0) = v(5)
			tmp.v(1) = v(1)
			tmp.v(2) = v(4)
			tmp.tx(0) = 0.25
			tmp.ty(0) = 0
			tmp.tx(1) = 0
			tmp.ty(1) = 0.5
			tmp.tx(2) = 0.5
			tmp.ty(2) = 0.5			
			decais.add tmp
			
			tmp.v(0) = v(5)
			tmp.v(1) = v(4)
			tmp.v(2) = v(1)
			tmp.v(0).x = tmp.v(0).x * -1
			tmp.v(1).x = tmp.v(1).x * -1
			tmp.v(2).x = tmp.v(2).x * -1
			decais.add tmp
	end select	
end sub

sub modelo.destroi
	painel.tex.destroi
	decais.tex.destroi
	farois.tex.destroi
	lanternas.tex.destroi
end sub
