type somBuffer
	public:
		snd as ALuint
	
		declare sub carrega (arquivo as string)
		declare sub destroi
end type

type somEngine
	private:
		voz (0 to MAX_CARROS + 1) as ALuint 'MAX_CARROS motor, +2 player skid/rumble
		efeito as ALuint
		
		declare function outOfRange (i as integer) as _bool_
		
	public:	
		declare sub cria
		declare sub destroi
		
		declare sub startYourEngines
		declare sub para
		
		declare sub toca (buf as somBuffer, p as coord3d, vol as ALfloat)
		declare sub defineVoz (i as integer, buf as somBuffer)
		declare sub setPos (i as integer, p as coord3d)
		declare sub setVelocity (i as integer, v as coord3d)
		declare sub setVolume (i as integer, vol as ALfloat)
		declare sub setPitch (i as integer, pitch as ALfloat)
		
		declare function getPlayer () as integer
end type

type musica
	private:
		buffer(0 to 1)	as ALuint
		source			as ALuint
		formato			as ALenum
		
		ogg_file		as FILE ptr
		ogg_stream		as OggVorbis_File
		ogg_info		as vorbis_info ptr
		tocando			as byte	
				
		declare function carregaBuffer (buffer_atual as ALuint) as integer	'retorna false se o stream acabou
		declare function atualiza () as integer								'passa o retorno de carregaBuffer
		declare sub pausa
		
	public:
		declare function toca () as integer									'retorna false se no houver mais o que tocar
		declare sub setVolume (volume as ALfloat)							'valor de fade (0-1) relativo a VOL_MUSICA
		
		declare sub carrega (arquivo as string)
		declare sub destroi
end type

'-----------------------------------------------------------------------
sub somBuffer.carrega (arquivo as string)
	snd = alutCreateBufferFromFile(arquivo)
end sub

sub somBuffer.destroi
	alDeleteBuffers 1, @snd
end sub

'-----------------------------------------------------------------------
function somEngine.getPlayer () as integer
	return MAX_CARROS
end function

function somEngine.outOfRange (i as integer) as _bool_
	if i < 0 or i > MAX_CARROS + 1 then
		return true
	else
		return false
	end if
end function

sub somEngine.cria
	dim i as integer
	
	'cria sources
	alGenSources MAX_CARROS + 2, @voz(0)
	alGenSources 1, @efeito
	
	'define propriedades
	for i = 0 to MAX_CARROS + 1
		alSourcei voz(i), AL_LOOPING, AL_TRUE
		alSourcef voz(i), AL_PITCH, 1.0
		alSourcef voz(i), AL_GAIN, 1.0
		alSourcei voz(i), AL_REFERENCE_DISTANCE, 20
		alSourcei voz(i), AL_ROLLOFF_FACTOR, 0.95
	next i

	alSourcei efeito, AL_LOOPING, AL_FALSE
	alSourcei efeito, AL_PITCH, 1.0
	alSourcei efeito, AL_GAIN, 1.0
	alSourcei efeito, AL_REFERENCE_DISTANCE, 20
	alSourcei efeito, AL_ROLLOFF_FACTOR, 0.95
end sub

sub somEngine.destroi
	dim i as integer
	
	'desliga buffers
	for i = 0 to MAX_CARROS + 1
		alSourcei voz(i), AL_BUFFER, NULL
	next
	
	alSourcei efeito, AL_BUFFER, NULL
	
	'remove sources
	alDeleteSources MAX_CARROS + 2, @voz(0)
	alDeleteSources 1, @efeito
end sub

sub somEngine.startYourEngines
	dim i as integer
	
	if not SFX_ON then exit sub
	
	for i = 0 to MAX_CARROS + 1
		alSourcePlay voz(i)
	next i
end sub

sub somEngine.para
	dim i as integer
	
	for i = 0 to MAX_CARROS + 1
		alSourceStop voz(i)
	next i
end sub

sub somEngine.toca (buf as somBuffer, p as coord3d, vol as ALfloat)
	'pode tocar?
	if not SFX_ON then exit sub

	'j est tocando?
	dim estado as ALenum
	
	alGetSourcei efeito, AL_SOURCE_STATE, @estado
	if not estado = AL_PLAYING then
		'define buffer
		alSourcei efeito, AL_BUFFER, buf.snd
		alSource3f efeito, AL_POSITION, p.x, p.y, p.z
		alSourcef efeito, AL_GAIN, vol
		
		'toca
		alSourcePlay efeito
	end if
end sub

sub somEngine.defineVoz (i as integer, buf as somBuffer)
	if outOfRange(i) then exit sub
	alSourcei voz(i), AL_BUFFER, buf.snd
end sub

sub somEngine.setPos (i as integer, p as coord3d)
	if outOfRange(i) then exit sub
	alSource3f voz(i), AL_POSITION, p.x, p.y, p.z
end sub

sub somEngine.setVelocity (i as integer, v as coord3d)
	if outOfRange(i) then exit sub
	alSource3f voz(i), AL_VELOCITY, v.x, v.y, v.z 
end sub

sub somEngine.setVolume (i as integer, vol as ALfloat)
	if outOfRange(i) then exit sub
	alSourcef voz(i), AL_GAIN, vol
end sub

sub somEngine.setPitch (i as integer, pitch as ALfloat)
	if outOfRange(i) then exit sub
	alSourcef voz(i), AL_PITCH, pitch
end sub

'-----------------------------------------------------------------------
function musica.carregaBuffer (buffer_atual as ALuint) as integer
	dim as zstring * BUFFER_VORBIS chunk
	dim as integer bytes = 0
	dim as integer lidos, bitstream
	
	'enche o tamanho de um buffer	
	while bytes < BUFFER_VORBIS
		lidos = ov_read(@ogg_stream, @chunk[bytes], BUFFER_VORBIS - bytes, 0, 2, 1, @bitstream)
	
		if lidos < 0 then
			print "Erro lendo a stream!"
			destroi
			end
		'fim da stream
		elseif lidos = 0 then
			exit while
		'tem mais
		else
			bytes = bytes + lidos
		end if
	wend
	
	'carregou alguma coisa?
	if bytes = 0 then return false
	
	'liga source ao buffer
	alBufferData buffer_atual, formato, @chunk, bytes, ogg_info->rate
	return true
end function

function musica.atualiza () as integer
	dim buffer_atual as ALuint
	dim processados as integer
	dim as integer ativo = true
	
	alGetSourcei source, AL_BUFFERS_PROCESSED, @processados
	
	'faz troca de buffers
	while processados > 0
		alSourceUnqueueBuffers source, 1, @buffer_atual
		ativo = carregaBuffer(buffer_atual)
		alSourceQueueBuffers source, 1, @buffer_atual

		processados = processados - 1
	wend
	
	if not ativo then pausa
	return ativo
end function

sub musica.pausa
	dim buffer_atual as ALuint
	dim enfileirados as integer
	
	'esvazia os buffers
	alGetSourcei source, AL_BUFFERS_QUEUED, @enfileirados
		
	while enfileirados > 0
		alSourceUnqueueBuffers source, 1, @buffer_atual
	
		enfileirados = enfileirados - 1
	wend
	
	'interrompe o playback
	tocando = false
	alSourceStop source
end sub

function musica.toca () as integer
	dim estado as ALenum
		
	'pode tocar?
	if not MUSICA_ON then
		pausa		
		return false

	'j comeou a tocar?
	elseif tocando then
		'checa se o stream no engasgou
		alGetSourcei source, AL_SOURCE_STATE, @estado
		if estado <> AL_PLAYING then alSourcePlay source
	
	'esvaziou todos os buffers
	else
		'h mais para carregar?
		if not carregaBuffer(buffer(0)) then return false
		if not carregaBuffer(buffer(1)) then return false
		
		'enfileira e toca novo
		alSourceQueueBuffers(source, 2, @buffer(0))
		alSourcePlay source
		tocando = true
	end if
	
	'faz streaming
	return atualiza
end function

sub musica.setVolume (volume as ALfloat)
	alSourcef source, AL_GAIN, volume * (VOL_MUSICA / 128)
end sub

sub musica.carrega (arquivo as string)
	'abre arquivo
	print "Abrindo stream: "; arquivo
	ogg_file = fopen(arquivo, "rb")
	
	if ogg_file = NULL then
		print "Arquivo nao encontrado"
		end
	end if
	
	'passa para libvorbis
	if ov_open(ogg_file, @ogg_stream, NULL, 0) < 0 then
		print "Erro abrindo arquivo"
		fclose ogg_file
		end
    end if
    
    'l informaes
    ogg_info = ov_info(@ogg_stream, -1)
    
    if ogg_info->channels = 1 then
		formato = AL_FORMAT_MONO16
	else
		formato = AL_FORMAT_STEREO16
	end if
	
	'gera buffers
	alGenBuffers 2, @buffer(0)
	alGenSources 1, @source
	
	'valores default
	tocando = false
	setVolume 1
	
	'debug
	print "Canais: "; ogg_info->channels
	print "Frequencia: "; ogg_info->rate
end sub

sub musica.destroi
	pausa
	alDeleteBuffers 2, @buffer(0)
	alDeleteSources 1, @source
	ov_clear @ogg_stream
end sub
