'::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
' Este arquivo contm cdigo relacionado ao 'track format' que  compartilhado pelo
'editor de circuitos e pelo jogo em si. Tambm rotinas relacionadas a coord3d,
'clculo de distncias, coliso com polgonos.
'::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

const as integer MAX_VERTICES = 256
const as double FATOR_MAG = 1.2

type coord3d
	public:
		x as double
		y as double
		z as double
		
		declare sub nova
end type

type roadVertex
	public:
		quad(0 to 7) as coord3d	'malha 3D, gerar com genQuads()
								'0-1: limites do asfalto
								'2-3: limites da pista inteira
								'4-5, 6-7: faixas de tinta
		path as coord3d			'melhor linha de IA
		
		'propriedades da pista
		coord as coord3d	
		ang as double
		dist as double
		roadWidth as double
		escapeWidth(0 to 1) as double
		cp_id as integer
		
		declare sub novo
		declare sub genQuads
end type

type distCoord
	public:
		coord as coord3d
		dist as double
end type

type carBoundaries
	public:
		coord (0 to 7) as coord3d
end type

type vertexList
	private:
		v (0 to MAX_VERTICES - 1) as roadVertex 
		
		declare function melhorCaminho (inicio as integer, i as integer, byval atual as distCoord) as distCoord
		
		const as integer TS_ASFALTO = 1
		const as integer TS_MURO0 = 2
		const as integer TS_MURO1 = 3
		const as integer TS_FAIXA = 4
		declare sub pintaTriangleStrip (i1 as integer, i2 as integer, modo as integer)
						
	public:
		cont as integer
		
		seed as double
		cenario as integer 	'0 = summer | 1 = autumn | 2 = winter
		aclive as integer 	'0 = flat   | 1 = normal | 2 = high
		
		cp (0 to 63) as coord3d 'pontos de controle para o editor
		cp_cont as integer
	
		'essas rotinas so definidas
		'(geral)
		declare sub nova
		declare sub load (arq as string)
		declare sub dump (arq as string)
		declare function getVertex (i as integer) as roadVertex
		
		'(editor)
		declare sub add (v as roadVertex)
		declare sub atualiza (i as integer, v as roadVertex)
		declare sub atualizaUltimo
		declare sub removeUltimo
		declare sub centraliza
		declare sub pathfinding
		
		'(simulao)
		declare function encontra (q as coord3d) as integer									'retorna o vrtice onde uma coordenada se encontra
		declare function dentro (q as coord3d, i as integer, asfalto as _bool_) as integer	'testa se uma coordenada est dentro de tal vrtice
		declare function colide (b as carBoundaries) as integer								'testa se um carro inteiro est dentro da pista
		declare function somaAngulos () as double											'descobre se  horrio ou anti-horrio
		declare function getRetaOposta () as integer										'retorna o maior vrtice <> 0 e -1
		
		'essas rotinas so apenas prototipadas
		declare sub pinta
		declare sub pintaSimples
		declare sub pintaMapa
		declare sub pintaEditor
end type

declare function dentroQuad (q as coord3d, p1 as coord3d, p2 as coord3d, p3 as coord3d, p4 as coord3d) as integer
declare function ang2p (p1 as coord3d, p2 as coord3d) as double
declare function dist2p (p1 as coord3d, p2 as coord3d) as double
declare function interpola (c1 as coord3d, c2 as coord3d, delta as double) as coord3d

'***
'checa coliso de um ponto com um quad (2D)
function dentroQuad (q as coord3d, p1 as coord3d, p2 as coord3d, p3 as coord3d, p4 as coord3d) as integer
	dim as coord3d t1, t2
	dim as double soma
	
	'calcula soma dos ngulos com q
	soma = 0
	
	t1.x = p1.x - q.x
	t1.y = p1.y - q.y
	t2.x = p2.x - q.x
	t2.y = p2.y - q.y
	soma += wrap(atan2(t1.y, t1.x) - atan2(t2.y, t2.x), pi * -1, pi)
	
	t1.x = p2.x - q.x
	t1.y = p2.y - q.y
	t2.x = p3.x - q.x
	t2.y = p3.y - q.y
	soma += wrap(atan2(t1.y, t1.x) - atan2(t2.y, t2.x), pi * -1, pi)
	
	t1.x = p3.x - q.x
	t1.y = p3.y - q.y
	t2.x = p4.x - q.x
	t2.y = p4.y - q.y
	soma += wrap(atan2(t1.y, t1.x) - atan2(t2.y, t2.x), pi * -1, pi)
				
	if soma > pi then
		return true
	else
		return false
	end if
end function

'***
'calcula o ngulo (2D) entre dois pontos
function ang2p (p1 as coord3d, p2 as coord3d) as double
	return atan2(p2.y - p1.y, p2.x - p1.x)
end function

'***
'calcula a distncia (2D) entre dois pontos
function dist2p (p1 as coord3d, p2 as coord3d) as double
	return sqr(((p1.x - p2.x) ^ 2) + ((p1.y - p2.y) ^ 2))
end function

'***
'encontra um ponto entre outros dois
function interpola (c1 as coord3d, c2 as coord3d, delta as double) as coord3d
	dim resultado as coord3d
	
	resultado.x = (c1.x * (1 - delta)) + (c2.x * delta)
	resultado.y = (c1.y * (1 - delta)) + (c2.y * delta)
	resultado.z = (c1.z * (1 - delta)) + (c2.z * delta)
	
	return resultado
end function

'------------------------------------------------------------------------------------
sub coord3d.nova
	x = 0
	y = 0
	z = 0
end sub

'------------------------------------------------------------------------------------
sub roadVertex.novo
	dim as integer i

	coord.nova
	path.nova
	ang = 0
	dist = 0
	roadWidth = 8
	escapeWidth(0) = 10
	escapeWidth(1) = 10
	
	for i = 0 to 7
		quad(i).nova
	next
end sub

sub roadVertex.genQuads
	quad(0).x = coord.x + (roadWidth * cos(ang - (pi / 2)))
	quad(0).y = coord.y + (roadWidth * sin(ang - (pi / 2)))
	quad(1).x = coord.x + (roadWidth * cos(ang + (pi / 2)))
	quad(1).y = coord.y + (roadWidth * sin(ang + (pi / 2)))
	
	quad(2).x = coord.x + ((roadWidth + escapeWidth(0)) * cos(ang - (pi / 2)))
	quad(2).y = coord.y + ((roadWidth + escapeWidth(0)) * sin(ang - (pi / 2)))
	quad(3).x = coord.x + ((roadWidth + escapeWidth(1)) * cos(ang + (pi / 2)))
	quad(3).y = coord.y + ((roadWidth + escapeWidth(1)) * sin(ang + (pi / 2)))
	
	quad(4).x = (quad(0).x * (127 / 128)) + (quad(1).x * (1 / 128))
	quad(4).y = (quad(0).y * (127 / 128)) + (quad(1).y * (1 / 128))
	quad(5).x = (quad(0).x * (123 / 128)) + (quad(1).x * (5 / 128))
	quad(5).y = (quad(0).y * (123 / 128)) + (quad(1).y * (5 / 128))
	quad(6).x = (quad(1).x * (127 / 128)) + (quad(0).x * (1 / 128))
	quad(6).y = (quad(1).y * (127 / 128)) + (quad(0).y * (1 / 128))
	quad(7).x = (quad(1).x * (123 / 128)) + (quad(0).x * (5 / 128))
	quad(7).y = (quad(1).y * (123 / 128)) + (quad(0).y * (5 / 128))
end sub

'------------------------------------------------------------------------------------
sub vertexList.nova
	cont = -1
end sub

sub vertexList.load (arq as string)
	dim as roadVertex tmp
	dim as string header
	dim as integer i
	
	nova
	cp_cont = 0
	print "Lendo circuito";
	
	open arq for input as #1
		while not eof(1)
			line input #1, header
			
			if instr(header, "seed") > 0 then
				input #1, seed
			
			elseif instr(header, "cenario") > 0 then
				input #1, cenario
				
			elseif instr(header, "aclive") > 0 then
				input #1, aclive
				
			elseif instr(header, "control") > 0 then
				if cp_cont < 61 then cp_cont = cp_cont + 1
				input #1, cp(cp_cont).x, cp(cp_cont).y, cp(cp_cont).z
							
			elseif instr(header, "vertex") > 0 then
				input #1, tmp.coord.x, tmp.coord.y, tmp.coord.z
				input #1, tmp.path.x, tmp.path.y, tmp.path.z
				input #1, tmp.ang
				input #1, tmp.dist
				input #1, tmp.roadWidth
				input #1, tmp.escapeWidth(0), tmp.escapeWidth(1)
				input #1, tmp.cp_id
				
				for i = 0 to 7
					input #1, tmp.quad(i).x, tmp.quad(i).y, tmp.quad(i).z
				next
				
				cont = cont + 1
				this.v(cont) = tmp
				
				print ".";
			end if
		wend
	close
	
	print
end sub

sub vertexList.dump (arq as string)
	dim as integer i, j, f
	
	open arq for output as #1
		print #1, "[seed]"
		print #1, seed
		
		print #1, "[cenario]"
		print #1, cenario
		
		print #1, "[aclive]"
		print #1, aclive
		
		for i = 1 to cp_cont
			print #1, "[control" + str(i) + "]"
			print #1, cp(i).x, cp(i).y, 0	
		next i
	
		for i = 0 to cont
			print #1, "[vertex" + str(i) + "]" 
			print #1, v(i).coord.x, v(i).coord.y, 0
			print #1, v(i).path.x, v(i).path.y, 0
			print #1, v(i).ang
			print #1, v(i).dist
			print #1, v(i).roadWidth
			print #1, v(i).escapeWidth(0), v(i).escapeWidth(1)
			print #1, v(i).cp_id
			
			for j = 0 to 7
				print #1, v(i).quad(j).x, v(i).quad(j).y, 0
			next j
		next i
	close #1
end sub

function vertexList.getVertex (i as integer) as roadVertex
	if i < 0 then
		return v(cont + (i + 1))	
	elseif i <= cont then
		return v(i)
	else
		return v(i - (cont + 1))
	end if
end function

sub vertexList.add (v as roadVertex)
	dim as roadVertex reant, ant, atual, prox
	
	if cont < (MAX_VERTICES - 1) then
		'incrementa contador
		cont = cont + 1
		
		'insere vrtice		
		this.v(cont) = v
		
		'distncias
		if cont > 0 then
			ant = getVertex(cont - 1)
			prox = getVertex(0)
			
			this.v(cont - 1).dist = dist2p(ant.coord, v.coord) / 20
			this.v(cont).dist = dist2p(prox.coord, v.coord) / 20
		end if
		
		'ngulos e vrtices extras com os quais o editor no precisa se preocupar!
		if cont > 1 then
			'calcula ngulo do vrtice anterior
			ant = getVertex(cont - 1)
			reant = getVertex(cont - 2)
			
			ant.ang = ang2p(ant.coord, v.coord)
			reant.ang = ang2p(reant.coord, ant.coord)

			this.v(cont - 1).ang = angmed(ant.ang, reant.ang)
				
			'recalcula ngulo do prximo vrtice (primeiro da lista)
			atual = getVertex(0)
			prox = getVertex(1)
			
			atual.ang = ang2p(v.coord, atual.coord)
			prox.ang = ang2p(atual.coord, prox.coord)
			
			this.v(0).ang = angmed(atual.ang, prox.ang)
			
			'de lambuja, temos o ngulo do vrtice atual (ser atualizado se no for o ltimo)
			this.v(cont).ang = angmed(atual.ang, ant.ang)
			
			'calcular vrtices da malha
			this.v(cont - 1).genQuads
			this.v(cont).genQuads			
			this.v(0).genQuads			
		end if
	end if
end sub

sub vertexList.atualiza (i as integer, v as roadVertex)
	dim j as integer
	dim tmp as vertexList
	tmp.nova
	
	for j = 0 to cont
		tmp.add this.v(j)
	next
	
	nova
	
	for j = 0 to tmp.cont
		if i = j then
			add v
		else
			add tmp.getVertex(j)
		end if
	next
end sub

sub vertexList.atualizaUltimo 
	if cont >= 0 then
		v(cont).dist = dist2p(v(cont).coord, v(0).coord) / 20
		v(cont).genQuads
	end if
end sub

sub vertexList.removeUltimo
	if cont >= 0 then cont = cont - 1
end sub

sub vertexList.centraliza
	dim as integer i, j
	dim as coord3d min, max, delta
	
	'inicializa contadores
	min.nova
	max.nova
	min.x = 0
	max.x = 0
	min.y = 0
	max.y = 0
	
	'calcula mnimos e mximos
	dim as roadVertex atual
	
	for i = 0 to cont
		atual = getVertex(i)
		
		if atual.coord.x < min.x then min.x = atual.coord.x
		if atual.coord.y < min.y then min.y = atual.coord.y
		if atual.coord.x > max.x then max.x = atual.coord.x
		if atual.coord.y > max.y then max.y = atual.coord.y
	next
	
	'calcula delta
	delta.x = int((abs(min.x) - max.x) / 2)
	delta.y = int((abs(min.y) - max.y) / 2)
	
	'reposiciona vrtices
	for i = 0 to cont
		v(i).coord.x = v(i).coord.x + delta.x
		v(i).coord.y = v(i).coord.y + delta.y
		v(i).path.x = v(i).path.x + delta.x
		v(i).path.y = v(i).path.y + delta.y
	
		for j = 0 to 7
			v(i).quad(j).x = v(i).quad(j).x + delta.x
			v(i).quad(j).y = v(i).quad(j).y + delta.y
		next
	next
	
	'reposiciona pontos de controle
	for i = 1 to cp_cont
		cp(i).x = cp(i).x + delta.x
		cp(i).y = cp(i).y + delta.y
	next i 
end sub

function vertexList.melhorCaminho (inicio as integer, i as integer, byval atual as distCoord) as distCoord
	dim as distCoord candidato (1 to 5), temp (1 to 5)
	dim as integer j, s, f
	
	if i > (cont + 1) or i > (inicio + 6) then
		return atual
	else
		'analisa prximo vrtice
		dim as roadVertex prox
		prox = getVertex(i + 1)
		
		'gera candidatos
		candidato(1).coord.x = (prox.coord.x * 0.3) + (prox.quad(0).x * 0.7)
		candidato(1).coord.y = (prox.coord.y * 0.3) + (prox.quad(0).y * 0.7)
		
		candidato(2).coord.x = (prox.coord.x * 0.6) + (prox.quad(0).x * 0.4)
		candidato(2).coord.y = (prox.coord.y * 0.6) + (prox.quad(0).y * 0.4)
		
		candidato(3).coord = prox.coord
		
		candidato(4).coord.x = (prox.coord.x * 0.6) + (prox.quad(1).x * 0.4)
		candidato(4).coord.y = (prox.coord.y * 0.6) + (prox.quad(1).y * 0.4)
		
		candidato(5).coord.x = (prox.coord.x * 0.3) + (prox.quad(1).x * 0.7)
		candidato(5).coord.y = (prox.coord.y * 0.3) + (prox.quad(1).y * 0.7)
		
		'calcula distncias
		dim as integer menor = 2
		
		if prox.roadWidth < 14 then
			s = 1
			f = 5
		else
			s = 2 'no busca os extremos em pistas muito largas
			f = 4
		end if
		
		for j = s to f
			candidato(j).dist = atual.dist + dist2p(atual.coord, candidato(j).coord) 'dist2p
			
			'recurso
			temp(j) = melhorCaminho(inicio, i + 1, candidato(j))
			
			if temp(j).dist < temp(menor).dist then menor = j
		next

		return candidato(menor)
	end if
end function

sub vertexList.pathfinding
	dim atual as distCoord
	dim as integer i, pass

	'comeamos traando uma linha do centro da largada
	atual.coord = v(0).coord
	atual.dist = 0
	
	'o segundo passo no  absolutamente necessrio,
	'mas resolve problemas com a linha ideal na reta dos boxes
	for pass = 0 to 1
		for i = 0 to cont		
			atual = melhorCaminho(i, i, atual)		
			
			if i = cont then
				v(0).path = atual.coord
			else
				v(i + 1).path = atual.coord
			end if
			
			'atualiza grficos do editor
			pintaEditor
		next i
	next pass
end sub

function vertexList.encontra (q as coord3d) as integer
	dim as integer i	
	
	'testa coliso para cada vrtice
	for i = 0 to cont
		if dentro(q, i, false) then return i
	next
	
	'no est dentro
	return -1
end function

function vertexList.dentro (q as coord3d, i as integer, asfalto as _bool_) as integer
	dim as roadVertex atual, prox
		
	atual = getVertex(i)
	prox = getVertex(i + 1)
		
	'cria polgono
	if asfalto then
		return dentroQuad(q, atual.quad(0), atual.quad(1), prox.quad(1), prox.quad(0))
	else
		return dentroQuad(q, atual.quad(2), atual.quad(3), prox.quad(3), prox.quad(2))
	end if	
end function

function vertexList.colide (b as carBoundaries) as integer
	dim i as integer
	
	'coliso lenta
	for i = 0 to 3
		if encontra(b.coord(i)) < 0 then return true
	next
	
	return false	
end function

function vertexList.somaAngulos () as double
	dim i as integer
	dim as double soma, s1, s2
	dim as roadVertex atual, prox
	
	soma = 0	
	
	for i = 0 to cont
		atual = getVertex(i)
		prox = getVertex(i + 1)
		
		s1 = atual.ang - prox.ang
		s2 = ((2 * pi) - atual.ang) - prox.ang
		
		if abs(s1) < abs(s2) then
			soma = soma + s1
		else
			soma = soma + s2
		end if
	next
	
	return soma
end function

function vertexList.getRetaOposta () as integer
	dim as integer i, j, menor
	dim as double dist, dist_menor
	
	menor = 2
	dist_menor = 1000
	
	for i = 2 to cont - 7
		dist = 0
	
		for j = i to i + 5
			dim as roadVertex atual, prox
			atual = getVertex(i)
			prox = getVertex(i + 1)
			
			dist = dist + abs(atual.ang - prox.ang)
		next j
		
		if dist < dist_menor then menor = i: dist_menor = dist
	next i
	
	return menor + 2
end function
