'#####################################################################
'# This program is free software; you can redistribute it and/or     #
'# modify it under the terms of the GNU General Public License       #
'# as published by the Free Software Foundation; either version 3    #
'# of the License, or (at your option) any later version.            #
'#                                                                   #
'# This program is distributed in the hope that it will be useful,   #
'# but WITHOUT ANY WARRANTY; without even the implied warranty of    #
'# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     #
'# GNU General Public License for more details.                      #
'#                                                                   #
'# You should have received a copy of the GNU General Public License #
'# along with this program; if not, write to the Free Software       #
'# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,        #
'# MA  02110-1301, USA.                                              #
'#####################################################################

#include "SDL/SDL.bi"
#include "GL/gl.bi"
#include "GL/glext.bi"
#include "GL/glu.bi"
#include "AL/al.bi"
#include "AL/alut.bi"
#include "vorbis/codec.bi"
#include "vorbis/vorbisenc.bi"
#include "vorbis/vorbisfile.bi"
#include "crt/stdio.bi"

#include "bi/util.bi"
#include "bi/configs.bi"
#include "bi/palete.bi"
#include "bi/trackformat.bi"
#include "bi/textura.bi"
#include "bi/som.bi"
#include "bi/framerate.bi"
#include "bi/controle_g27.bi"
#include "bi/modelo.bi"
#include "bi/editor.bi"

#define _notex_ GLBindTexture GL_TEXTURE_2D, 0

const as string VERSAO = "Beta 0.2"

'declara objetos
type cronometro
	private:
		tempo (0 to 127) as longint	'ms
		melhor as integer
		last_vertex as integer
		
		declare function formataTempo (t as longint) as string
		
	public:			
		volta as integer			'pblico apenas para comparao com atras() - e resultados finais
		cont as integer				'para contar se fez uma volta completa antes de
									'cruzar a linha. ainda permite cortar caminho,
									'o trabalho de impedir  da coliso
		
		declare sub novo
		declare sub pintaHud
		declare function atras (cron as cronometro) as _bool_
		declare function atualiza (vertex as integer) as _bool_ 'retorna true se acaba a corrida
		declare function getTotal as string
end type

type sprite3d
	public:
		coord as coord3d
		ang as double
end type

type sprite2d
	public:
		coord as coord3d
		id as integer
		tam as double			'coordenadas do mundo
		visivel as _bool_		'true/false
		vencimento as double	'ms | usar -1 para infinito (permanente)
		dist as double			'usado apenas por spriteList.ordena(), no precisa inicializar
end type

type spriteList
	private:
		spr (0 to MAX_SPRITES - 1) as sprite2d
		cont as integer
		
		declare sub ordena
		
	public:	
		declare sub nova
		declare sub add (spr as sprite2d)
		declare sub pinta
end type

type carInfo
	public:
		nome as string
		mdl as modelo
		motor as somBuffer
		acc (1 to 4) as double
		inercia (1 to 4) as double
		freio as double
		grip as double
		max_vel as double
		max_ang as double
		max_dano as double
		
		declare sub cria (id as string)
		declare sub destroi
end type

type carro
	private:
		info as carInfo ptr
		
		marcha as integer
		rpm as double
		rpmvis as double
		acc as double
		brk as double
		clutch as double
		clutch_timer as double
		angacc as double
		angaccvis as double
		drift as double
		grip as double
		anim as double
	
	public:
		piloto as string
		cor as integer
		autogear as _bool_
		is_ai as _bool_
		ai_hp as single				'fora dos carros AI | 0-1
		
		dano as double
		vel as double		
		spr as sprite3d				'coordenada e ngulo que o carro aponta
		ang as double				'ngulo do vetor acelerao
		bounds as carBoundaries		'limites do carro para coliso, atualizados em fisica()
		vertex as integer			'ndice do vrtice atual em vertex list
		cron_id as integer			'pseudo-ponteiro para tempos em raceControl
		
		evt_chegada as _bool_
		evt_dnf as _bool_
		evt_grama as _bool_
		evt_colisao as _bool_
		evt_derrapagem as _bool_
		evt_cambio as _bool_
		evt_arranha as _bool_
		
		declare sub novo (info as carInfo ptr)
		
		declare sub controle
		declare sub AI
		declare sub atualizaBoundaries
		declare sub fisica
		declare sub atualizaSons
		declare sub chassisRollTransform (sentido as integer) '1 = normal | -1 = inverso (alinhar cmera)
		declare sub pinta (sombra as _bool_)
		declare sub pinta
		declare sub pintaHud
		declare function getInfo () as carInfo
end type

type carList
	private:
		car (0 to MAX_CARROS - 1) as carro
		cont as integer
		viewing as integer
		largada as double
						
		declare sub ordena
		declare sub colisao (byref car1 as carro, byref car2 as carro)
		declare sub pintaMapa
		
	public:	
		declare sub novo
		
		declare sub add (info as carInfo ptr, piloto as string, cor as integer, autogear as _bool_, is_ai as _bool_, ai_hp as single)
		declare sub add (info as carInfo ptr, piloto as string, cor as integer, autogear as _bool_) 	'overload jogador
		declare sub add (info as carInfo ptr, piloto as string, ai_hp as single)						'overload AI
		declare sub finaliza
		declare sub atualiza
		declare sub atualizaReplay
		declare sub pinta
		declare sub pintaHud
		declare sub viewUp
		declare sub viewDown
		declare function colide (byref car1 as carro, byref car2 as carro) as _bool_
		declare function raycasting (byref car1 as carro, byref car2 as carro, ang as double, dist as integer) as _bool_
																									'true para caminho livre
		declare function getCarro (posicao as integer) as carro
end type

type replayData
	public:
		estado as carList
		fps as double
end type

type replayManager
	private:
		buf (0 to MAX_REPLAY_BUFFERS - 1) as replayData
		cabecote as integer
		cheio as _bool_
		
	public:
		declare sub novo
		declare sub atualizaCabecote
		declare sub grava (estado as carList)
		declare sub toca
end type

type raceControl
	public:
		cron (0 to MAX_CARROS - 1) as cronometro
	
		voltas as integer
		evt_largada as _bool_
		evt_wait_for_others as _bool_
		evt_fim as _bool_
				
		declare sub nova (voltas as integer)
end type

type track
	private:
		const as integer NUM_ARVORES = 384
		const as integer NUM_FISCAIS = 10
									  '---somados, devem ser < MAX_SPRITES (com espao para partculas tambm)
		const as integer NUM_VEICULOS = 8
		const as integer HEIGHTMAP_RES = 64
	
		sl as spriteList
		veiculo (0 to NUM_VEICULOS - 1) as carro
		v_billboard (0 to 2) as coord3d
		v_billboard2 (0 to 3) as coord3d
		
		heightmap (0 to HEIGHTMAP_RES - 1, 0 to HEIGHTMAP_RES - 1) as coord3d
		normal (0 to HEIGHTMAP_RES - 1, 0 to HEIGHTMAP_RES - 1) as coord3d
		aclive as double
		
		declare sub geraHeightmap
		declare sub pintaHeightmap
		declare function espacoVazio (c as coord3d, delta as double) as integer
		declare sub plantaArvores
		declare sub plantaFiscais
		declare sub plantaVeiculos		

	public:	
		vl as vertexList
		cor_grama as rgbval
			
		const as double MURO_H = 1.7
		
		declare sub inicia
		declare sub pintaPoste (coord as coord3d, bottom as double, top as double)
		declare sub pintaObjetos
		declare sub pintaPista
		declare sub criaParticula (coord as coord3d, id as integer)
		declare function getBalao () as coord3d
end type

type camera
	private:
		const as integer MAX_CONFIGS = 7
		
		config as integer
		car as carro
		
		prev_config as integer
		transform as double
		quick_look as double
		
		declare function getEye (i as integer) as coord3d
		declare function getLook (i as integer) as coord3d
		
	public:
		eye as coord3d								'ltima posio da cmera
		look as coord3d								'ltimo foco da cmera
		tv as _bool_
		montada as _bool_
		espelhada as _bool_
		zoom as double
	
		declare sub nova
		declare sub orto							'configura projeo 2D
		declare sub restaura						'retorna para projeo 3D
		declare sub setCarro (car as carro)			'define carro a seguir
		declare sub atualiza (teste as _bool_)		'define a matriz de projeo principal
		declare sub atualiza
		declare sub espelho							'configura projeo do espelho
		declare sub controle
		declare sub up								'altera configuraes
		declare sub down
end type

type environment
	private:
		hora as double		'0-24
		lapso as double		'horas/quadro
	
		cor_amb as rgbval
		cor_hor as rgbval

		estrela_alpha as double	

	public:		
		cor_clr as rgbval
				
		estrelas_on as _bool_
				
		bg as textura
		estrelas as textura

		declare sub pintaSkydome
		declare sub setCor (r as double, g as double, b as double)
		declare sub setCor (r as double, g as double, b as double, hor as integer)
		declare sub setCor (cor as rgbval)
		declare sub setHora (hora as double, lapso as double)
		declare function getAng as double 'para o relgio
		declare sub atualiza
end type

type graficos
	public:
		asfalto as textura
		faixa as textura
		grama as textura
		muro as textura
		billboard as textura
		relogio as textura
		arvores as textura
		hud as textura
		spritegfx (0 to MAX_SPRITEGFX - 1) as textura
		sombra as textura
		roda as textura
		pneu as textura
		capacete as textura
		reflexo_dia as textura
		reflexo_noite as textura
		xadrez as textura
end type

type menuContainer
	private:
		tracks as fileList
		escolha as carro
		modo_corrida as integer
		timeframe as integer
		anim as double
		jukebox as integer
		
		declare sub pintaFundo (texto as string)
		declare sub inicioLoop (intervalo_poll as double)
		declare sub inicioLoop ()
		declare sub fimLoop
		declare sub escolheModo
		declare sub escolheCarro
		declare sub escolhePista
		declare sub configCorrida
		declare sub configEditor
		declare sub pintaResultados
		declare sub pintaAjuda
		
	public:
		logo as textura
		bg as textura
		ajuda as textura
		mus as musica
	
		declare sub inicio
		declare sub pintaLoading (progresso as integer)
end type

'declara rotinas
declare sub inicio
declare sub carregaJogo
declare sub fim

declare sub jogo
declare sub pintaCena
declare sub pintaFPS
declare sub pausa

'instancia objetos
dim shared fnt as fonte
dim shared as somBuffer derrapagem, grama, colisao, cambio, ignicao
dim shared som as somEngine
dim shared trk as track
dim shared cam as camera
dim shared env as environment
dim shared gfx as graficos
dim shared as carInfo gol, crane, van, balao, casa
dim shared grid as carList
dim shared replay as replayManager
dim shared race as raceControl
dim shared menu as menuContainer

'chama rotinas
print "Tamanho do track: " + str(sizeof(trk) \ 1024) + " KB"
print "Tamanho do buffer de replay: " + str(sizeof(replay) \ 1024) + " KB"
print "Tamanho do carro: " + str(sizeof(carro)) + " B"
print "Tamanho do grid: " + str(sizeof(grid)) + " B"

carregaCfg
inicio
carregaJogo
menu.inicio

'***
sub inicio
	'gera tabela de senos usada por modelos
	tabs.gera
	
	'inicia bibliotecas
	alutInit 0, 0
	if SDL_Init(SDL_INIT_EVERYTHING) <> 0 then end

	'inicia uma janela
	dim video as SDL_Surface ptr
	
	if FULLSCREEN then
		video = SDL_SetVideoMode(w, h, 16, SDL_OPENGL or SDL_DOUBLEBUF or SDL_FULLSCREEN)
	else
		video = SDL_SetVideoMode(w, h, 16, SDL_OPENGL or SDL_DOUBLEBUF)
	end if
	
	if video = 0 then SDL_Quit: end
	
	dim as string titulo_utf8 = VERSAO
	SDL_WM_SetCaption titulo_utf8, titulo_utf8
	SDL_ShowCursor SDL_DISABLE

	'inicia controle de framerate
	fps.reinicia
	fps.setMaxFPS MAX_FPS
	
	'inicia ambiente
	env.setHora 12, 0
	
	'configura visual
	glEnable GL_BLEND
	glBlendFunc GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA
	glShadeModel GL_SMOOTH
	glEnable GL_DEPTH_TEST
	glDisable GL_CULL_FACE
	glDisable GL_POLYGON_SMOOTH
	glDisable GL_POINT_SMOOTH
	glDisable GL_LINE_SMOOTH
	glHint GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST
	glHint GL_FOG_HINT, GL_NICEST
	
	if WIREFRAME_ON then
		glDisable GL_TEXTURE_2D
		glPolygonMode GL_FRONT_AND_BACK, GL_LINE
	else
		glEnable GL_TEXTURE_2D	
		glPolygonMode GL_FRONT_AND_BACK, GL_FILL
	end if
	
	dim l_amb(0 to 3) as single => {0.3, 0.3, 0.3, 1} 
	dim l_diff(0 to 3) as single => {0.5, 0.5, 0.5, 1}
	dim l_spec(0 to 3) as single => {0, 0, 0, 1}
	
	glLightfv GL_LIGHT0, GL_AMBIENT, @l_amb(0)
	glLightfv GL_LIGHT0, GL_DIFFUSE, @l_diff(0)
	glLightfv GL_LIGHT0, GL_SPECULAR, @l_spec(0)
	glEnable GL_LIGHT0 
	glEnable GL_NORMALIZE
	glEnable GL_COLOR_MATERIAL
	glDisable GL_LIGHTING
	
	'configura udio
	alDistanceModel AL_INVERSE_DISTANCE_CLAMPED
end sub

'***
sub carregaJogo
	dim i as integer

	print "Carregando graficos..."

	'carrega fonte
	fnt.carrega "gfx/fonte.bmp"

	'carrega grficos [filtering,] [mipmapping, clamping]
	env.bg.carrega "gfx/bg.bmp", false, false
	env.estrelas.carrega "gfx/estrelas.bmp", false, false
	menu.logo.carrega "gfx/logo.bmp", false, true
	menu.bg.carrega "gfx/menu.bmp", false, true
	menu.ajuda.carrega "gfx/ajuda.bmp", false, false, true
	gfx.asfalto.carrega "gfx/asfalto.bmp", true, false
	gfx.faixa.carrega "gfx/faixa.bmp", true, false
	gfx.grama.carrega "gfx/grama.bmp", true, false
	gfx.muro.carrega "gfx/muro.bmp", true, true
	gfx.billboard.carrega "gfx/billboard.bmp", true, true
	gfx.relogio.carrega "gfx/relogio.bmp", false, true
	gfx.arvores.carrega "gfx/arvores.bmp", true, true
	gfx.hud.carrega "gfx/hud.bmp", false, true
	gfx.sombra.carrega "gfx/sombra.bmp"
	gfx.roda.carrega "gfx/roda.bmp", true, true
	gfx.pneu.carrega "gfx/pneu.bmp", true, false
	gfx.capacete.carrega "gfx/capacete.bmp", true, true
	gfx.reflexo_dia.carrega "gfx/reflexo_dia.bmp", false, false
	gfx.reflexo_noite.carrega "gfx/reflexo_noite.bmp", false, false
	gfx.xadrez.carrega "gfx/xadrez.bmp", false, false, false
	
	for i = 0 to MAX_SPRITEGFX - 1
		gfx.spritegfx(i).carrega "gfx/sprite" + str(i) + ".bmp", true, true
	next
			
	'carrega sons globais
	derrapagem.carrega "sfx/derrapagem.wav"
	grama.carrega "sfx/grama.wav"
	colisao.carrega "sfx/colisao.wav"
	cambio.carrega "sfx/cambio.wav"
	ignicao.carrega "sfx/ignicao.wav"
	
	'define carros
	print "Gerando modelos";
	
	gol.cria "gol"
	crane.cria "crane"
	van.cria "van"
	
	'outras decoraes
	balao.cria "balao"
	casa.cria "casa"
	
	print
	
	'inicia engine sonora
	som.cria
	som.defineVoz som.getPlayer, derrapagem
	som.defineVoz som.getPlayer + 1, grama
end sub

'***
sub fim
	dim i as integer

	'libera memria
	for i = 0 to MAX_SPRITEGFX - 1
		gfx.spritegfx(i).destroi
	next
	
	gol.destroi
	crane.destroi
	van.destroi
	balao.destroi
	casa.destroi
	
	fnt.destroi
	env.bg.destroi
	env.estrelas.destroi
	menu.logo.destroi
	menu.bg.destroi
	menu.ajuda.destroi
	menu.mus.destroi
	
	gfx.asfalto.destroi
	gfx.faixa.destroi
	gfx.grama.destroi
	gfx.muro.destroi
	gfx.billboard.destroi
	gfx.relogio.destroi
	gfx.arvores.destroi
	gfx.hud.destroi
	gfx.sombra.destroi
	gfx.roda.destroi
	gfx.pneu.destroi
	gfx.capacete.destroi
	gfx.reflexo_dia.destroi
	gfx.reflexo_noite.destroi
	gfx.xadrez.destroi
	
	derrapagem.destroi
	grama.destroi
	colisao.destroi
	cambio.destroi
	ignicao.destroi
	som.destroi
	
	'encerra bibliotecas
	SDL_Quit
	alutExit
	
	'tchau!
	end
end sub

'***
sub jogo
	som.toca ignicao, trk.vl.getVertex(0).coord, 1
		
	do
		fps.atualiza
		kev.atualiza
	
		'controles
		cam.controle
		
		if kev.confereTecla(kev.K_CARUP, kev.PRESSED) then grid.viewUp
		if kev.confereTecla(kev.K_CARDOWN, kev.PRESSED) then grid.viewDown
		if kev.confereTecla(kev.K_HUD, kev.PRESSED) then HUD_ON = not HUD_ON
		if kev.confereTecla(kev.K_LOD, kev.PRESSED) then LOD = LOD - 1: if LOD < 0 then LOD = 3
		if kev.confereTecla(kev.K_MIRROR, kev.PRESSED) then MIRROR_ON = not MIRROR_ON
		if kev.confereTecla(kev.K_RETORNA, kev.PRESSED) then exit do
		if kev.confereTecla(kev.K_PAUSA, kev.PRESSED) then pausa
		if kev.confereTecla(kev.K_REPLAY, kev.PRESSED) then replay.toca
				
		'carros
		grid.atualiza
		
		'tempo
		env.atualiza
		
		'faz grficos
		cam.atualiza
		pintaCena
				
		'hud
		cam.orto
				
		if HUD_ON then
			pintaFPS		
			grid.pintaHud		
		end if	
					
		'espelho
		if MIRROR_ON and not cam.tv then	
			cam.espelho
			
			if LOD < 2 then
				grid.pinta
			else
				pintaCena
			end if
		else
			cam.restaura
		end if
				
		'atualiza tela
		SDL_GL_SwapBuffers
		
		'intervalo
		fps.preciseSleep	
	loop until race.evt_fim
	
	som.para
end sub

'***
sub pintaCena
	'reposiciona luz
	dim l_pos(0 to 3) as single => {2048, 0, 2000, 1}
	glLightfv GL_LIGHT0, GL_POSITION, @l_pos(0)

	env.pintaSkydome
	trk.pintaPista
	grid.pinta
	trk.pintaObjetos
end sub

'***

sub pintaFPS
	'fonte tamanho mnimo
	dim as integer _dx_temp = _dx_
	dim as integer _dy_temp = _dy_
	_dx_ = 6
	_dy_ = 9
	
	'texto
	fnt.setCor 1, 1, 1
	fnt.setSombra false
	fnt.pintaTopoDir fps.getFps, 1
	
	'restaura
	_dx_ = _dx_temp
	_dy_ = _dy_temp
end sub

'***
sub pausa
	som.para

	do
		fps.atualiza
		kev.atualiza
				
		sleep 20
	loop until kev.confereTecla(kev.K_RETORNA, kev.PRESSED) or kev.confereTecla(kev.K_PAUSA, kev.PRESSED)
	
	som.startYourEngines
end sub

'------------------------------------------------------------------------------------
sub replayManager.novo
	dim as integer i, j

	cabecote = 0
	cheio = false
end sub

sub replayManager.atualizaCabecote
	'move cabeote
	cabecote = cabecote + 1
	
	if cabecote >= MAX_REPLAY_BUFFERS then
		cabecote = 0
		cheio = true
	end if
	
	'escreve cabealho
	buf(cabecote).fps = fps.fps
end sub

sub replayManager.grava (estado as carList)
	buf(cabecote).estado = estado
end sub

sub replayManager.toca
	'o buffer de replay est vazio?
	if not cheio then exit sub
	
	'ok...
	dim as integer i, pos_atual, max_fps
	
	'guarda estados
	dim temp_grid as carList
	dim temp_cam as camera
	temp_grid = grid 
	temp_cam = cam
	
	'inicializa
	pos_atual = cabecote
	max_fps = fps.getMaxFPS
	cam.nova
	cam.down
	
	do
		fps.atualiza
		kev.atualiza
		
		'controles
		cam.controle
		
		if kev.confereTecla(kev.K_PAUSA, kev.PRESSED) then pausa
		if kev.confereTecla(kev.K_RETORNA, kev.PRESSED) then exit do
		
		'passa dados de replay para carList
		grid = buf(pos_atual).estado
		grid.atualizaReplay
		
		'grficos
		cam.atualiza
		pintaCena
		
		'hud
		cam.orto
		
		'marca d'gua
		dim tam as double
		tam = h * 0.04
		 
		glPushMatrix
		glTranslatef w - tam - borda_hud, h - tam - borda_hud, 0
		
		glColor4f 1, 1, 1, 0.5
		gfx.hud.aplica
		glBegin GL_QUADS
			glTexCoord2f 0.5, 0.5
			glVertex2f 0, 0
			glTexCoord2f 0.5, 1
			glVertex2f 0, tam
			glTexCoord2f 1, 1
			glVertex2f tam, tam
			glTexCoord2f 1, 0.5
			glVertex2f tam, 0
		glEnd
		
		glPopMatrix
		
		'fita
		tam = h * 0.02
		
		_notex_
		glColor4f 0, 0, 0, 0.2
		glBegin GL_QUADS
			glVertex2f borda_hud, borda_hud
			glVertex2f borda_hud, borda_hud + tam
			glVertex2f w - borda_hud, borda_hud + tam
			glVertex2f w - borda_hud, borda_hud
		glEnd
		
		'cabeote
		dim pos_cabecote as integer
		pos_cabecote = wrap(pos_atual - cabecote, 0, MAX_REPLAY_BUFFERS)
		
		glPushMatrix
		glTranslatef borda_hud + ((pos_cabecote / MAX_REPLAY_BUFFERS) * (w - (borda_hud * 2))), borda_hud, 0
		
		glColor4f 1, 1, 1, 1
		glBegin GL_QUADS
			glVertex2f -1, 0
			glVertex2f -1, tam
			glVertex2f 1, tam
			glVertex2f 1, 0
		glEnd
		glPopMatrix
		
		cam.restaura
		
		'atualiza tela
		SDL_GL_SwapBuffers
		
		'ajusta a taxa de playback
		fps.setMaxFPS buf(pos_atual).fps
		fps.preciseSleep
		
		'incrementa posio
		pos_atual = pos_atual + 1
		if pos_atual >= MAX_REPLAY_BUFFERS then pos_atual = 0
	loop
	
	'restaura objetos
	grid = temp_grid
	cam = temp_cam
	fps.setMaxFPS max_fps
	
	'deixa pausado
	cam.atualiza
	pintaCena
	
	cam.orto
	fnt.setCor cor_texto
	fnt.setSombra true
	fnt.pintaCentro "paused"
	cam.restaura
	
	SDL_GL_SwapBuffers
	
	pausa
end sub

'------------------------------------------------------------------------------------
sub raceControl.nova (voltas as integer)
	dim i as integer

	'informaes da corrida
	this.voltas = voltas
	this.evt_largada = false
	this.evt_wait_for_others = false
	this.evt_fim = false
	
	'zera tabela de cronmetros
	for i = 0 to MAX_CARROS - 1
		cron(i).novo
	next
end sub

'------------------------------------------------------------------------------------
sub environment.pintaSkydome
	if cam.espelhada or LOD <= 1 then exit sub

	'calcula coordenadas de textura
	dim as double tx, tw, th
	tx = (ang2p(cam.eye, cam.look) * -1) / pi
	tw = (w / h) * 0.4	'deve ser 0.4-0.5
	th = 0.8			'o dobro desse valor
	
	'zoom
	tx = tx - (tw * (cam.zoom / FOV) * 0.5)
	tw = tw * (cam.zoom / FOV)
	th = th * (cam.zoom / FOV)

	'prepara projeo ortogrfica
	cam.orto

	'nuvens
	if estrela_alpha < 1 then
		setCor 1, 1, 1, true
		bg.aplica
		
		glBegin GL_QUADS
			glTexCoord2f tx, 1 - th
			glVertex2f 0, 0
			glTexCoord2f tx, 1
			glVertex2f 0, h / 2
			glTexCoord2f tx + tw, 1
			glVertex2f w, h / 2
			glTexCoord2f tx + tw, 1 - th
			glVertex2f w, 0
		glEnd
	end if
	
	'noite estrelada
	if estrelas_on then
		tx = tx * 2
		tw = tw * 2
		th = th * 1.4
	
		glColor4f 1, 1, 1, estrela_alpha
		estrelas.aplica
		
		glBegin GL_QUADS
			glTexCoord2f tx, 1 - th
			glVertex2f 0, 0
			glTexCoord2f tx, 1
			glVertex2f 0, h / 2
			glTexCoord2f tx + tw, 1
			glVertex2f w, h / 2
			glTexCoord2f tx + tw, 1 - th
			glVertex2f w, 0
		glEnd
	end if

	'retorna
	cam.restaura
end sub

sub environment.setCor (r as double, g as double, b as double)
	r = r * cor_amb.r
	g = g * cor_amb.g
	b = b * cor_amb.b
	glColor3f r, g, b
end sub

sub environment.setCor (r as double, g as double, b as double, hor as integer)
	r = r * cor_hor.r
	g = g * cor_hor.g
	b = b * cor_hor.b
	glColor3f r, g, b
end sub

sub environment.setCor (cor as rgbval)
	setCor cor.r, cor.g, cor.b
end sub

sub environment.setHora (hora as double, lapso as double)
	this.hora = wrap(hora, 0, 24)
	this.lapso = lapso
end sub

function environment.getAng () as double
	return wrap(hora * 30, 0, 360)
end function

sub environment.atualiza
	dim as double d, e
	
	'passa tempo
	hora = wrap(hora + (lapso * fps.getFator), 0, 24)
	
	'define parmetros	
	select case hora
		case 21 to 24, 0 to 3
			cor_amb.r = 0.2
			cor_amb.g = 0.3
			cor_amb.b = 0.5
			
			cor_hor.r = 0
			cor_hor.g = 0
			cor_hor.b = 0
						
			estrela_alpha = 1
			estrelas_on = true
			
		case 3 to 6
			d = (hora - 3) / 3
			e = 1 - d
				
			cor_amb.r = (0.8 * d) + (0.2 * e)
			cor_amb.g = (0.6 * d) + (0.3 * e)
			cor_amb.b = (0.4 * d) + (0.5 * e)
			
			cor_hor.r = (0.8 * d) + (0 * e)
			cor_hor.g = (0.6 * d) + (0 * e)
			cor_hor.b = (0.4 * d) + (0 * e)
			
			estrela_alpha = e
			estrelas_on = true
	
		case 6 to 9
			d = (hora - 6) / 3
			e = 1 - d
			
			cor_amb.r = (1 * d) + (0.8 * e)
			cor_amb.g = (1 * d) + (0.6 * e)
			cor_amb.b = (1 * d) + (0.4 * e)
			
			cor_hor = cor_amb
			
			estrela_alpha = 0
			estrelas_on = false
			
		case 9 to 15
			cor_amb.r = 1
			cor_amb.g = 1
			cor_amb.b = 1
			
			cor_hor = cor_amb		
			
			estrela_alpha = 0
			estrelas_on = false
		
		case 15 to 18
			d = (hora - 15) / 3
			e = 1 - d
			
			cor_amb.r = (1 * e) + (0.8 * d)
			cor_amb.g = (1 * e) + (0.6 * d)
			cor_amb.b = (1 * e) + (0.4 * d)
			
			cor_hor = cor_amb			
			
			estrela_alpha = 0
			estrelas_on = false
		
		case 18 to 21
			d = (hora - 18) / 3
			e = 1 - d
		
			cor_amb.r = (0.8 * e) + (0.2 * d)
			cor_amb.g = (0.6 * e) + (0.3 * d)
			cor_amb.b = (0.4 * e) + (0.5 * d)
			
			cor_hor.r = (0.8 * e) + (0 * d)
			cor_hor.g = (0.6 * e) + (0 * d)
			cor_hor.b = (0.4 * e) + (0 * d)
						
			estrela_alpha = d
			estrelas_on = true		
	end select
		
	cor_clr.r = (138 / 256) * cor_hor.r
	cor_clr.g = (167 / 256) * cor_hor.g
	cor_clr.b = (197 / 256) * cor_hor.b
	
	'configura openGL
	dim as single c(0 to 3)
	c(0) = cor_clr.r
	c(1) = cor_clr.g
	c(2) = cor_clr.b
	c(3) = 1
	
	d = 1.4 / FOG_DIST
	
	glFogi GL_FOG_MODE, GL_EXP2
	glFogf GL_FOG_DENSITY, d
	glFogfv GL_FOG_COLOR, @c(0)
	glEnable GL_FOG
			
	if LOD <= 1 then
		glClearColor cor_clr.r, cor_clr.g, cor_clr.b, 1	
	else
		glClearColor (130 / 256) * cor_hor.r, (164 / 256) * cor_hor.g, (201 / 256) * cor_hor.b, 1
	end if
end sub

'------------------------------------------------------------------------------------
sub triangulo.pinta
	'rpido
	dim i as integer
	
	glNormal3f normal.x, normal.y, normal.z
	
	for i = 0 to 2
		glTexCoord2f tx(i), ty(i)
		glVertex3f v(i).x, v(i).y, v(i).z
	next
end sub

sub triangulo.pinta (cor_default as rgbval)
	dim i as integer

	glNormal3f normal.x, normal.y, normal.z

	'rosa no tringulo: aplica default
	if cor.r = 1 and cor.g = 0 and cor.b = 1 then
		env.setCor cor_default
		
	'rosa no argumento: no ilumina
	elseif cor_default.r = 1 and cor_default.g = 0 and cor_default.b = 1 then
		glColor3f 1, 1, 1
	
	'normal
	else
		env.setCor cor
	end if
	
	for i = 0 to 2
		glTexCoord2f tx(i), ty(i)
		glVertex3f v(i).x, v(i).y, v(i).z
	next
end sub

'------------------------------------------------------------------------------------
sub triangleList.pinta (cor_default as rgbval)
	'lista est vazia?
	if cont < 0 then exit sub
	
	'ok...
	dim i as integer

	'configura textura
	if texturizado then
		tex.aplica
	else
		if LOD = 3 then
			'reflection mapping
			if env.estrelas_on then			'devia estar fazendo algo melhor com shaders
				gfx.reflexo_noite.aplica
			else
				gfx.reflexo_dia.aplica
			end if
			
			glTexGeni GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP
			glTexGeni GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP
			
			glEnable GL_TEXTURE_GEN_S
			glEnable GL_TEXTURE_GEN_T
			
			glTexEnvf GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_ADD
			
			'gira o reflexo com a coordenada do mundo
			glMatrixMode GL_TEXTURE
			glPushMatrix
			glScalef 1, -1, 1
			glTranslatef ang2p(cam.eye, cam.look) / (-1 * pi), 0, 0
		else
			'plano
			_notex_
		end if
	end if
	
	'pinta geometria
	glBegin GL_TRIANGLES
		for i = 0 to cont
			tri(i).pinta cor_default
		next i
	glEnd
	
	'desliga reflection mapping
	if not texturizado and LOD = 3 then
		glDisable GL_TEXTURE_GEN_S
		glDisable GL_TEXTURE_GEN_T
		
		glTexEnvf GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE
		
		glPopMatrix
		glMatrixMode GL_MODELVIEW
	end if
end sub

'------------------------------------------------------------------------------------
sub modelo.pintaRoda (anim as double)
	dim res as integer
	if LOD = 3 then
		res = 1
	else
		res = 2
	end if
	
	dim as double x, c
	dim as integer i, j
	
	glPushMatrix
	glTranslatef 0, 0, raio
	glRotatef (anim / (4 * pi)) * 360, -1, 0, 0
		
	env.setCor 1, 1, 1
	gfx.roda.aplica
	for i = 1 to 2
		x = TALA
		if i = 2 then x = x * -1
	
		glBegin GL_TRIANGLE_FAN
			glTexCoord2f 0.5, 0.5
			glVertex3f x, 0, 0
			
			for j = 0 to 16 step res
				glTexCoord2f 0.5 + (0.48 * tabs.cosseno(j)), 1 - (0.5 + (0.48 * tabs.seno(j)))
				glVertex3f x, RAIO * tabs.cosseno(j), RAIO * tabs.seno(j)
			next j
		glEnd
	next i
	
	gfx.pneu.aplica
	glBegin GL_TRIANGLE_STRIP
		for j = 0 to 16 step res
			c = 0.1 + (0.1 * tabs.seno(j))
			env.setCor c, c, c
			
			glTexCoord2f 0, j / 2
			glVertex3f TALA, RAIO * tabs.cosseno(j), RAIO * tabs.seno(j)
			glTexCoord2f 1, j / 2
			glVertex3f TALA * -1, RAIO * tabs.cosseno(j), RAIO * tabs.seno(j)
		next j
	glEnd	
	
	env.setCor 1, 1, 1
	glPopMatrix
end sub

sub modelo.pintaSombra (ang as double, coord as coord3d)
	if simples then exit sub

	_notex_
	glColor4f 0, 0, 0, 0.5
	
	glEnable GL_POLYGON_OFFSET_FILL
	glPolygonOffset -0.8, -1
	
	'mapeia sombra dinmica
	dim sol as coord3d
	sol.nova
	sol.x = 1024 'efeito mais visvel, real  2048
	
	dim ang_final as double
	ang_final = toRad(ang + 90) - (ang2p(sol, coord) + pi)
	ang_final = wrap(ang_final, 0, 2 * pi)
	
	dim as coord3d c(0 to 7)
	c(0).x = largura * -1:	c(0).y = comprimento * 0.95
	c(1).x = largura * -1:	c(1).y = comprimento * -0.95
	c(2).x = largura * 1:	c(2).y = comprimento * -0.95
	c(3).x = largura * 1:	c(3).y = comprimento * 0.95
	
	dim as integer i
	for i = 4 to 7
		c(i).x = c(i - 4).x + (0.8 * cos(ang_final))
		c(i).y = c(i - 4).y + (-0.8 * sin(ang_final))
	next
	
	'pinta na ordem certa
	glBegin GL_TRIANGLE_FAN
		glVertex3f 0, 0, 0
		
		if ang_final < (pi / 2) then
			glVertex3f c(0).x, c(0).y, 0
			glVertex3f c(1).x, c(1).y, 0
			glVertex3f c(5).x, c(5).y, 0
			glVertex3f c(6).x, c(6).y, 0
			glVertex3f c(7).x, c(7).y, 0
			glVertex3f c(3).x, c(3).y, 0
			glVertex3f c(0).x, c(0).y, 0
		elseif ang_final < pi then
			glVertex3f c(0).x, c(0).y, 0
			glVertex3f c(4).x, c(4).y, 0
			glVertex3f c(5).x, c(5).y, 0
			glVertex3f c(6).x, c(6).y, 0
			glVertex3f c(2).x, c(2).y, 0
			glVertex3f c(3).x, c(3).y, 0
			glVertex3f c(0).x, c(0).y, 0
		elseif ang_final < (pi * 1.5) then
			glVertex3f c(4).x, c(4).y, 0
			glVertex3f c(5).x, c(5).y, 0
			glVertex3f c(1).x, c(1).y, 0
			glVertex3f c(2).x, c(2).y, 0
			glVertex3f c(3).x, c(3).y, 0
			glVertex3f c(7).x, c(7).y, 0
			glVertex3f c(4).x, c(4).y, 0
		else		
			glVertex3f c(0).x, c(0).y, 0
			glVertex3f c(1).x, c(1).y, 0
			glVertex3f c(2).x, c(2).y, 0
			glVertex3f c(6).x, c(6).y, 0
			glVertex3f c(7).x, c(7).y, 0
			glVertex3f c(4).x, c(4).y, 0
			glVertex3f c(0).x, c(0).y, 0
		end if
	glEnd
	
	glDisable GL_POLYGON_OFFSET_FILL
	glColor4f 1, 1, 1, 1
end sub

sub modelo.pintaBase (ang as double, anim as double)
	if simples then exit sub

	dim as double x1 = largura - TALA
	dim as double x2 = (largura - TALA) * -1	
	dim as integer i, j
	
	'eixos
	for i = 0 to 1
		glPushMatrix
		glTranslatef 0, eixo(i), RAIO
		
		_notex_
		glBegin GL_TRIANGLE_STRIP
			for j = 0 to 16 step 4
				dim as double c = 0.2 + (0.2 * tabs.seno(j))
				env.setCor c, c, c
			
				glVertex3f x1, 0.06 * tabs.cosseno(j), 0.06 * tabs.seno(j)
				glVertex3f x2, 0.06 * tabs.cosseno(j), 0.06 * tabs.seno(j)
			next j
		glEnd
		
		glPopMatrix
	next i
	
	'rodas
	for i = 1 to 4
		glPushMatrix
		
		'posiciona
		select case i
			case 1: glTranslatef x1, eixo(0), 0
			case 2: glTranslatef x2, eixo(0), 0
			case 3: glTranslatef x1, eixo(1), 0
			case 4: glTranslatef x2, eixo(1), 0
		end select
		
		'angula
		if i >= 3 then glRotatef ang, 0, 0, 1
		
		'pinta
		pintaRoda anim
		
		glPopMatrix
	next i
	
	env.setCor 1, 1, 1
end sub

sub modelo.pintaCasco (cor_default as rgbval)
	dim as rgbval nulo, no_light
	nulo.r = 0
	nulo.g = 0
	nulo.b = 0
	no_light.r = 1
	no_light.g = 0
	no_light.b = 1

	glEnable GL_LIGHTING
	casco.pinta cor_default
	decais.pinta nulo
	farois.pinta no_light
	
	glEnable GL_POLYGON_OFFSET_FILL
	glPolygonOffset -0.2, -2
	lanternas.pinta no_light	
			
	glDisable GL_POLYGON_OFFSET_FILL
	glDisable GL_LIGHTING	
	painel.pinta nulo
end sub

sub modelo.pintaPiloto (ang as double, angvel as double, coord as coord3d)
	if simples then exit sub
		
	'piloto
	if LOD > 0 then
		dim as integer i, j
		
		dim cor as rgbval
		cor.r = 0.8
		cor.g = 0.8
		cor.b = 0.8
		
		dim luvas as rgbval
		luvas.r = 169 / 256
		luvas.g = 162 / 256
		luvas.b = 159 / 256
		
		glPushMatrix
		glTranslatef volante.x, volante.y - 1.1, RAIO
		
		'braos
		dim bracos as triangleList
		bracos.nova		
		bracos.debug = false 'no deixa gastar cpu com prints ao terminal
		
		for i = -1 to 1 step 2
			dim p (0 to 1) as coord3d
			p(0).x = 0.25 * i
			p(0).y = 0
			p(0).z = 0.75
			p(1).x = 0.35 * i * cos(toRad(ang * -1))
			p(1).y = 1.05
			p(1).z = 0.7 + (0.35 * i * sin(toRad(ang * -1)))
			
			'gera vrtices
			dim v (0 to 11) as coord3d
			const as double tam2 = 0.10
			for j = 0 to 3
				v(j).x = p(0).x + (tam2 * cos(toRad((ang * -1) + (90 * j))))
				v(j).y = p(0).y
				v(j).z = p(0).z + (tam2 * sin(toRad((ang * -1) + (90 * j))))
				
				v(8 + j).x = p(1).x + (tam2 * cos(toRad((ang * -1) + (90 * j))))
				v(8 + j).y = p(1).y
				v(8 + j).z = p(1).z + (tam2 * sin(toRad((ang * -1) + (90 * j))))
				
				v(4 + j) = interpola(v(j), v(8 + j), 0.8)
			next j
			
			'gera tringulos
			for j = 0 to 3				
				dim prox as integer
				prox = (j + 1) mod 4
				
				bracos.addTri v(j), v(4 + j), v(prox), cor
				bracos.addTri v(4 + j), v(4 + prox), v(prox), cor
				
				bracos.addTri v(4 + j), v(8 + j), v(4 + prox), luvas
				bracos.addTri v(8 + j), v(8 + prox), v(4 + prox), luvas
			next j
			
			'caps
			bracos.addTri v(0), v(1), v(2), cor
			bracos.addTri v(0), v(2), v(3), cor
			bracos.addTri v(10), v(9), v(8), luvas
			bracos.addTri v(11), v(10), v(8), luvas
		next i
		
		'tronco
		dim tronco as triangleList
		tronco.nova
		tronco.debug = false
		
		scope
			'gera vrtices
			dim v (0 to 3) as coord3d
			v(0).x = 0.24
			v(0).y = 0.16
			v(0).z = 0.8
									
			v(2).x = 0.42
			v(2).y = -0.3
			v(2).z = 0
			
			v(1) = v(0)
			v(1).y = v(1).y * -1
			
			v(3) = v(2)
			v(3).y = v(3).y * -1
			
			v(0).z = v(0).z - 0.05
			
			'gera tringulos
			tronco.add2Tri v(0), v(1), v(2), cor
			tronco.add2Tri v(0), v(2), v(3), cor
			tronco.addQuad v(3), v(0), cor
			tronco.addQuad v(0), v(1), cor 
			tronco.addQuad v(1), v(2), cor
		end scope		
		
		glEnable GL_LIGHTING
		
		bracos.pinta cor
		tronco.pinta cor
		
		glDisable GL_LIGHTING
		
		'banco
		_notex_
		env.setCor 0, 0, 0
		glBegin GL_QUADS
			glVertex3f -0.2, -0.4, 1.1
			glVertex3f -0.35, -0.3, 0
			glVertex3f 0.35, -0.3, 0
			glVertex3f 0.2, -0.4, 1.1
		glEnd
		
		if dist2p(coord, cam.eye) > 2 then
			dim tx as double
			dim res as integer
			
			res = 2
							
			'cabea
			env.setCor 1, 1, 1
			gfx.capacete.aplica
			glBegin GL_TRIANGLE_FAN
				glTexCoord2f 0.5, 0
				glVertex3f 0, 0, 1.3
				
				for i = 0 to 16 step res
					tx = edge(i / 8, 0, 1)
					
					glTexCoord2f tx, 0.25
					glVertex3f 0.2 * tabs.cosseno(i), 0.2 * tabs.seno(i), 1.22
				next i
			glEnd
			
			glBegin GL_TRIANGLE_STRIP
				for i = 0 to 16 step res
					tx = edge(i / 8, 0, 1)
				
					glTexCoord2f tx, 0.25
					glVertex3f 0.2 * tabs.cosseno(i), 0.2 * tabs.seno(i), 1.22
					glTexCoord2f tx, 0.5
					glVertex3f 0.26 * tabs.cosseno(i), 0.26 * tabs.seno(i), 1.05
				next i
			glEnd
			
			glBegin GL_TRIANGLE_STRIP
				for i = 0 to 16 step res
					tx = edge(i / 8, 0, 1)
							
					glTexCoord2f tx, 0.5
					glVertex3f 0.26 * tabs.cosseno(i), 0.26 * tabs.seno(i), 1.05
					glTexCoord2f tx, 0.75
					glVertex3f 0.2 * tabs.cosseno(i), 0.2 * tabs.seno(i), 0.88
				next i
			glEnd
			
			glBegin GL_TRIANGLE_FAN
				glTexCoord2f 0.5, 1
				glVertex3f 0, 0, 0.8
				
				for i = 0 to 16 step res
					glTexCoord2f i / 8, 0.8
					glVertex3f 0.2 * tabs.cosseno(i), 0.2 * tabs.seno(i), 0.88
				next i
			glEnd
		end if

		glPopMatrix
	end if
	
	'velocmetro
	dim as coord3d p1, p2
					
	p1 = velocimetro
	p2.x = p1.x + (0.09 * sin(angvel))
	p2.y = p1.y
	p2.z = p1.z + (0.09 * cos(angvel))
			
	_notex_
	glColor3f 1, 0.1, 0
	glBegin GL_LINES
		glVertex3f p1.x, p1.y, p1.z
		glVertex3f p2.x, p2.y, p2.z
	glEnd
	
	'volante
	const as double tam = 0.38
	
	glPushMatrix
	glTranslatef volante.x, volante.y, volante.z
	glRotatef ang, 0, 1, 0
	
	env.setCor 1, 1, 1
	painel.tex.aplica
	glBegin GL_TRIANGLE_STRIP
		glTexCoord2f 0, 0.625
		glVertex3f tam * -1, 0, tam * 0.5
		glTexCoord2f 0.125, 0.625
		glVertex3f tam * -0.5, 0, tam * 0.5
		glTexCoord2f 0.125, 0.5
		glVertex3f tam * -0.5, 0, tam * 1
		glTexCoord2f 0.375, 0.625
		glVertex3f tam * 0.5, 0, tam * 0.5
		glTexCoord2f 0.375, 0.5
		glVertex3f tam * 0.5, 0, tam * 1
		glTexCoord2f 0.5, 0.625
		glVertex3f tam * 1, 0, tam * 0.5
	glEnd
		
	glBegin GL_QUADS			
		glTexCoord2f 0, 0.625
		glVertex3f tam * -1, 0, tam * 0.5
		glTexCoord2f 0, 0.999
		glVertex3f tam * -1, 0, tam * -1	
		glTexCoord2f 0.5, 0.999
		glVertex3f tam * 1, 0, tam * -1		
		glTexCoord2f 0.5, 0.625
		glVertex3f tam * 1, 0, tam * 0.5
	glEnd
	
	glPopMatrix	
end sub

sub modelo.pintaLuz
	if simples or LOD = 0 then exit sub

	_notex_	
	glBegin GL_QUADS
		glColor4f 1, 1, 1, 0.5
		glVertex3f largura * -0.8, comprimento * 1, 1
		glVertex3f largura * 0.8, comprimento * 1, 1
		
		glColor4f 1, 1, 1, 0
		glVertex3f largura * 1.5, comprimento * 2, 0.5
		glVertex3f largura * -1.5, comprimento * 2, 0.5
	glEnd
	
	glColor4f 1, 1, 1, 1
	env.setCor 1, 1, 1
end sub

'------------------------------------------------------------------------------------
sub carInfo.cria (id as string)
	nome = id
	
	'define caractersticas de fsica e gameplay
	select case id
		case "gol"
			acc(1) = 0.0045 * 3
			acc(2) = 0.0045 * 2
			acc(3) = 0.0045 * 1.4
			acc(4) = 0.0045 * 0.9
			inercia(1) = 0.965 ^ 0.7
			inercia(2) = 0.985 ^ 0.7
			inercia(3) = 0.9925 ^ 0.7
			inercia(4) = 0.996 ^ 0.7
			freio = 0.003
			grip = 0.9
			max_vel = 1.421
			max_ang = 1.2
			max_dano = 5
	end select
	
	'cria modelo
	mdl.cria id
	
	'carrega som
	motor.carrega "sfx/" + id + ".wav"
end sub

sub carInfo.destroi
	mdl.destroi
	motor.destroi
end sub

'------------------------------------------------------------------------------------
sub carro.novo (info as carInfo ptr)
	this.info = info
	
	'valores default
	piloto = "Racer X"
	cor = 0
	autogear = true
	is_ai = false
	ai_hp = 1
		
	spr.coord = trk.vl.getVertex(0).coord
	spr.ang = toGrau(ang2p(trk.vl.getvertex(0).coord, trk.vl.getVertex(1).coord))
	ang = spr.ang
	marcha = 1
	vel = 0
	rpm = 0
	acc = 0
	brk = 0
	angacc = 0
	angaccvis = 0
	drift = 0
	clutch = 0
	clutch_timer = 0
	evt_chegada = false
	evt_dnf = false
	evt_grama = false
	evt_colisao = false
	evt_derrapagem = false
	evt_cambio = false
	evt_arranha = false
	dano = 0
	rpmvis = 0
	anim = 0
	cron_id = 0
	vertex = 0
end sub

sub carro.controle
	acc = 0
	brk = 0
	angacc = 0
	
	'controle analgico
	angacc = *info.max_ang * kev.eixo_x * -1	
	acc = *info.acc(marcha) * kev.eixo_acc
	brk = *info.freio * kev.eixo_brk
	clutch = kev.eixo_clu

	'teclado / botes
	if not autogear then
		evt_cambio = false
		
		'cmbio semi-automtico
		if kev.confereTecla(kev.K_MARCHAUP, kev.PRESSED) and marcha < 4 then
			marcha = marcha + 1
			evt_cambio = true
			clutch_timer = 300
		end if
		
		if kev.confereTecla(kev.K_MARCHADOWN, kev.PRESSED) and marcha > 1 then
			marcha = marcha - 1
			evt_cambio = true
			clutch_timer = 300
		end if
				
		'H-shifter
		if kev.confereTecla(kev.K_MARCHA1, kev.PRESSED) then marcha = 1: evt_cambio = true
		if kev.confereTecla(kev.K_MARCHA2, kev.PRESSED) then marcha = 2: evt_cambio = true
		if kev.confereTecla(kev.K_MARCHA3, kev.PRESSED) then marcha = 3: evt_cambio = true
		if kev.confereTecla(kev.K_MARCHA4, kev.PRESSED) then marcha = 4: evt_cambio = true
	end if
	
	'para o ngulo do volante
	angaccvis = angacc
end sub

sub carro.AI
	dim path as coord3d
	dim as roadVertex atual, prox, prox2	
	dim as double ang_curva, vel_curva, dif, peso1, peso2
	dim as _bool_ freio = false	
	
	'necessrio atualizar antes de chamar fsica
	vertex = trk.vl.encontra(spr.coord)

	'carrega vrtices importantes
	atual = trk.vl.getVertex(vertex)
	prox = trk.vl.getVertex(vertex + 1)
	prox2 = trk.vl.getVertex(vertex + 2)
	
	'define coordenada de path:
	'motor estourado, sai da pista
	if evt_dnf then
		if dist2p(prox.path, prox.quad(0)) < dist2p(prox.path, prox.quad(1)) then
			path = prox.quad(3)
		else
			path = prox.quad(2)
		end if
	
	'correndo
	else
		'tenta traado normal
		path = prox.path
		
		'encontra mais prximo
		dim vizinho as carro
		dim as double menor_dist = 100
		
		dim i as integer
		for i = 0 to MAX_CARROS - 1
			dim c as carro
			c = grid.getCarro(i)
			
			if c.piloto <> this.piloto and c.piloto <> "No correu" then
				dim as double dist
				dist = dist2p(this.spr.coord, c.spr.coord)
				
				if dist < menor_dist then
					vizinho = c
					menor_dist = dist
				end if
			end if
		next i
		
		'collision avoidance
		if menor_dist < 20 then
			'livre adiante?
			dim as double ang = ang2p(spr.coord, path)
			dim sentido as integer
			
			if grid.raycasting(this, vizinho, ang, 15) then
				if dist2p(prox.path, bounds.coord(0)) < dist2p(prox.path, bounds.coord(1)) then
					sentido = 1
				else
					sentido = -1	
				end if
				
				'tem espao o suficiente para mudar de traado?
				dim as carro lado_esq, lado_dir
				
				lado_esq = this
				lado_esq.spr.coord.x = spr.coord.x + (4 * cos(toRad(spr.ang + 110)))
				lado_esq.spr.coord.y = spr.coord.y + (4 * sin(toRad(spr.ang + 110)))
				lado_esq.atualizaBoundaries
				
				lado_dir = this
				lado_dir.spr.coord.x = spr.coord.x + (4 * cos(toRad(spr.ang - 110)))
				lado_dir.spr.coord.y = spr.coord.y + (4 * sin(toRad(spr.ang - 110)))
				lado_dir.atualizaBoundaries
				
				if (sentido = 1 and grid.colide(lado_esq, vizinho)) or (sentido = -1 and grid.colide(lado_dir, vizinho)) then
					'segue reto
					ang = angmed(atual.ang, prox.ang)
					path = spr.coord
					path.x = spr.coord.x + (5 * cos(ang))
					path.y = spr.coord.y + (5 * sin(ang))
				end if

			'tenta desviar
			else
				if dist2p(prox.path, prox.quad(0)) < dist2p(prox.path, prox.quad(1)) then
					sentido = 1
				else
					sentido = -1	
				end if
			
				'quanto pode virar?
				dim as double max_ang, res
				max_ang = toRad(45)
				res = max_ang / 10
					
				'at encontrar caminho livre
				freio = true
				
				dim as integer passo = 0
				ang = atual.ang
				
				do
					'incremento
					if sentido = 1 then
						if passo = 0 then
							ang = ang - res
							if ang < (atual.ang - max_ang) then
								ang = atual.ang
								passo = 1
							end if
						else
							ang = ang + res
							if ang > (atual.ang + max_ang) then exit do
						end if
					else
						if passo = 0 then
							ang = ang + res
							if ang > (atual.ang + max_ang) then
								ang = atual.ang
								passo = 1
							end if
						else
							ang = ang - res
							if ang < (atual.ang - max_ang) then exit do
						end if
					end if

					'novo traado
					path = spr.coord
					path.x = spr.coord.x + (5 * cos(ang))
					path.y = spr.coord.y + (5 * sin(ang))
					
					'dentro da pista?
					if trk.vl.dentro(path, vertex, true) or trk.vl.dentro(path, vertex + 1, true) then
						' livre?
						if grid.raycasting(this, vizinho, ang, 9) then
							freio = false
							exit do
						end if
					end if
				loop
				
				'tenta travar reto
				if freio then
					ang = angmed(atual.ang, prox.ang)
					path = spr.coord
					path.x = spr.coord.x + (5 * cos(ang))
					path.y = spr.coord.y + (5 * sin(ang))
				end if
			end if
		end if
	end if
	
	'aponta para prxima coordenada
	ang_curva = wrap(toGrau(ang2p(spr.coord, path)), 0, 360)
	
	dif = ang - ang_curva
	if dif > 180 or (dif < 0 and abs(dif) < 180) then
		ang = ang + (1.25 * edge(abs(dif) / 5, 0, 1) * fps.getFator)
	else
		ang = ang - (1.25 * edge(abs(dif) / 5, 0, 1) * fps.getFator)
	end if
	ang = wrap(ang, 0, 360)
	spr.ang = ang
	
	'descobre velocidade da curva
	dif = abs(angdif(toRad(ang), ang2p(spr.coord, prox2.path)))
	
	peso1 = ((0.8 - edge(dif - 0.1, 0, 0.8)) / 0.8) ^ 1.3
	peso2 = edge(prox.dist - 0.7, 0, 1.1) / 1.1
		
	vel_curva = *info.max_vel * ((peso1 * 1) + (peso2 * 0))
	vel_curva = edge(vel_curva, *info.max_vel * 0.3, *info.max_vel * 1.03)
	
	'acelera ou freia para alcanar velocidade
	dif = abs(vel - vel_curva)
		
	if not freio and vel < vel_curva then
		acc = *info.acc(marcha) * edge(dif * 8, 0, 1) * ai_hp
		brk = 0
	else
		acc = 0
		brk = *info.freio * 3 'powerbrakes
	end if
	
	'embreagem  automtica
	clutch = 0
end sub

sub carro.atualizaBoundaries
	dim as double comprimento, largura
	comprimento = *info.mdl.comprimento
	largura = *info.mdl.largura

	'mapeia quinas do carro para coliso
	dim as coord3d diant, tras
	
	'1-4-0
	'|   |
	'5   7
	'|   |
	'2-6-3
	
	diant.x = spr.coord.x + (comprimento * cos(toRad(spr.ang)))
	diant.y = spr.coord.y + (comprimento * sin(toRad(spr.ang)))
	tras.x = spr.coord.x - (comprimento * cos(toRad(spr.ang)))
	tras.y = spr.coord.y - (comprimento * sin(toRad(spr.ang)))
	bounds.coord(0).x = diant.x + (largura * 0.8 * cos(toRad(spr.ang + 90)))
	bounds.coord(0).y = diant.y + (largura * 0.8 * sin(toRad(spr.ang + 90)))
	bounds.coord(1).x = diant.x + (largura * 0.8 * cos(toRad(spr.ang - 90)))
	bounds.coord(1).y = diant.y + (largura * 0.8 * sin(toRad(spr.ang - 90)))
	bounds.coord(2).x = tras.x + (largura * cos(toRad(spr.ang - 90)))
	bounds.coord(2).y = tras.y + (largura * sin(toRad(spr.ang - 90)))
	bounds.coord(3).x = tras.x + (largura * cos(toRad(spr.ang + 90)))
	bounds.coord(3).y = tras.y + (largura * sin(toRad(spr.ang + 90)))
	bounds.coord(4) = diant
	bounds.coord(6) = tras
	bounds.coord(5) = interpola(bounds.coord(1), bounds.coord(2), 0.5)
	bounds.coord(7) = interpola(bounds.coord(0), bounds.coord(3), 0.5)
end sub

sub carro.fisica
	dim as integer i, j

	'guarda estado atual
	dim lastcoord as coord3d
	lastcoord = spr.coord

	'encontra posio do carro
	vertex = trk.vl.encontra(spr.coord)

	'reseta eventos
	evt_grama = false
	evt_colisao = false
	evt_derrapagem = false
	evt_arranha = false
		
	'coliso com grama
	atualizaBoundaries
	
	if not is_ai then
		'lento
		for i = 0 to 3
			j = trk.vl.encontra(bounds.coord(i))		
			if not trk.vl.dentro(bounds.coord(i), j, true) then evt_grama = true
		next
	else
		'rpido
		evt_grama = not(trk.vl.dentro(spr.coord, vertex, true))
	end if
	
	'carro em ordem?
	if dano >= *info.max_dano then
		evt_dnf = true
		evt_chegada = true 'no chegou, s sinaliza para acabar a corrida
		
		if acc > 0 then acc = 0
	end if
	
	'cmbio semi-automtico
	if clutch_timer > 0 then
		clutch = 1
		if acc > 0 then acc = 0
		
		clutch_timer = fps.subtraiTempo(clutch_timer)
	end if
	
	'acabou a corrida: limitador
	if evt_chegada and vel > (*info.max_vel / 2) then acc = 0

	'perda de trao com derrapagem
	if not evt_grama then
		acc = acc * (1 - (edge(abs(drift) - 10, 0, 50) / 50))
		brk = brk * (1 - (edge(abs(drift) - 10, 0, 50) / 50))
	else
		acc = acc * (1 - (edge(abs(drift) - 10, 0, 50) / 150))
		brk = brk * (1 - (edge(abs(drift) - 10, 0, 50) / 150))
	end if

	'velocidade	
	vel = vel + (acc * (1 - clutch) * fps.getFator)
		
	if vel < 0 then 'neutraliza freio com velocidade negativa
		vel = vel + (brk * fps.getFator)
	else
		vel = vel - (brk * fps.getFator)
	end if
	
	'inrcia
	vel = vel * (*info.inercia(marcha) ^ fps.getFator ^ (1 - clutch)) * (*info.inercia(4) ^ fps.getFator ^ clutch)
	
	'ngulo do volante
	if brk > 0 then angacc += angacc * 0.4 * (brk / *info.freio)				'increase with brakes (cavalo de pau)
	if abs(vel) < 0.3 then angacc = angacc * (abs(vel) / 0.3)										'reduce with slow
	angacc = (angacc * 0.4) + ((angacc * ((*info.max_vel - abs(vel)) / *info.max_vel)) * 0.6)	'reduce with speed
	
	if vel < 0 then 'corrige vetor com velocidade negativa
		spr.ang = spr.ang + (angacc * -1 * fps.getFator)
		drift = drift + (angacc * -1 * fps.getFator)
	else
		spr.ang = spr.ang + (angacc * fps.getFator)
		drift = drift + (angacc * fps.getFator)
	end if
	
	'deslocamento no eixo do carro
	grip = (0.3 + (((*info.max_vel - vel) / *info.max_vel) * 0.7)) * *info.grip 'reduce with speed
	if vel <= 0 then
		grip = 1.5
	elseif vel < 0.5 then
		grip = grip ^ (vel / 0.5) 'increase with slow
	end if
	
	if evt_grama then
		vel = vel * (0.996 ^ fps.getFator)
		
		grip = grip * 0.5
		if vel < 0.25 then grip = grip ^ (vel / 0.25)
	end if
	
	'drift
	if angdif (toRad(ang), toRad(spr.ang)) > toRad(60) then 'no deixa fugir do controle
		if drift > 0 then
			spr.ang = ang + 60
			drift = 60
		elseif drift < 0 then
			spr.ang = ang - 60
			drift = -60
		end if
	end if
		
	if drift > 0 then
		ang = ang + (*info.max_ang * grip * fps.getFator)
		drift = drift - (*info.max_ang * grip * fps.getFator)
		
		if drift < 0 then
			ang = ang + drift
			drift = 0
		end if			
	elseif drift < 0 then
		ang = ang - (*info.max_ang * grip * fps.getFator)
		drift = drift + (*info.max_ang * grip * fps.getFator)
		
		if drift > 0 then
			ang = ang + drift
			drift = 0
		end if
	end if
	
	if abs(drift) > 15 then evt_derrapagem = true
	
	'voltas completas
	ang = wrap(ang, 0, 360)
	spr.ang = wrap(spr.ang, 0, 360)
	
	'deslocamento do carro	
	spr.coord.x = spr.coord.x + (vel * cos(toRad(ang)) * fps.getFator)
	spr.coord.y = spr.coord.y + (vel * sin(toRad(ang)) * fps.getFator)
	
	'coliso simples
	dim as integer old_vertex = vertex
	vertex = trk.vl.encontra(spr.coord)
	
	if vertex = -1 then
		'retorna  pista
		dim ant as roadVertex
		ant = trk.vl.getVertex(old_vertex)
		
		spr.coord = ant.coord
		spr.ang = toGrau(ant.ang)
		ang = toGrau(ant.ang)
		vel = 0		
		
	elseif trk.vl.colide(bounds) then
		spr.coord = lastcoord
				
		'descobre de que lado bateu
		dim as roadVertex atual, prox
		dim as double dist_esq, dist_dir, ang_frente
		
		atual = trk.vl.getVertex(vertex)
		prox = trk.vl.getVertex(vertex + 1)		
		
		dist_esq = (dist2p(spr.coord, atual.quad(2)) + dist2p(spr.coord, prox.quad(2))) / dist2p(atual.quad(2), prox.quad(2))
		dist_dir = (dist2p(spr.coord, atual.quad(3)) + dist2p(spr.coord, prox.quad(3))) / dist2p(atual.quad(3), prox.quad(3))
		
		if dist_esq < dist_dir then
			ang_frente = ang2p(atual.quad(2), prox.quad(2))
		else
			ang_frente = ang2p(atual.quad(3), prox.quad(3))
		end if
		
		'descobre se bateu de frente
		dim as _bool_ bateu_frente = false
		
		for i = 0 to 1
			if trk.vl.encontra(bounds.coord(i)) < 0 then bateu_frente = true
		next
		
		'amortece velocidade
		vel = vel * (0.95 ^ abs(cos(toRad(ang + 90) - ang_frente)) ^ fps.getFator)
		
		'move perpendicularmente ao muro
		spr.coord.x = spr.coord.x + (vel * cos(ang_frente) * sin(toRad(ang + 90) - ang_frente) * fps.getFator)
		spr.coord.y = spr.coord.y + (vel * sin(ang_frente) * sin(toRad(ang + 90) - ang_frente) * fps.getFator)
		
		'empurra de volta  pista
		if (not bateu_frente) or angacc <> 0 or is_ai then
			if dist_esq < dist_dir then	
				spr.coord.x = spr.coord.x + (0.05 * cos(ang_frente + (pi / 2)) * fps.getFator)
				spr.coord.y = spr.coord.y + (0.05 * sin(ang_frente + (pi / 2)) * fps.getFator)
			else
				spr.coord.x = spr.coord.x + (0.05 * cos(ang_frente - (pi / 2)) * fps.getFator)
				spr.coord.y = spr.coord.y + (0.05 * sin(ang_frente - (pi / 2)) * fps.getFator)
			end if
		end if
		
		'dano de coliso
		if vel > 0.5 then dano = edge(dano + ((vel - 0.5) * abs(cos(toRad(ang + 90) - ang_frente)) * 0.4 * fps.getFator), 0, *info.max_dano)
	end if
	
	'rpm / marcha automtica
	dim as double rpm_clutch
	
	rpm = abs(vel) / (*info.acc(1) / *info.acc(marcha))
	rpm_clutch = (((acc / *info.acc(marcha)) * 0.46) * clutch) + (rpm * (1 - clutch))
	
	if rpmvis < rpm_clutch then
		rpmvis = edge(rpmvis + (0.02 * fps.getFator), 0, rpm_clutch)
	elseif rpmvis > rpm_clutch then
		rpmvis = edge(rpmvis - (0.015 * fps.getFator), rpm_clutch, 999)
	end if
	
	if autogear then
		evt_cambio = false
	
		if rpm > 0.40 and marcha < 4 then marcha = marcha + 1: evt_cambio = true
		if rpm < 0.22 and marcha > 1 then marcha = marcha - 1: evt_cambio = true
		
		if evt_cambio then
			clutch = 1
			clutch_timer = 300
		end if
	end if

	'dano da caixa de cmbio
	if rpmvis > 0.46 then dano = edge(dano + ((rpmvis - 0.46) * fps.getFator), 0, *info.max_dano)
			
	if evt_cambio and clutch < 1 then
		evt_arranha = true	
		dano = edge(dano + ((*info.max_dano / 8) * (1 - clutch)), 0, *info.max_dano)
	end if
	
	'animaes cclicas
	anim = wrap(anim + (edge(vel, 0, 0.5) * 2 * fps.getFator), 0, 32 * pi)
end sub

sub carro.atualizaSons
	dim as integer i
	
	'atualiza posio e velocidade
	dim velocity as coord3d
	velocity.x = vel * cos(toRad(ang)) * FATOR_DOPPLER
	velocity.y = vel * sin(toRad(ang)) * FATOR_DOPPLER
	velocity.z = 0
	
	som.setPos cron_id, spr.coord
	som.setVelocity cron_id, velocity

	'nota do motor
	som.setPitch cron_id, 0.4 + (1.4 * rpmvis)
		
	if evt_dnf then
		som.setVolume cron_id, 0
	else
		som.setVolume cron_id, 0.6
	end if
	
	'coliso
	if evt_colisao then
		som.toca colisao, spr.coord, 1
	end if
	
	'jogador
	if not is_ai then
		som.setPos som.getPlayer, spr.coord
		som.setPos som.getPlayer + 1, spr.coord
		som.setVelocity som.getPlayer, velocity
		som.setVelocity som.getPlayer + 1, velocity
	
		'derrapagem
		if not evt_grama then
			som.setVolume som.getPlayer, (edge(abs(drift), 0, 30) / 30) * 1.2
		else
			som.setVolume som.getPlayer, 0
		end if
		
		'grama
		if evt_grama and vel > 0.1 then
			som.setPitch som.getPlayer + 1, 0.8 + edge(vel, 0, 0.8)
			som.setVolume som.getPlayer + 1, 1
		else
			som.setVolume som.getPlayer + 1, 0
		end if
	
		'cmbio
		if evt_cambio and not cam.tv then som.toca cambio, spr.coord, 0.6
	end if
end sub

sub carro.chassisRollTransform (sentido as integer)
	'roll horizontal
	if sentido = 1 then				'casco
		glRotatef angacc * 4, 0, 1, 0
	elseif sentido = -1 then		'cmera
		glRotatef angacc * 4, 0, 0, 1
	end if
	
	'roll vertical
	if not is_ai and brk > 0 then glRotatef edge(vel, 0, 0.5) * (brk / *info.freio) * -3 * sentido, 1, 0, 0
	
	'vibrao grama
	if evt_grama then
		if sentido = 0 then			'rodas
			glTranslatef 0, 0, (1 + sin(anim)) * 0.05 * vel
		elseif sentido = 1 then		'casco
			glTranslatef 0, 0, (1 + sin(anim)) * 0.025 * vel
		elseif sentido = -1 then	'cmera
			glTranslatef 0, (1 + sin(anim)) * -0.03 * vel, 0
		end if
		
	'rumble
	elseif sentido = 1 then
		glRotatef toGrau(sin(anim)) * 0.15 * edge(rpmvis - 0.41, 0, 0.1) * (1 - clutch), 1, 0, 0
	end if
end sub

sub carro.pinta (sombra as _bool_)
	'desenha base
	glPushMatrix
		glTranslatef spr.coord.x, spr.coord.y, spr.coord.z		
				
		glRotatef spr.ang - 90, 0, 0, 1
		
		if sombra then *info.mdl.pintaSombra spr.ang, spr.coord
		
		glPushMatrix
			chassisRollTransform 0
			*info.mdl.pintaBase angaccvis * 25, anim
		glPopMatrix
			
		glPushMatrix
			chassisRollTransform 1
			*info.mdl.pintaCasco palete(Cor)
			*info.mdl.pintaPiloto angaccvis * -55, toRad(-90 + ((vel / *info.max_vel) * 180)), spr.coord
			if env.estrelas_on then *info.mdl.pintaLuz
		glPopMatrix
	glPopMatrix
end sub

sub carro.pinta
	pinta true
end sub

sub carro.pintaHud
	dim i as double
	dim as double raio_cron, h_cron, w_cron, w_cron2, w_dano
	raio_cron = h * 0.15			'raio do velocmetro
	h_cron = raio_cron * 1.5		'altura do velocmetro na tela
	w_cron = h * 0.0233				'grossura das marcaes
	w_cron2 = w_cron / 2
	w_dano = raio_cron * 0.06		'grossura da barra de dano

	'cronmetro
	race.cron(cron_id).pintaHud
	
	if not cam.tv then
		'rpm
		glPushMatrix
		glTranslatef w - raio_cron - (borda_hud * 2), h - (h_cron - raio_cron) - borda_hud, 0
		
		gfx.hud.aplica
		for i = -120 to 120 step 20
			glPushMatrix
			glRotatef i, 0, 0, 1
			
			if i <= 60 then
				glColor3f 1, 1, 1
			else
				glColor3f 1, 0.1, 0
			end if
			
			glBegin GL_QUADS
				glTexCoord2f 0, 0
				glVertex2f w_cron * -1, raio_cron * -1
				glTexCoord2f 0, 0.5
				glVertex2f w_cron * -1, raio_cron * -0.74
				glTexCoord2f 0.5, 0.5
				glVertex2f w_cron, raio_cron * -0.74
				glTexCoord2f 0.5, 0
				glVertex2f w_cron, raio_cron * -1
			glEnd
			
			glPopMatrix
		next i
		
		'ponteiro
		dim as double ang_ponteiro
		ang_ponteiro = edge((rpmvis - 0.25) * 520, -120, 120)
		
		'pino central
		gfx.hud.aplica
		glColor3f 1, 1, 1
		glBegin GL_QUADS
			glTexCoord2f 0, 1
			glVertex2f w_cron2 * -1, w_cron2
			glTexCoord2f 0.5, 1
			glVertex2f w_cron2, w_cron2
			glTexCoord2f 0.5, 0.5
			glVertex2f w_cron2, w_cron2 * -1
			glTexCoord2f 0, 0.5
			glVertex2f w_cron2 * -1, w_cron2 * -1
		glEnd
		
		'agulha
		glPushMatrix
		glRotatef ang_ponteiro, 0, 0, 1
				
		glEnable GL_LINE_SMOOTH	
		if frac(w / 360) > 0 then
			glLineWidth fix(w / 360) + 1 'arredonda para cima
		else
			glLineWidth w / 360
		end if	
		
		_notex_
		glColor3f 1, 0.1, 0
		glBegin GL_LINES
			glVertex2f 0, raio_cron * 0.1
			glVertex2f 0, raio_cron * -0.65
		glEnd
		
		glLineWidth 1
		glDisable GL_LINE_SMOOTH
		
		glPopMatrix		
			
		'marcha
		glPushMatrix
		glTranslatef 0, h_cron - raio_cron, 0
		glTranslatef _dx_ * -1.5, _dy_ * -1, 0
		
		fnt.setCor 1, 0.1, 0
		fnt.setSombra true
		fnt.pinta "( )", 0, 0
		fnt.setCor 1, 1, 1
		fnt.pinta " " + str(marcha), 0, 0
		
		glPopMatrix
		glPopMatrix
		
		'barra de dano		
		glPushMatrix
		glTranslatef w - borda_hud - w_dano, h - h_cron - borda_hud, 0
		
		_notex_			
		if dano > 0 then
			glColor4f 1, 0, 0, 0.5
			glBegin GL_QUADS
				glVertex2f w_dano * -1, h_cron
				glVertex2f w_dano, h_cron
				glVertex2f w_dano, h_cron * (1 - (dano / *info.max_dano))
				glVertex2f w_dano * -1, h_cron * (1 - (dano / *info.max_dano))
			glEnd
		end if
		
		glColor4f 1, 1, 1, 1
		glBegin GL_LINE_LOOP
			glVertex2f w_dano * -1, h_cron
			glVertex2f w_dano, h_cron
			glVertex2f w_dano, 0
			glVertex2f w_dano * -1, 0
		glEnd
		
		glPointSize 1
		glBegin GL_POINTS
			glVertex2f w_dano * -1, h_cron
			glVertex2f w_dano, h_cron
			glVertex2f w_dano, 0
			glVertex2f w_dano * -1, 0
		glEnd
		
		glPopMatrix
	end if
	
	'debug AI
	if is_ai then
		fnt.setSombra true
		fnt.setCor cor_texto
		fnt.pintaCentro piloto, 1
	end if
end sub

function carro.getInfo () as carInfo
	return *info
end function

'------------------------------------------------------------------------------------
sub carList.novo
	dim i as integer

	cont = -1
	viewing = 0
	largada = 3000
	
	for i = 0 to MAX_CARROS - 1
		som.defineVoz i, gol.motor
		som.setVolume i, 0
	next i
end sub

sub carList.add (info as carInfo ptr, piloto as string, cor as integer, autogear as _bool_, is_ai as _bool_, ai_hp as single)
	'estourou a lista?
	if cont >= (MAX_CARROS - 1) then exit sub
	
	'incrementa contador
	cont = cont + 1
	
	'cria carro
	car(cont).novo info
	car(cont).piloto = piloto
	car(cont).autogear = autogear
	car(cont).cor = cor
	car(cont).is_ai = is_ai
	car(cont).ai_hp = ai_hp
	car(cont).cron_id = cont
	
	'cria motor
	som.defineVoz cont, car(cont).getInfo.motor
	
	'posiciona no grid
	dim as double dist = 25 * cont
	dim as integer i = 0
	
	dim atual as roadVertex
	do
		atual = trk.vl.getVertex(i)
		
		if dist > (atual.dist * 20) then
			dist = dist - (atual.dist * 20)
			i = i + 1
		else
			exit do
		end if
	loop
	
	car(cont).spr.coord.x = atual.coord.x + (dist * cos(atual.ang))
	car(cont).spr.coord.y = atual.coord.y + (dist * sin(atual.ang))
	car(cont).spr.ang = toGrau(atual.ang)
	car(cont).ang = car(cont).spr.ang
	
	dim as double ang
	if (cont mod 2) = 0 then
		ang = atual.ang + (pi / 2)
	else
		ang = atual.ang - (pi / 2)
	end if
	
	car(cont).spr.coord.x = car(cont).spr.coord.x + (3 * cos(ang))
	car(cont).spr.coord.y = car(cont).spr.coord.y + (3 * sin(ang))
end sub

sub carList.add (info as carInfo ptr, piloto as string, cor as integer, autogear as _bool_)
	'insere jogador
	add info, piloto, cor, autogear, false, 0
end sub

sub carList.add (info as carInfo ptr, piloto as string, ai_hp as single)
	dim as integer i, cor, repetida
	
	'sorteia cor
	randomize timer
	
	do
		cor = int(rnd * MAX_CORES)
		repetida = false
		
		if cont >= 0 then
			for i = 0 to cont
				if car(i).cor = cor then repetida = true
			next
		end if
	loop while repetida
	
	'insere AI
	add info, piloto, cor, true, true, ai_hp
end sub

sub carList.finaliza
	dim as integer i, j	
	
	'inverte a ordem dos carros
	j = 0
	
	for i = 0 to cont
		race.cron(car(i).cron_id).cont = j
		j = j + 1
	next
	
	ordena
	
	'restaura crons
	for i = 0 to cont
		race.cron(car(i).cron_id).novo
	next
end sub

sub carList.ordena
	dim i as integer
	dim troca as _bool_
	dim tmp as carro

	'h uma lista para ordenar?
	if cont < 1 then exit sub

	'bubblesort
	do
		troca = false
		
		for i = 1 to cont
			dim as cronometro cron1, cron2
			cron1 = race.cron(car(i - 1).cron_id)
			cron2 = race.cron(car(i).cron_id)
			
			if cron1.atras(cron2) then troca = true
			
			'no deixa trocar aps a linha de chegada
			if (car(i).evt_chegada or car(i - 1).evt_chegada) and not (car(i).evt_dnf or car(i - 1).evt_dnf) then troca = false
			
			if troca then
				tmp = car(i - 1)
				car(i - 1) = car(i)
				car(i) = tmp
					
				'no perde vista do carro atual
				if viewing = i - 1 then
					viewing = i
				elseif viewing = i then
					viewing = i - 1
				end if
			end if
		next	
	loop while troca
end sub

sub carList.atualiza
	dim as integer i, j
	
	'tempo de espera
	largada = fps.subtraiTempo(largada)
	if largada = 0 and not race.evt_largada then
		race.evt_largada = true
		som.startYourEngines
	end if
	
	if cont >= 0 then
		'ordena se tem corrida rolando
		ordena
		
		'replay buffer
		replay.atualizaCabecote
		replay.grava this
		
		'processa cada carro
		dim as _bool_ todos_chegaram = true
		
		for i = 0 to cont
			if race.evt_largada then
				'controle
				if car(i).is_ai then
					car(i).AI
				else
					car(i).controle
				end if
				
				if not car(i).evt_chegada then
					'atualiza cronmetro
					if race.cron(car(i).cron_id).atualiza(car(i).vertex) then
						'no evento da corrida acabar
						car(i).evt_chegada = true
					end if
				end if
			end if

			'fsica e partculas
			car(i).fisica
			
			if car(i).vel > 0.25 and LOD = 3 then
				if car(i).evt_grama then
					trk.criaParticula car(i).bounds.coord(4), 7
					trk.criaParticula car(i).bounds.coord(6), 7
				elseif car(i).evt_derrapagem or car(i).evt_dnf then
					trk.criaParticula car(i).bounds.coord(4), 8
					trk.criaParticula car(i).bounds.coord(6), 8
				end if
			end if
			
			'esse carro chegou?
			if not car(i).evt_chegada then todos_chegaram = false				
		next
				
		'pares de carros
		for i = 0 to cont
			for j = i + 1 to cont
				'coliso
				if colide(car(i), car(j)) then colisao car(i), car(j)
			next j
		next i
	
		'sons (depois de coliso)
		for i = 0 to cont
			car(i).atualizaSons
		next i
	
		'configura cmera
		cam.setCarro car(viewing)
		
		'acabou a corrida?
		if todos_chegaram then race.evt_fim = true
	else
		'grid vazio - retornar erro?
	end if
end sub

sub carList.atualizaReplay
	'processa a carList sem que o replay grave por cima de si mesmo
	dim i as integer
	
	if cont >= 0 then
		'processa cada carro
		for i = 0 to cont
			'sons
			car(i).atualizaSons
		
			'recria partculas (mudar esse cdigo para uma sub)
			if car(i).vel > 0.25 and LOD = 3 then
				if car(i).evt_grama then
					trk.criaParticula car(i).bounds.coord(4), 7
					trk.criaParticula car(i).bounds.coord(6), 7
				elseif car(i).evt_derrapagem or car(i).evt_dnf then
					trk.criaParticula car(i).bounds.coord(4), 8
					trk.criaParticula car(i).bounds.coord(6), 8
				end if
			end if
		next
			
		'configura cmera
		cam.setCarro car(viewing)
	end if	
end sub

function carList.colide (byref car1 as carro, byref car2 as carro) as _bool_
	dim i as integer
	
	for i = 0 to 7
		if dentroQuad(car1.bounds.coord(i), car2.bounds.coord(0), car2.bounds.coord(1), car2.bounds.coord(2), car2.bounds.coord(3)) then
			return true
		end if
	next i
	
	return false
end function

function carList.raycasting (byref car1 as carro, byref car2 as carro, ang as double, dist as integer) as _bool_
	dim tmp as carro	
	dim d as integer
	
	tmp = car1
	
	for d = 3 to dist step 3
		'testa coliso (bem lento?)
		tmp.spr.coord.x = car1.spr.coord.x + (d * cos(ang))
		tmp.spr.coord.y = car1.spr.coord.y + (d * sin(ang))
		tmp.atualizaBoundaries
		
		if grid.colide(tmp, car2) then return false
	next d

	return true
end function

sub carList.colisao (byref car1 as carro, byref car2 as carro)
	'marca evento
	car1.evt_colisao = true
	car2.evt_colisao = true
	
	'vetor normal
	dim as double media, delta
	media = (car1.vel + car2.vel) / 2
	delta = cos(angdif(toRad(car1.ang), toRad(car2.ang)))
	
	if car1.vel > car2.vel then
		car1.vel = media * delta * 0.95 'simula dissipao (mal)
		car2.vel = media * delta * 1.05
	else
		car1.vel = media * delta * 1.05
		car2.vel = media * delta * 0.95
	end if
							
	'nudge
	dim i as integer
	dim dentro as _bool_
	dim as double ang_nudge, dist
	ang_nudge = ang2p(car1.spr.coord, car2.spr.coord)
	dist = 1
	
	do
		dentro = false
		
		car1.spr.coord.x = car1.spr.coord.x + (dist * -1 * cos(ang_nudge))
		car1.spr.coord.y = car1.spr.coord.y + (dist * -1 * sin(ang_nudge))
		
		car2.spr.coord.x = car2.spr.coord.x + (dist * cos(ang_nudge))
		car2.spr.coord.y = car2.spr.coord.y + (dist * sin(ang_nudge))
		
		'testa se j saiu
		car1.atualizaBoundaries
		car2.atualizaBoundaries
		
		for i = 0 to 5
			if dentroQuad(car1.bounds.coord(i), car2.bounds.coord(0), car2.bounds.coord(1), car2.bounds.coord(2), car2.bounds.coord(3)) then dentro = true
		next
	loop while dentro
	
	'dano
	dim as double amt_dano
	
	if delta > 0 then
		amt_dano = abs(car1.vel - car2.vel)
	else
		amt_dano = abs(car1.vel + car2.vel)
	end if
	
	amt_dano = amt_dano / ((car1.getInfo.max_vel + car2.getInfo.max_vel) / 2)
	
	car1.dano = edge(car1.dano + amt_dano, 0, car1.getInfo.max_dano)
	car2.dano = edge(car2.dano + amt_dano, 0, car2.getInfo.max_dano)
end sub

sub carList.pinta
	dim i as integer
	
	if cont >= 0 then
		for i = 0 to cont
			car(i).pinta
		next i
	end if
end sub

sub carList.pintaHud
	'contagem para largada
	if not race.evt_largada then
		fnt.setCor cor_texto
		fnt.setSombra true
		
		fnt.pintaCentro str(int(largada / 1000) + 1)
	
	'durante corrida
	else
		fnt.setCor cor_texto
		fnt.setSombra true
				
		if cont > 0 then
			fnt.pintaTopo "lap:", 1
			fnt.pintaTopo "pos:", 2
			
			'volta
			fnt.setCor 1, 1, 1
			fnt.pintaTopo "     " + str(race.cron(car(viewing).cron_id).volta + 1) + "/" + str(race.voltas), 1
			
			'posio
			dim as string sufixo
			select case viewing + 1
				case 1
					sufixo = "st"
				case 2
					sufixo = "nd"
				case 3
					sufixo = "rd"
				case 4 to 20
					sufixo = "th"
			end select
			
			fnt.pintaTopo "     " + str(viewing + 1) + sufixo, 2
		end if

		'mostradores e cronmetro individual
		car(viewing).pintaHud
		
		'resultados preliminares
		if race.evt_wait_for_others then
			fnt.setCor cor_texto
			fnt.pintaCentro "Race won by " + car(0).piloto
		end if
		
		'mapa
		pintaMapa
	end if
end sub

sub carList.pintaMapa
	dim temp as coord3d
	dim i as integer
	
	glPushMatrix
	'glTranslatef w * (1 - (0.125 * (1.333 / aspect_ratio))), (h * 0.125), 0
	'glTranslatef borda_hud * -1, borda_hud, 0
	glTranslatef w * (0.125 * (1.333 / aspect_ratio)), (h * (1 - 0.125)), 0
	glTranslatef borda_hud, borda_hud * -1, 0
	glScalef (w / (1024 * FATOR_MAG)) * 0.25 * (1.333 / aspect_ratio), (h / (768 * FATOR_MAG)) * -0.25, 0
	
	'pinta pista
	glEnable GL_POLYGON_SMOOTH
	
	_notex_	
	trk.vl.pintaMapa
	
	glDisable GL_POLYGON_SMOOTH
	
	'pinta carro(s)
	dim as double tam = 30
	
	gfx.hud.aplica
	if cont >= 0 then
		for i = cont to 0 step - 1
			temp = car(i).spr.coord
			
			glColor4f palete(car(i).cor).r, palete(car(i).cor).g, palete(car(i).cor).b, 1
			glBegin GL_QUADS
				glTexCoord2f 0.5, 0
				glVertex2f temp.x - tam, temp.y - tam
				glTexCoord2f 0.5, 0.5
				glVertex2f temp.x - tam, temp.y + tam
				glTexCoord2f 1, 0.5
				glVertex2f temp.x + tam, temp.y + tam
				glTexCoord2f 1, 0
				glVertex2f temp.x + tam, temp.y - tam
			glEnd
		next
	end if	
	glColor3f 1, 1, 1

	glPopMatrix
end sub

sub carList.viewUp
	'up significa ir ao incio do grid
	viewing = viewing - 1
	if viewing < 0 then viewing = cont
end sub

sub carList.viewDown
	'down significa ir ao final do grid
	viewing = viewing + 1
	if viewing > cont then viewing = 0
end sub

function carList.getCarro (posicao as integer) as carro
	if posicao >= 0 and posicao <= cont then
		return car(posicao)
	end if
	
	dim nulo as carro
	nulo.novo @gol
	nulo.piloto = "No correu"
	return nulo
end function

'------------------------------------------------------------------------------------
sub camera.nova
	car.spr.coord.x = 0
	car.spr.coord.y = 0
	car.spr.coord.z = 0
	car.spr.ang = 90
	car.ang = 90
	config = 3
	tv = false
	montada = false
	espelhada = false
	zoom = FOV
	transform = 0
	quick_look = 0
end sub

sub camera.orto
	'configura projeo ortogonal
	glDisable GL_DEPTH_TEST
	
	glMatrixMode GL_PROJECTION
	glPushMatrix
	glLoadIdentity
	
	glOrtho 0, w, h, 0, -1, 1
	
	glMatrixMode GL_MODELVIEW
	glPushMatrix
	glLoadIdentity
end sub

sub camera.restaura
	'restaura projeo 3D
	glMatrixMode GL_PROJECTION
	glPopMatrix
	glMatrixMode GL_MODELVIEW
	glPopMatrix
	
	glEnable GL_DEPTH_TEST
end sub

function camera.getEye (i as integer) as coord3d
	dim as coord3d temp
	
	select case i
		'front
		case 7
			temp.x = car.spr.coord.x - (10 * cos(toRad(car.ang - 160)))
			temp.y = car.spr.coord.y - (10 * sin(toRad(car.ang - 160)))
			temp.z = 3
		'hood
		case 6
			temp.x = car.spr.coord.x + (1.7 * cos(toRad(car.spr.ang)))
			temp.y = car.spr.coord.y + (1.7 * sin(toRad(car.spr.ang)))
			temp.z = 1.74			
		'cockpit
		case 5
			temp.x = car.spr.coord.x + (0.67 * cos(toRad(car.spr.ang + 128)))
			temp.y = car.spr.coord.y + (0.67 * sin(toRad(car.spr.ang + 128)))
			temp.z = 1.65
		'follow
		case 4
			temp.x = car.spr.coord.x - (9.5 * cos(toRad(car.ang + quick_look)))
			temp.y = car.spr.coord.y - (9.5 * sin(toRad(car.ang + quick_look)))
			temp.z = 3.6			
		'birds eye
		case 3
			temp.x = car.spr.coord.x - (25 * cos(toRad(car.ang + (quick_look * 0.3))))
			temp.y = car.spr.coord.y - (25 * sin(toRad(car.ang + (quick_look * 0.3))))
			temp.z = 10
		'TV
		case 2
			dim as roadVertex atual, prox
			atual = trk.vl.getVertex(car.vertex - (car.vertex mod 8) + 4)
			prox = trk.vl.getVertex(car.vertex - (car.vertex mod 8) + 5)
			
			if atual.ang < prox.ang and abs(atual.ang - prox.ang) < pi then
				temp = atual.quad(2)
				temp.x = temp.x + (1 * cos(atual.ang - (pi / 2)))
				temp.y = temp.y + (1 * sin(atual.ang - (pi / 2)))
			else
				temp = atual.quad(3)
				temp.x = temp.x + (1 * cos(atual.ang + (pi / 2)))
				temp.y = temp.y + (1 * sin(atual.ang + (pi / 2)))
			end if			
			
			temp.z = 2.5 + (dist2p(temp, car.spr.coord) * 0.03)
		'blimp
		case 1
			temp.x = car.spr.coord.x
			temp.y = car.spr.coord.y
			temp.z = 200
	
		case else
			temp.nova
	end select
	
	return temp
end function

function camera.getLook (i as integer) as coord3d
	dim as coord3d temp

	select case i
		'front
		case 7
			temp.x = car.spr.coord.x
			temp.y = car.spr.coord.y
			temp.z = 3
		'hood
		case 6
			temp.x = car.spr.coord.x + (20 * cos(toRad(car.spr.ang + quick_look)))
			temp.y = car.spr.coord.y + (20 * sin(toRad(car.spr.ang + quick_look)))
			temp.z = 1.8
		'cockpit
		case 5
			temp.x = car.spr.coord.x + (20 * cos(toRad(car.spr.ang + quick_look)))
			temp.y = car.spr.coord.y + (20 * sin(toRad(car.spr.ang + quick_look)))
			temp.z = 1
		'follow
		case 4
			temp.x = car.spr.coord.x
			temp.y = car.spr.coord.y
			temp.z = 3.6
		'birds eye
		case 3
			temp.x = car.spr.coord.x + (20 * cos(toRad(car.ang)))
			temp.y = car.spr.coord.y + (20 * sin(toRad(car.ang)))
			temp.z = 10
		'TV
		case 2
			temp.x = car.spr.coord.x - (2 * car.vel * cos(toRad(car.ang)))
			temp.y = car.spr.coord.y - (2 * car.vel * sin(toRad(car.ang)))
			temp.z = car.spr.coord.z
		'blimp
		case 1
			temp.x = car.spr.coord.x
			temp.y = car.spr.coord.y + 1
			temp.z = 0
		
		case else
			temp.nova
	end select
	
	return temp
end function

sub camera.atualiza (teste as _bool_)
	dim as double zfar = 4096

	'treme a cmera?
	if config = 5 or config = 6 then
		montada = true
	else
		montada = false
	end if
	
	'no mostra espelho, hud?
	if config = 1 or config = 2 or config = 7 then
		tv = true
	else
		tv = false
	end if
	
	espelhada = false
	
	'calcula coordenadas
	dim as coord3d eye1, eye2, look1, look2
	eye1 = getEye(config)
	eye2 = getEye(prev_config)
	look1 = getLook(config)
	look2 = getLook(prev_config)
	
	'calcula transio suave
	dim as double n_transform
	
	transform = edge(transform - (0.04 * fps.getFator), 0, 1)
	n_transform = 1 - transform
	
	eye = interpola(eye1, eye2, transform)
	look = interpola(look1, look2, transform)
	
	zoom = FOV
	if config = 2 or prev_config = 2 then
		zoom = edge(zoom ^ ((320 - dist2p(eye, look)) / 320), 5, 90)
		
		if config = 2 then
			zoom = (FOV * transform) + (zoom * n_transform)
		else
			zoom = (zoom * transform) + (FOV * n_transform)
		end if
	end if

	if not teste then
		'prepara projeo grfica
		glClear GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT
		
		glMatrixMode GL_PROJECTION
		glLoadIdentity
		glViewport 0, 0, w, h
		gluPerspective zoom, aspect_ratio, 0.1, zfar	
			
		glMatrixMode GL_MODELVIEW
		glLoadIdentity
		if montada then car.chassisRollTransform -1
		gluLookAt eye.x, eye.y, eye.z, look.x, look.y, look.z, 0, 0, 1
		
		'atualiza posio do listener
		dim orientv (0 to 5) as ALfloat
		orientv(0) = cos(ang2p(eye, look))
		orientv(1) = sin(ang2p(eye, look))
		orientv(2) = 0
		orientv(3) = 0
		orientv(4) = 0
		orientv(5) = 1
		
		dim velocity as coord3d
		if config = 2 then
			velocity.x = 0
			velocity.y = 0
			velocity.z = 0
		else
			velocity.x = car.vel * cos(toRad(car.ang)) * FATOR_DOPPLER
			velocity.y = car.vel * sin(toRad(car.ang)) * FATOR_DOPPLER
			velocity.z = 0
		end if
		
		alListenerfv AL_ORIENTATION, @orientv(0)
		alListener3f AL_POSITION, eye.x, eye.y, eye.z
		alListener3f AL_VELOCITY, velocity.x, velocity.y, velocity.z
	end if
end sub

sub camera.atualiza
	atualiza false
end sub


sub camera.espelho
	'define os limites do viewport
	dim as integer vpw, vph, vpx, vpy
	vpw = h * 0.5
	vph = h * 0.16
	vpx = (w - vpw) / 2
	vpy = h - vph - borda_hud	
	 
	'ainda estamos em modo ortogrfico, vamos preencher a janela do espelho
	_notex_	
	glBegin GL_QUADS
		glColor3f env.cor_clr.r, env.cor_clr.g, env.cor_clr.b
		glVertex2f vpx, h - (vpy + vph)
		glVertex2f vpx, h - (vpy + (vph / 2))
		glVertex2f vpx + vpw, h - (vpy + (vph / 2))
		glVertex2f vpx + vpw, h - (vpy + vph)
		
		env.setCor 0.5, 0.5, 0.5
		glVertex2f vpx, h - (vpy + (vph / 2))
		glVertex2f vpx, h - vpy
		glVertex2f vpx + vpw, h - vpy
		glVertex2f vpx + vpw, h - (vpy + (vph / 2))
	glEnd		
	glColor3f 1, 1, 1
	
	'sai do modo ortogrfico
	'se esta funo no  chamada, cam.restaura() deve ser chamada no fluxo!
	cam.restaura
	
	'limpa apenas depth, no color
	glClear GL_DEPTH_BUFFER_BIT
	
	'finalmente atualizamos eye e look...
	dim as double ang
	if montada then
		ang = toRad(car.spr.ang + 180)
	else
		ang = toRad(car.ang + 180)
	end if
	
	eye.x = car.spr.coord.x + (3 * cos(ang))
	eye.y = car.spr.coord.y + (3 * sin(ang))
	eye.z = 1.6
	look.x = car.spr.coord.x + (10 * cos(ang))
	look.y = car.spr.coord.y + (10 * sin(ang))
	look.z = 1.6
	
	montada = false 'apesar de ser montada, no queremos que trema ou no mostre partculas
	espelhada = true 'avisa para tratar face culling diferentemente e usar outros truques

	'...e configuramos o viewport com eixo x invertido
	glMatrixMode GL_PROJECTION
	glLoadIdentity
	glScalef -1, 1, 1
	glViewport vpx, vpy, vpw, vph
	gluPerspective 50, vpw / vph, 0.1, 4096
	
	glMatrixMode GL_MODELVIEW
	glLoadIdentity
	gluLookAt eye.x, eye.y, eye.z, look.x, look.y, look.z, 0, 0, 1
end sub

sub camera.setCarro (car as carro)
	this.car = car
end sub

sub camera.controle
	if kev.confereTecla(kev.K_CAMUP, kev.PRESSED) then up
	if kev.confereTecla(kev.K_CAMDOWN, kev.PRESSED) then down
	
	if kev.confereTecla(kev.K_LOOKL, kev.DOWN) then
		quick_look = edge(quick_look + (4 * fps.getFator), -70, 70)
	elseif kev.confereTecla(kev.K_LOOKR, kev.DOWN) then
		quick_look = edge(quick_look - (4 * fps.getFator), -70, 70)
	else
		if quick_look < 0 then
			quick_look = edge(quick_look + (5 * fps.getFator), -70, 0)
		else
			quick_look = edge(quick_look - (5 * fps.getFator), 0, 70)
		end if
	end if
end sub

sub camera.up
	prev_config = config
	transform = 1

	config = config + 1
	if config > MAX_CONFIGS then config = 1
end sub

sub camera.down
	prev_config = config
	transform = 1

	config = config - 1
	if config < 1 then config = MAX_CONFIGS
end sub

'------------------------------------------------------------------------------------
sub vertexList.pintaTriangleStrip (i1 as integer, i2 as integer, modo as integer)
	dim as double ty = 0
	dim i as integer
	dim atual as roadVertex
	
	glBegin GL_TRIANGLE_STRIP
		if modo = TS_ASFALTO then
			for i = 0 to cont + 1
				atual = getVertex(i)
				
				glTexCoord2f 0, ty
				glVertex3f atual.quad(i1).x, atual.quad(i1).y, 0
				
				glTexCoord2f atual.roadWidth / 4, ty
				glVertex3f atual.quad(i2).x, atual.quad(i2).y, 0
				
				ty = ty + atual.dist
			next
			
		elseif modo = TS_MURO0 then
			for i = 0 to cont + 1
				atual = getVertex(i)
				dim as double ang = atual.ang + (pi / 2)
				glNormal3f cos(ang), sin(ang), 2
				
				if not cam.espelhada then
					glTexCoord2f ty, 0.5
					glVertex3f atual.quad(i1).x, atual.quad(i1).y, 0
					
					glTexCoord2f ty, 0
					glVertex3f atual.quad(i2).x, atual.quad(i2).y, trk.MURO_H
				else
					glTexCoord2f ty, 0
					glVertex3f atual.quad(i1).x, atual.quad(i1).y, trk.MURO_H
					
					glTexCoord2f ty, 0.5
					glVertex3f atual.quad(i2).x, atual.quad(i2).y, 0
				end if
				
				ty = ty + atual.dist + atual.dist
			next
			
		elseif modo = TS_MURO1 then
			for i = 0 to cont + 1
				dim as double h1, h2
				if not cam.espelhada then
					h1 = 0
					h2 = trk.MURO_H
				else
					h1 = trk.MURO_H
					h2 = 0
				end if
			
				atual = getVertex(i)
				dim as double ang = atual.ang + (pi / 2)
				glNormal3f cos(ang) * -1, sin(ang) * -1, 2
				
				if not cam.espelhada then
					glTexCoord2f ty, 0
					glVertex3f atual.quad(i1).x, atual.quad(i1).y, trk.MURO_H
					
					glTexCoord2f ty, 0.5
					glVertex3f atual.quad(i2).x, atual.quad(i2).y, 0
				else
					glTexCoord2f ty, 0.5
					glVertex3f atual.quad(i1).x, atual.quad(i1).y, 0
					
					glTexCoord2f ty, 0
					glVertex3f atual.quad(i2).x, atual.quad(i2).y, trk.MURO_H
				end if
				
				ty = ty + atual.dist + atual.dist
			next
			
		elseif modo = TS_FAIXA then
			for i = 0 to cont + 1
				atual = getVertex(i)
			
				glTexCoord2f 0, ty
				glVertex3f atual.quad(i1).x, atual.quad(i1).y, 0
				
				glTexCoord2f 0.156, ty
				glVertex3f atual.quad(i2).x, atual.quad(i2).y, 0
				
				ty = ty + atual.dist
			next
		end if
	glEnd
end sub

sub vertexList.pinta
	const as double arv_h = 20
	const as double r_w = 8
	const as double r_h = 4
	const as double r_l = 1.5
	const as double poste_h = 10
	const as double poste_raio = 0.5
	
	dim as roadVertex largada
	dim as integer i, j, m
	dim as integer tx(0 to 1)
	
	glShadeModel GL_FLAT
		
	'pista
	if cam.eye.z > 100 then	glDisable GL_DEPTH_TEST 'evita z-fighting em cmera area
	
	if LOD = 0 then
		env.setCor 0.3, 0.3, 0.3
		_notex_
	else
		env.setCor 1, 1, 1
		gfx.asfalto.aplica
	end if
	pintaTriangleStrip 0, 1, TS_ASFALTO
	
	'muros
	glEnable GL_CULL_FACE
	glEnable GL_LIGHTING
	
	env.setCor 1, 1, 1
	gfx.muro.aplica
	pintaTriangleStrip 2, 2, TS_MURO0
	pintaTriangleStrip 3, 3, TS_MURO1
	pintaTriangleStrip 2, 2, TS_MURO1
	pintaTriangleStrip 3, 3, TS_MURO0
	
	glDisable GL_LIGHTING
	glDisable GL_CULL_FACE
	
	'faixas
	glEnable GL_POLYGON_OFFSET_FILL
	glPolygonOffset -0.4, -1
	
	if LOD = 0 then
		env.setCor 1, 0.98, 0.9
		_notex_
	else
		env.setCor 1, 1, 1
		gfx.faixa.aplica
	end if		
	pintaTriangleStrip 4, 5, TS_FAIXA
	pintaTriangleStrip 6, 7, TS_FAIXA
		
	glDisable GL_POLYGON_OFFSET_FILL
	if cam.eye.z > 100 then glEnable GL_DEPTH_TEST
			
	'::: LINHA DE CHEGADA :::
	largada = getVertex(0)
	
	glPushMatrix
	if largada.escapeWidth(0) < largada.escapeWidth(1) then
		glTranslatef largada.quad(2).x, largada.quad(2).y, 0
		glRotatef toGrau(largada.ang) + 90, 0, 0, 1
	else
		glTranslatef largada.quad(3).x, largada.quad(3).y, 0
		glRotatef toGrau(largada.ang) - 90, 0, 0, 1
	end if
	
	'poste
	env.setCor 1, 1, 1
	gfx.relogio.aplica	
	glBegin GL_QUAD_STRIP
		for i = 0 to 16 step 2
			glTexCoord2f i / 64, 0.5
			glVertex3f poste_raio * tabs.seno(i), poste_raio * tabs.cosseno(i), poste_h - r_h
			glTexCoord2f i / 64, 1
			glVertex3f poste_raio * tabs.seno(i), poste_raio * tabs.cosseno(i), 0
		next i
	glEnd

	'relgio e sinal
	glBegin GL_QUAD_STRIP
		env.setCor 0.5, 0.5, 0.5	
		glTexCoord2f 0, 0
		glVertex3f r_w * -0.1, r_l * 0.5, poste_h
		glTexCoord2f 0, 0.5
		glVertex3f r_w * -0.1, r_l * 0.5, poste_h - r_h
		
		env.setCor 1, 1, 1
		glTexCoord2f 1, 0
		glVertex3f r_w * 0.9, r_l * 0.5, poste_h
		glTexCoord2f 1, 0.5
		glVertex3f r_w * 0.9, r_l * 0.5, poste_h - r_h
		
		env.setCor 0.5, 0.5, 0.5
		glTexCoord2f 1, 0
		glVertex3f r_w * 0.9, r_l * -0.5, poste_h
		glTexCoord2f 1, 0.5
		glVertex3f r_w * 0.9, r_l * -0.5, poste_h - r_h
		
		env.setCor 1, 1, 1
		glTexCoord2f 0, 0
		glVertex3f r_w * -0.1, r_l * -0.5, poste_h
		glTexCoord2f 0, 0.5
		glVertex3f r_w * -0.1, r_l * -0.5, poste_h - r_h
		
		env.setCor 0.5, 0.5, 0.5
		glTexCoord2f 0, 0
		glVertex3f r_w * -0.1, r_l * 0.5, poste_h
		glTexCoord2f 0, 0.5
		glVertex3f r_w * -0.1, r_l * 0.5, poste_h - r_h
	glEnd
	
	'tampas e sombra
	glEnable GL_POLYGON_OFFSET_FILL
	glPolygonOffset -0.5, -1
	
	_notex_
	glBegin GL_QUADS
		env.setCor 0.1, 0.12, 0.11
	
		glVertex3f r_w * -0.1, r_l * 0.5, poste_h
		glVertex3f r_w * 0.9, r_l * 0.5, poste_h
		glVertex3f r_w * 0.9, r_l * -0.5, poste_h
		glVertex3f r_w * -0.1, r_l * -0.5, poste_h
			
		glVertex3f r_w * -0.1, r_l * 0.5, poste_h - r_h
		glVertex3f r_w * 0.9, r_l * 0.5, poste_h - r_h
		glVertex3f r_w * 0.9, r_l * -0.5, poste_h - r_h
		glVertex3f r_w * -0.1, r_l * -0.5, poste_h - r_h
		
		glColor4f 0, 0, 0, 0.3
		glVertex3f 0, r_l * 0.5, 0
		glVertex3f r_w * 0.9, r_l * 0.5, 0
		glVertex3f r_w * 0.9, r_l * -0.5, 0
		glVertex3f 0, r_l * -0.5, 0
	glEnd	
	
	glDisable GL_POLYGON_OFFSET_FILL
	
	'ponteiros
	glColor4f 1, 1, 1, 1
	
	if not cam.espelhada then
		glDisable GL_DEPTH_TEST 'o correto seria fazer polygon_offset, mas, pela ordem da geometria,  possvel
		glEnable GL_CULL_FACE
	
		glPushMatrix
			glTranslatef (0.0956 * r_w), 0, poste_h - (0.36 * r_h)
			
			for i = 0 to 1
				'hora ou minuto?
				dim as double ang
				
				if i = 1 then
					ang = env.getAng
					env.setCor 0, 0, 0, 1
				else
					ang = wrap(env.getAng * 12, 0, 360)
					env.setCor 0.5, 0.5, 0.5
				end if				
			
				'pinta
				for j = 0 to 1
					glPushMatrix
						if j = 0 then
							glTranslatef 0, r_l * 0.51, 0
							glRotatef ang * -1, 0, 1, 0
							glRotatef 180, 0, 0, 1
						else
							glTranslatef 0, r_l * -0.51, 0
							glRotatef ang, 0, 1, 0
						end if
							
						glBegin GL_TRIANGLE_FAN
							glVertex3f 0, 0, 0
							glVertex3f 0.05 + (0.05 * i), 0, -0.05
							glVertex3f 0, 0, 0.5 + (0.2 * (1 - i))
							glVertex3f -0.05 - (0.05 * i), 0, -0.1 
						glEnd
					glPopMatrix
				next j
			next i
		glPopMatrix
		
		glEnable GL_DEPTH_TEST
		glDisable GL_CULL_FACE
	end if		

	glPopMatrix	
	
	'postes menores
	glShadeModel GL_SMOOTH
	
	trk.pintaPoste largada.quad(2), trk.MURO_H, 9.5
	trk.pintaPoste largada.quad(3), trk.MURO_H, 9.5
	
	tx(0) = (largada.roadWidth + largada.escapeWidth(0) + largada.escapeWidth(1)) / 6
	
	env.setCor 1, 1, 1
	gfx.muro.aplica
	glBegin GL_QUADS
		glTexCoord2f 0, 0.85
		glVertex3f largada.quad(2).x, largada.quad(2).y, 9.5
		glTexCoord2f 0, 1
		glVertex3f largada.quad(2).x, largada.quad(2).y, 8.5
		glTexCoord2f tx(0), 1
		glVertex3f largada.quad(3).x, largada.quad(3).y, 8.5
		glTexCoord2f tx(0), 0.85
		glVertex3f largada.quad(3).x, largada.quad(3).y, 9.5
	glEnd
	
	'faixa
	if not cam.espelhada then
		tx(0) = 0
		tx(1) = 1
	else
		tx(0) = 1
		tx(1) = 0
	end if
	
	glEnable GL_CULL_FACE
	
	glColor3f 1, 1, 1
	gfx.billboard.aplica
	for i = -1 to 1 step 2	
		glPushMatrix
		glTranslatef largada.coord.x, largada.coord.y, 0
		glRotatef toGrau(largada.ang) + (90 * i), 0, 0, 1			
		
		glBegin GL_QUADS
			glTexCoord2f tx(0), 0.01
			glVertex3f -5, 0, 8.5
			glTexCoord2f tx(0), 0.24
			glVertex3f -5, 0, 6.5
			glTexCoord2f tx(1), 0.24
			glVertex3f 5, 0, 6.5
			glTexCoord2f tx(1), 0.01
			glVertex3f 5, 0, 8.5
		glEnd
		
		glPopMatrix
	next
	
	glDisable GL_CULL_FACE
end sub

sub vertexList.pintaSimples
	dim atual as roadVertex
	dim i as integer
	
	_notex_
	env.setCor 0.3, 0.3, 0.3
	glBegin GL_TRIANGLE_STRIP
		for i = 0 to cont + 1
			atual = getVertex(i)
			
			glVertex3f atual.quad(0).x, atual.quad(0).y, 0
			glVertex3f atual.quad(1).x, atual.quad(1).y, 0
		next
	glEnd
	
	env.setCor 138/256, 135/256, 69/256
	glBegin GL_TRIANGLE_STRIP
		for i = 0 to cont + 1
			atual = getVertex(i)
			
			glVertex3f atual.quad(2).x, atual.quad(2).y, 0
			glVertex3f atual.quad(0).x, atual.quad(0).y, 0
		next
	glEnd
	
	glBegin GL_TRIANGLE_STRIP
		for i = 0 to cont + 1
			atual = getVertex(i)
			
			glVertex3f atual.quad(1).x, atual.quad(1).y, 0
			glVertex3f atual.quad(3).x, atual.quad(3).y, 0
		next
	glEnd
end sub

sub vertexList.pintaMapa
	dim atual as roadVertex
	dim as integer i, j

	'configurao de projeo ortogonal a cargo da funo que chama!
	'precisa flipar eixo y
	glColor4f 0, 0, 0, 0.25
	glBegin GL_QUADS
		glVertex2f -512 * FATOR_MAG, 384 * FATOR_MAG
		glVertex2f -512 * FATOR_MAG, -384 * FATOR_MAG
		glVertex2f 512 * FATOR_MAG, -384 * FATOR_MAG
		glVertex2f 512 * FATOR_MAG, 384 * FATOR_MAG
	glEnd
	
	glColor4f 1, 1, 1, 0.5
	glBegin GL_TRIANGLE_STRIP
		for i = 0 to cont + 1
			atual = getVertex(i)
						
			glVertex2f atual.quad(0).x, atual.quad(0).y
			glVertex2f atual.quad(1).x, atual.quad(1).y
		next
	glEnd
end sub

sub vertexList.pintaEditor
	'configura projeo ortogrfica
	cam.atualiza
	cam.orto
	
	'fundo
	gfx.xadrez.aplica
	glBegin GL_QUADS
		glTexCoord2f 0, 0
		glVertex2f 0, 0
		glTexCoord2f 0, (h / 16)
		glVertex2f 0, h
		glTexCoord2f (w / 16), (h / 16)
		glVertex2f w, h
		glTexCoord2f (w / 16), 0
		glVertex2f w, 0
	glEnd
	
	'configura sistema de coordenadas
	glPushMatrix	
	glTranslatef w / 2, h / 2, 0
	glScalef (1 / FATOR_MAG) * (h / 768), (-1 / FATOR_MAG) * (h / 768), 1
	
	'pista
	if cp_cont > 2 then
		glPolygonMode GL_FRONT_AND_BACK, GL_LINE	
			
		pintaSimples
				
		glPolygonMode GL_FRONT_AND_BACK, GL_FILL
	end if
	
	'limites (widescreen)
	glColor3f 0, 0, 0
	
	glBegin GL_QUADS
		glVertex2f -2000, 512
		glVertex2f -2000, -512
		glVertex2f -512 * FATOR_MAG, -512
		glVertex2f -512 * FATOR_MAG, 512

		glVertex2f 512 * FATOR_MAG, 512
		glVertex2f 512 * FATOR_MAG, -512
		glVertex2f 2000, -512
		glVertex2f 2000, 512
	glEnd
		
	'traado AI
	dim i as integer
	dim atual as roadVertex
	
	glEnable GL_LINE_SMOOTH	
	glLineWidth 2
	
	_notex_
	glColor3f 0.6, 0.4, 1
	
	glBegin GL_LINE_STRIP
		for i = 0 to cont + 1
			atual = getVertex(i)
		
			glVertex2f atual.path.x, atual.path.y
		next
	glEnd
	
	glDisable GL_LINE_SMOOTH
	glLineWidth 1
	
	'pontos de controle
	glColor3f 1, 0, 0
	glPointSize 5
	fnt.setCor 1, 0, 0
	fnt.setSombra false
		
	glBegin GL_POINTS
		for i = 1 to cp_cont
			glVertex2f cp(i).x, cp(i).y			
		next
	glEnd		
	
	glPointSize 1	
	glColor3f 1, 1, 1
	
	'ponto de largada
	if cp_cont > 0 then
		gfx.spritegfx(3).aplica
		glBegin GL_QUADS
			glTexCoord2f 0, 0
			glVertex2f cp(1).x - 24, cp(1).y + 52
			glTexCoord2f 0, 1
			glVertex2f cp(1).x - 24, cp(1).y + 4
			glTexCoord2f 1, 1
			glVertex2f cp(1).x + 24, cp(1).y + 4
			glTexCoord2f 1, 0
			glVertex2f cp(1).x + 24, cp(1).y + 52
		glEnd
	end if
	
	glPopMatrix
	
	cam.restaura
	SDL_GL_SwapBuffers
end sub

'------------------------------------------------------------------------------------
sub spriteList.nova
	cont = -1
end sub

sub spriteList.add (spr as sprite2d)
	dim i as integer

	'h espao na lista?
	if cont < (MAX_SPRITES - 1) then
		'se est muito prximo de outro, descarta
		if spr.id < 5 then
			for i = 0 to cont
				if dist2p(this.spr(i).coord, spr.coord) < 2 then exit sub
			next i
		end if
		
		'adiciona
		cont = cont + 1
		this.spr(cont) = spr
	end if
end sub

sub spriteList.ordena
	dim i as integer
	dim troca as _bool_
	dim tmp as sprite2d
	
	'atualiza a distncia e a durao de cada sprite
	for i = 0 to cont
		spr(i).dist = ((cam.look.x - cam.eye.x) * (spr(i).coord.x - cam.eye.x)) + ((cam.look.y - cam.eye.y) * (spr(i).coord.y - cam.eye.y)) 'projeo
		spr(i).visivel = dist2p(cam.eye, spr(i).coord) < FOG_DIST
		
		'sprite no  permanente?
		if spr(i).vencimento > -1 and not cam.espelhada then 'no pode fazer isso duas vezes!
			spr(i).vencimento = spr(i).vencimento - ((1000 / 50) * fps.getFator)
			
			'efeito fumaa
			spr(i).tam = spr(i).tam + (0.08 * fps.getFator)	
			
			'joga vencidos para o final da lista no bubblesort
			if spr(i).vencimento < 0 then spr(i).dist = 999999 'valores normais de dist podem chegar a 20,000?
		end if
	next
	
	'h uma lista para ordenar?
	if cont < 1 then exit sub
	
	'bubblesort
	do
		troca = false
		
		for i = 1 to cont
			if spr(i - 1).dist > spr(i).dist then
				tmp = spr(i - 1)
				spr(i - 1) = spr(i)
				spr(i) = tmp
				troca = true
			end if
		next	
	loop while troca
	
	'retira elementos mortos do final da lista
	for i = cont to 0 step -1
		if spr(i).dist >= 999999 then
			cont = cont - 1
		else
			exit for
		end if
	next
end sub

sub spriteList.pinta
	dim i as integer
	
	'sort your shit!
	'se for o espelho, no precisa ordenar. vamos apenas ler a lista de trs para frente
	if not cam.espelhada then ordena
	
	'calcula o ngulo de todos os sprites (no precisa atribuir individualmente spr.ang se usa projeo de vetores)
	dim as double ang
	ang = toGrau(ang2p(cam.eye, cam.look))
	
	glEnable GL_POLYGON_OFFSET_FILL
	glPolygonOffset -0.5, -1
	
	'percorre sprites em ordem inversa de distncia
	if not cam.espelhada then i = cont: else i = 0: end if
	do
		if spr(i).visivel and _						'no mostra muito distantes
		not((spr(i).id = 7 or spr(i).id = 8) and cam.montada) then	'no mostra partculas em uma cmera interna...
			'posiciona sprite
			glPushMatrix
			glTranslatef spr(i).coord.x, spr(i).coord.y, spr(i).coord.z
			glRotatef ang - 90, 0, 0, 1
		
			dim as double tam, tam2, tam2n, tam4, tam4n 'otimiza?
			tam = spr(i).tam
			tam2 = tam / 2
			tam2n = tam / -2
		
			'pinta sombra
			if spr(i).id < 6 and LOD = 3 then
				tam4 = tam / 5 'devia ser 4, fiz um pouco menor
				tam4n = tam / -5
				
				gfx.sombra.aplica
				
				glBegin GL_QUADS
					glTexCoord2f 0, 0
					glVertex3f tam4n, tam4, 0
					
					glTexCoord2f 0, 1
					glVertex3f tam4n, tam4n, 0
					
					glTexCoord2f 1, 1
					glVertex3f tam4, tam4n, 0
					
					glTexCoord2f 1, 0
					glVertex3f tam4, tam4, 0
				glEnd
			end if
		
			'pinta sprite			
			gfx.spritegfx(spr(i).id).aplica
			
			glBegin GL_QUADS
				glTexCoord2f 0, 0
				glVertex3f tam2n, 0, tam
				
				glTexCoord2f 0, 1
				glVertex3f tam2n, 0, 0
				
				glTexCoord2f 1, 1
				glVertex3f tam2, 0, 0
				
				glTexCoord2f 1, 0
				glVertex3f tam2, 0, tam
			glEnd
			
			glPopMatrix
		end if
		
		'faz incremento
		if not cam.espelhada then
			i = i - 1
			if i <= 0 then exit do
		else
			i = i + 1
			if i >= cont then exit do
		end if
	loop
	
	glDisable GL_POLYGON_OFFSET_FILL
end sub

'------------------------------------------------------------------------------------
function track.espacoVazio (c as coord3d, delta as double) as integer
	dim as integer i, m, n
	dim temp as coord3d
	
	'calcula uma bounding box
	for i = 0 to 4
		select case i
			case 0:
				temp = c
			case 1:
				temp.x = c.x - delta
				temp.y = c.y - delta
			case 2:
				temp.x = c.x + delta
				temp.y = c.y - delta
			case 3:
				temp.x = c.x - delta
				temp.y = c.y + delta
			case 4:
				temp.x = c.x + delta
				temp.y = c.y + delta
		end select
		
		'est dentro da pista?
		if not(trk.vl.encontra(temp) < 0) then
			return false
			
		else
			'colide com veculos?
			for m = 0 to NUM_VEICULOS - 1
				'if dist2p(temp, veiculo(m).spr.coord) < 4 then return false
				if dentroQuad(temp, veiculo(m).bounds.coord(0), veiculo(m).bounds.coord(1), _
					veiculo(m).bounds.coord(2), veiculo(m).bounds.coord(3)) then return false
			next m
		end if
	next i
	
	'coordenada vlida
	return true
end function

sub track.plantaArvores
	dim i as integer
	dim spr as sprite2d

	print "Plantando arvores";
			
	for i = 0 to NUM_ARVORES - 1
		'encontra um lugar fora da pista
		dim arvore as coord3d
		
		do
			dim as integer x, y
			x = int(rnd * HEIGHTMAP_RES)
			y = int(rnd * HEIGHTMAP_RES)
			
			dim c as coord3d
			arvore = heightmap(x, y)
		loop until espacoVazio(arvore, 3)
				
		'coloca na lista de sprites
		spr.coord = arvore
		spr.id = int(rnd * 3)
		spr.tam = 8 + (rnd * 4)
		spr.vencimento = -1
		
		sl.add spr
		
		print ".";
	next i
	
	print
end sub

sub track.plantaFiscais
	dim as integer i, j, k
	dim spr as sprite2d
	
	'fiscais
	for i = 1 to NUM_FISCAIS
		'encontra um lugar prximo do guard-rail
		dim fiscal as coord3d
		
		do
			'sorteia um vrtice
			j = int(rnd * vl.cont)
			
			dim as roadVertex v
			v = trk.vl.getVertex(j)
			
			'esquerda ou direita?
			k = 2 + int(rnd * 2)
			
			fiscal.x = v.quad(k).x + ((rnd * 6) - 3)
			fiscal.y = v.quad(k).y + ((rnd * 6) - 3)
		loop until espacoVazio(fiscal, 1)
		
		'fiscal ou fotgrafo?
		if (i mod 2) = 0 then
			spr.id = 4
		else
			spr.id = 5
		end if
		
		'coloca na lista de sprites
		spr.coord = fiscal
		spr.tam = 4.5
		spr.vencimento = -1
		
		sl.add spr
	next i
end sub

sub track.plantaVeiculos
	dim as integer i, j, m

	'posiciona
	for i = 0 to NUM_VEICULOS - 2
		for j = 0 to 100
			'sorteia um vrtice
			do
				m = int(rnd * vl.cont)
			loop while m <= 3 or m >= (vl.cont - 3)
			
			dim atual as roadVertex
			atual = vl.getVertex(m)
			
			'define modelo
			select case i mod 3
				case 0: veiculo(i).novo @crane
				case 1: veiculo(i).novo @van
				case 2: veiculo(i).novo @casa
			end select
			
			'posiciona
			dim as double ang
			if rnd < 0.5 then
				veiculo(i).spr.coord = atual.quad(2)
				ang = pi / -2
			else
				veiculo(i).spr.coord = atual.quad(3)
				ang = pi / 2
			end if
			
			'flipa ngulo errado do guindaste
			if veiculo(i).getInfo.nome = "crane" then
				veiculo(i).spr.ang = toGrau(atual.ang - ang)
			else
				veiculo(i).spr.ang = toGrau(atual.ang + ang)
			end if
			
			veiculo(i).spr.coord.x = veiculo(i).spr.coord.x + (6 * cos(atual.ang + ang))
			veiculo(i).spr.coord.y = veiculo(i).spr.coord.y + (6 * sin(atual.ang + ang))
			veiculo(i).atualizaBoundaries
			
			'testa coliso
			dim as _bool_ colisao = false
			
			'...com pista
			for m = 0 to 7
				if not(trk.vl.encontra(veiculo(i).bounds.coord(m)) < 0) then colisao = true
			next m
			
			'...com outros veculos
			for m = 0 to i - 1
				if grid.colide(veiculo(i), veiculo(m)) or grid.colide(veiculo(m), veiculo(i)) then colisao = true
			next m
			
			if not colisao then exit for
		next j
	next i
	
	'balo
	i = NUM_VEICULOS - 1
	m = int(vl.cont / 2)
	
	dim atual as roadVertex
	atual = vl.getVertex(m)
	
	veiculo(i).novo @balao
	veiculo(i).spr.coord = atual.coord
	veiculo(i).spr.coord.z = 100
end sub

function track.getBalao () as coord3d
	return veiculo(NUM_VEICULOS - 1).spr.coord
end function

sub track.criaParticula (coord as coord3d, id as integer)
	'descomente esse cdigo se quiser controlar a criao em fps muito alto com rudo.
	'a melhor soluo  limitar o fps a 60 para uma animao constante.
	'------------------------
	'dim as double r
	'r = 80 / val(fps.getFps)
	'if rnd > r then exit sub

	'cria sprite
	dim temp as sprite2d
	temp.coord = coord
	temp.id = id
	temp.tam = 2.2
	temp.vencimento = 500
	
	'adiciona  lista
	sl.add temp
end sub

sub track.geraHeightmap
	dim as integer i, j, m
	
	print "Gerando heightmap..."
	
	'calcula os vrtices
	for i = 0 to HEIGHTMAP_RES - 1
		for j = 0 to HEIGHTMAP_RES - 1
			'acha coordenada
			dim c as coord3d
			c.nova
			c.x = ((i * (1024 / (HEIGHTMAP_RES - 1))) - 512) * FATOR_MAG
			c.y = ((j * (768 / (HEIGHTMAP_RES - 1))) - 384) * FATOR_MAG
			
			'acha altura
			if vl.encontra(c) < 0 then
				'acha distncia dos pontos
				dim as double min = 9999
				
				for m = 0 to vl.cont
					dim atual as roadVertex
					atual = vl.getVertex(m)
					
					if dist2p(c, atual.quad(2)) < min then min = dist2p(c, atual.quad(2))
					if dist2p(c, atual.quad(3)) < min then min = dist2p(c, atual.quad(3))
				next m
				
				c.z = int(edge((min - 25) * (aclive * 0.5), 0, 100))
				'c.z = c.z * -1
			else
				c.z = 0
			end if				
			
			heightmap(i, j) = c	
		next j
		
		'isso  muito lento, informa o progresso
		if (i mod 2) = 0 then menu.pintaLoading int((i / HEIGHTMAP_RES) * 60)
	next i
	
	'calcula geometria e tira normais das faces
	for i = 1 to HEIGHTMAP_RES - 2
		for j = 1 to HEIGHTMAP_RES - 2
			dim tri (0 to 5) as triangulo
			
			for m = 0 to 5
				tri(m).novo
				tri(m).v(0) = heightmap(i, j)
			next m
			
			'determina vizinhos
			if (i mod 2) = 1 then
				tri(0).v(1) = heightmap(i - 1, j)
				tri(0).v(2) = heightmap(i, j - 1)
				
				tri(1).v(1) = heightmap(i - 1, j + 1)
				tri(1).v(2) = heightmap(i - 1, j)

				tri(2).v(1) = heightmap(i, j + 1)
				tri(2).v(2) = heightmap(i - 1, j + 1)
				
				tri(3).v(1) = heightmap(i + 1, j + 1)
				tri(3).v(2) = heightmap(i, j + 1)
				
				tri(4).v(1) = heightmap(i + 1, j)
				tri(4).v(2) = heightmap(i + 1, j + 1)
				
				tri(5).v(1) = heightmap(i, j - 1)
				tri(5).v(2) = heightmap(i + 1, j)
			else
				tri(0).v(1) = heightmap(i - 1, j - 1)
				tri(0).v(2) = heightmap(i, j - 1)
			
				tri(1).v(1) = heightmap(i - 1, j)
				tri(1).v(2) = heightmap(i - 1, j - 1)
				
				tri(2).v(1) = heightmap(i, j + 1)
				tri(2).v(2) = heightmap(i - 1, j)
				
				tri(3).v(1) = heightmap(i + 1, j)
				tri(3).v(2) = heightmap(i, j + 1)
				
				tri(4).v(1) = heightmap(i + 1, j - 1)
				tri(4).v(2) = heightmap(i + 1, j)
				
				tri(5).v(1) = heightmap(i, j - 1)
				tri(5).v(2) = heightmap(i + 1, j - 1)
			end if
			
			normal(i, j).nova
			for m = 0 to 5
				'altera a altura para sombras mais agressivas
				tri(m).v(0).z = tri(m).v(0).z * 1.7
				tri(m).v(1).z = tri(m).v(1).z * 1.7
				tri(m).v(2).z = tri(m).v(2).z * 1.7
			
				'calcula
				tri(m).calculaNormal
				
				'soma ao vetor, opengl vai normalizar
				normal(i, j).x = normal(i, j).x + tri(m).normal.y
				normal(i, j).y = normal(i, j).y + tri(m).normal.y
				normal(i, j).z = normal(i, j).z + tri(m).normal.z
			next m
		next j
		
		if (i mod 2) = 0 then menu.pintaLoading int(60 + ((i / HEIGHTMAP_RES) * 40))
	next i
end sub

sub track.pintaHeightmap
	dim as integer i, j
	
	'heightmap
	if LOD >= 2 then
		env.setCor cor_grama
		gfx.grama.aplica
	else
		env.setCor cor_grama.r * 0.75, cor_grama.g * 0.75, cor_grama.b * 0.75
		_notex_
	end if
	
	glEnable GL_POLYGON_OFFSET_FILL
	glPolygonOffset 2, 1
	glEnable GL_LIGHTING
		
	for i = 0 to HEIGHTMAP_RES - 2
		glBegin GL_TRIANGLE_STRIP
			for j = 0 to HEIGHTMAP_RES - 1
				dim as integer m
			
				if (i mod 2) = 0 then
					m = j
				else
					m = (HEIGHTMAP_RES - 1) - j
				end if
				
				dim as coord3d c, n
				
				'primeiro
				n = normal(i, m)
				glNormal3f n.x, n.y, n.z
					
				c = heightmap(i, m)
				glTexCoord2f i, m
				glVertex3f c.x, c.y, c.z
									
				'segundo
				n = normal(i + 1, m)
				glNormal3f n.x, n.y, n.z
				
				c = heightmap(i + 1, m)	
				glTexCoord2f i + 1, m
				glVertex3f c.x, c.y, c.z
			next j
		glEnd
	next i
	
	glDisable GL_LIGHTING
		
	'parede externa
	env.setCor cor_grama
	gfx.arvores.aplica
	
	for i = 0 to HEIGHTMAP_RES - 1 step HEIGHTMAP_RES - 1
		glBegin GL_TRIANGLE_STRIP
			for j = 0 to HEIGHTMAP_RES - 1
				dim c as coord3d
				c = heightmap(i, j)
				
				glTexCoord2f j / 4, 0
				glVertex3f c.x, c.y, c.z + 20
				glTexCoord2f j / 4, 1
				glVertex3f c.x, c.y, c.z
			next j
		glEnd
		
		glBegin GL_TRIANGLE_STRIP
			for j = 0 to HEIGHTMAP_RES - 1
				dim c as coord3d
				c = heightmap(j, i)
				
				glTexCoord2f j / 4, 0
				glVertex3f c.x, c.y, c.z + 20
				glTexCoord2f j / 4, 1
				glVertex3f c.x, c.y, c.z
			next j
		glEnd
	next i
	
	glDisable GL_POLYGON_OFFSET_FILL
end sub

sub track.inicia
	'reseta valor de seed
	randomize vl.seed

	'cores
	cor_grama = bioma(int(edge(vl.cenario, 0, MAX_BIOMAS - 1)))

	'heightmap
	aclive = edge(0.05 + (vl.aclive * 0.325), 0, 1)
	geraHeightmap
	
	'billboard
	dim as integer i, altura, largura
	dim as roadVertex atual, prox
	dim ang as double
	
	i = vl.getRetaOposta
	atual = vl.getVertex(i)
	prox = vl.getVertex(i + 1)	
	
	largura = 16
	altura = 14
	
	if vl.somaAngulos < 0 then
		v_billboard(1) = interpola(atual.quad(2), prox.quad(2), 0.5)
		ang = ang2p(atual.quad(2), prox.quad(2))
	else
		v_billboard(1) = interpola(atual.quad(3), prox.quad(3), 0.5)
		ang = ang2p(prox.quad(3), atual.quad(3))
	end if
	v_billboard(1).z = altura
	
	v_billboard(0).x = v_billboard(1).x + (largura * cos(ang))
	v_billboard(0).y = v_billboard(1).y + (largura * sin(ang))
	v_billboard(0).z = altura
	
	v_billboard(2).x = v_billboard(1).x - (largura * cos(ang))
	v_billboard(2).y = v_billboard(1).y - (largura * sin(ang))
	v_billboard(2).z = altura
	
	'billboard suspensa
	altura = 10
	largura = 12
	
	for i = 0 to 1
		'esquerda ou direita?
		if i = 0 then
			atual = vl.getVertex(int(vl.cont * 0.25))
			ang = atual.ang + (pi / 2)
		else
			atual = vl.getVertex(int(vl.cont * 0.75))
			ang = atual.ang - (pi / 2)
		end if
						
		v_billboard2(i * 2) = atual.quad(2 + i)
		v_billboard2(i * 2).z = altura
		
		v_billboard2((i * 2) + 1).x = atual.quad(2 + i).x + (largura * cos(ang))
		v_billboard2((i * 2) + 1).y = atual.quad(2 + i).y + (largura * sin(ang))
		v_billboard2((i * 2) + 1).z = altura
	next
	
	'veculos
	plantaVeiculos
	
	'cria sprite list
	sl.nova
		
	'popula sprite list
	plantaArvores
	plantaFiscais
	
	'bandeira quadriculada
	atual = vl.getVertex(0)
	
	dim temp as sprite2d
	temp.coord = atual.quad(1)
	temp.id = 3
	temp.tam = 4
	temp.vencimento = -1
	sl.add temp
	
	'bandeirinhas dos postes
	for i = 0 to 2
		temp.coord = v_billboard(i)
		temp.id = 6
		temp.tam = 5
		temp.vencimento = -1
		sl.add temp
	next
	
	'pblico
	for i = -16 to 16 step 8
		temp.coord.x = atual.quad(2).x + (8 * cos(atual.ang - (pi / 2))) + (i * cos(atual.ang))
		temp.coord.y = atual.quad(2).y + (8 * sin(atual.ang - (pi / 2))) + (i * sin(atual.ang))
		temp.coord.z = 0
		temp.id = 9 + abs((i / 8) mod 2)
		temp.tam = 8
		temp.vencimento = -1
		sl.add temp
		
		temp.coord.x = atual.quad(3).x + (8 * cos(atual.ang + (pi / 2))) + (i * cos(atual.ang))
		temp.coord.y = atual.quad(3).y + (8 * sin(atual.ang + (pi / 2))) + (i * sin(atual.ang))
		temp.coord.z = 0
		temp.id = 9 + abs((i / 8) mod 2)
		temp.tam = 8
		temp.vencimento = -1
		sl.add temp
	next i
end sub

sub track.pintaPoste (coord as coord3d, bottom as double, top as double)
	const as double poste_w = 0.1
		
	glPushMatrix
	glTranslatef coord.x, coord.y, 0
				
	_notex_
	glBegin GL_QUAD_STRIP
		env.setCor 0.2, 0.2, 0.2
		glVertex3f poste_w * -1, 0, top
		glVertex3f poste_w * -1, 0, bottom
		
		env.setCor 0.7, 0.7, 0.7
		glVertex3f 0, poste_w * -1, top
		glVertex3f 0, poste_w * -1, bottom
		
		env.setCor 0.2, 0.2, 0.2
		glVertex3f poste_w, 0, top
		glVertex3f poste_w, 0, bottom
		
		env.setCor 0.7, 0.7, 0.7
		glVertex3f 0, poste_w, top
		glVertex3f 0, poste_w, bottom
		
		env.setCor 0.2, 0.2, 0.2
		glVertex3f poste_w * -1, 0, top
		glVertex3f poste_w * -1, 0, bottom
	glEnd
	
	_notex_
	env.setCor 0.4, 0.4, 0.4
	glBegin GL_QUADS
		glVertex3f poste_w * -1, 0, top
		glVertex3f 0, poste_w * -1, top
		glVertex3f poste_w, 0, top
		glVertex3f 0, poste_w, top
	glEnd
	env.setCor 1, 1, 1
	
	glPopMatrix
end sub

sub track.pintaObjetos
	dim as integer i, j
	
	'postes
	for i = 0 to 2
		pintaPoste v_billboard(i), MURO_H, v_billboard(i).z
	next
	
	'placas
	gfx.billboard.aplica
	glBegin GL_TRIANGLE_STRIP
		for i = 0 to 2
			glTexCoord2f i, 0.5
			glVertex3f v_billboard(i).x, v_billboard(i).y, 12
			
			glTexCoord2f i, 1
			glVertex3f v_billboard(i).x, v_billboard(i).y, 3
		next		
	glEnd
	
	'billboard suspensa
	dim as integer h = 3
	
	for i = 0 to 1
		dim as integer x
		if not cam.espelhada then
			x = 0
		else
			x = 1
		end if
	
		pintaPoste v_billboard2(i * 2), MURO_H, v_billboard2(i * 2).z
				
		glEnable GL_CULL_FACE
		
		gfx.billboard.aplica
		glBegin GL_QUADS
			'frente
			j = i * 2			
			glTexCoord2f x, 0.26
			glVertex3f v_billboard2(j).x, v_billboard2(j).y, v_billboard2(j).z
			glTexCoord2f x, 0.5
			glVertex3f v_billboard2(j).x, v_billboard2(j).y, v_billboard2(j).z - h
			
			j = j + 1			
			glTexCoord2f 1 - x, 0.5
			glVertex3f v_billboard2(j).x, v_billboard2(j).y, v_billboard2(j).z - h
			glTexCoord2f 1 - x, 0.26
			glVertex3f v_billboard2(j).x, v_billboard2(j).y, v_billboard2(j).z
			
			'trs
			j = j - 1
			glTexCoord2f 1 - x, 0.5
			glVertex3f v_billboard2(j).x, v_billboard2(j).y, v_billboard2(j).z - h
			glTexCoord2f 1 - x, 0.26
			glVertex3f v_billboard2(j).x, v_billboard2(j).y, v_billboard2(j).z
			
			j = j + 1
			glTexCoord2f x, 0.26
			glVertex3f v_billboard2(j).x, v_billboard2(j).y, v_billboard2(j).z
			glTexCoord2f x, 0.5
			glVertex3f v_billboard2(j).x, v_billboard2(j).y, v_billboard2(j).z - h
		glEnd
		
		glDisable GL_CULL_FACE	
	next i
	
	'---
	'ignore em detalhe mnimo
	if LOD = 0 then exit sub
	
	'veculos
	dim as integer old_lod = LOD
	if LOD = 3 then LOD = 2 'desliga reflexes nas decoraes
	
	for i = 0 to NUM_VEICULOS - 1
		veiculo(i).pinta
	next i
	
	LOD = old_lod
	
	'sprites
	env.setCor 1, 1, 1
	sl.pinta
end sub

sub track.pintaPista
	pintaHeightmap
	vl.pinta
end sub

'------------------------------------------------------------------------------------
sub cronometro.novo
	dim as integer i

	'zera contadores
	last_vertex = 0
	cont = 0
	volta = 0
	melhor = 0
	
	'zera folha de tempos
	for i = 0 to 127
		tempo(i) = 0
	next
end sub

function cronometro.formataTempo (t as longint) as string
	dim as string s = ""
	
	'negativo?
	if t < 0 then
		s = "-"
		t = abs(t)
	end if
	
	'minutos
	if t >= 60000 then
		s = s + str(t \ 60000)
		s = s + ":"
		
		t = t - (60000 * (t \ 60000))
		
		if t < 10000 then s = s + "0" 'completa duas casas
	end if
		
	'segundos
	s = s + str(t \ 1000)
	s = s + "."

	'milsimos
	t = t - (1000 * (t \ 1000))
	'completa trs casas
	if t < 100 then s = s + "0"
	if t < 10 then s = s + "0"
	
	s = s + str(t)
	
	'remove duas casas de centsimos
	's = left(s, len(s) - 2)
	
	return s	
end function

sub cronometro.pintaHud
	dim as integer i, s
		
	'ok...	
	fnt.setSombra true
	
	if volta >= 9 then
		s = volta - 9
	else
		s = 0
	end if
	
	for i = s to volta
		if volta > 0 and i = melhor then
			fnt.setCor cor_hl
		else
			fnt.setCor cor_texto
		end if
	
		fnt.pintaTopo str(i + 1) + ") " + formataTempo(tempo(i)), i - s + 4
	next
end sub

function cronometro.atras (cron as cronometro) as _bool_
	if this.volta < cron.volta then
		return true
	elseif this.volta = cron.volta then
		if this.cont < cron.cont then
			return true
		else
			return false
		end if
	else
		return false
	end if
end function

function cronometro.atualiza (vertex as integer) as _bool_
	'conta tempo
	tempo(volta) = tempo(volta) + ((1000 / 50) * fps.getFator)
	
	'verifica mudana de posio
	if vertex <> last_vertex then
		'completou uma volta
		if vertex = 0 and cont = trk.vl.cont then
			'melhor volta?
			if tempo(volta) <= tempo(melhor) then melhor = volta
			
			'acabou a corrida?
			if race.evt_wait_for_others then
				return true
			elseif volta = (race.voltas - 1) then
				race.evt_wait_for_others = true
				return true
			end if
			
			'incrementa contador de volta
			volta = int(wrap(volta + 1, 0, 127))
			cont = 0	
			
		'atualiza contador de posio
		else
			if vertex = 0 and last_vertex = trk.vl.cont then
				cont = cont + 1
			elseif vertex = trk.vl.cont and last_vertex = 0 then
				cont = cont - 1
			else
				cont = cont + (vertex - last_vertex)
			end if
		end if
	end if
	
	last_vertex = vertex
	return false
end function

function cronometro.getTotal () as string
	dim as longint total = 0
	dim i as integer
		
	for i = 0 to 127
		total = total + tempo(i)
	next
	
	return formataTempo(total)
end function

'------------------------------------------------------------------------------------
sub menuContainer.pintaFundo (texto as string)
	cam.atualiza
	
	'reposiciona luz
	glEnable GL_LIGHTING
	dim l_pos(0 to 3) as single => {0, -20, 15, 1}
	glLightfv GL_LIGHT0, GL_POSITION, @l_pos(0)
	
	'bandeira animada
	anim = wrap(anim - (0.05 * fps.getFator), 0, 2 * pi)
	
	glPushMatrix
	glTranslatef 0, -4, 11
	glRotatef 80, 1, 0, 0
		
	const as integer res = 2
	dim as integer x, y
		
	glColor3f 1, 1, 1
	bg.aplica
	
	for y = -13 to 14 step res
		glBegin GL_TRIANGLE_STRIP
			for x = -24 to 24 step res
				dim c(0 to 1) as coord3d
				
				c(0).x = x
				c(0).y = y
				c(0).z = sin((x / 4) + anim)
				
				c(1) = c(0)
				c(1).y = c(1).y + res
				
				dim as integer i
				for i = 0 to 1
					glNormal3f cos((x / 4) + anim) * -1, 0, 1
					glTexCoord2f c(i).x / 28, ((c(i).y * -1) + 14) / 28
					glVertex3f c(i).x, c(i).y, c(i).z
				next i
			next x
		glEnd
	next y
		
	glPopMatrix
	glDisable GL_LIGHTING
	
	'logo
	cam.orto
	
	if texto = "main menu" then
		glColor3f 1, 1, 1
		logo.aplica
		glBegin GL_QUADS
			glTexCoord2f 0, 0
			glVertex2f 0, 0
			glTexCoord2f 0, 1
			glVertex2f 0, h / 8
			glTexCoord2f 1, 1
			glVertex2f h / 2, h / 8
			glTexCoord2f 1, 0
			glVertex2f h / 2, 0
		glEnd
		
		fnt.setSombra false
		fnt.setCor cor_preto
		fnt.pinta VERSAO + " # 2013 Thiago Marques", _dx_, h / 8
	end if
		
	'texto
	_notex_
	for y = 0 to 1
		dim as single cor = 1 - y
	
		glBegin GL_LINES
			glColor4f cor, cor, cor, 0
			glVertex2f (w / 2), (_dy_ * 2.5) + y
			glColor4f cor, cor, cor, 1
			glVertex2f w, (_dy_ * 2.5) + y
		glEnd
	next
	
	fnt.setSombra true
	fnt.setCor cor_titulo
	fnt.pintaTopoDir texto, 1

	fnt.setCor cor_texto
	cam.restaura	
end sub

sub menuContainer.inicioLoop (intervalo_poll as double)
	fps.atualiza
	if intervalo_poll = 0 then kev.atualiza
	mus.toca
end sub

sub menuContainer.inicioLoop ()
	inicioLoop 0
end sub

sub menuContainer.fimLoop
	SDL_GL_SwapBuffers
	fps.preciseSleep	
end sub

sub menuContainer.pintaLoading (progresso as integer)
	inicioLoop
	pintaFundo " "
	
	cam.orto
	
	'fundo escurecido
	_notex_
	glColor4f 0, 0, 0, progresso / 100
	glBegin GL_QUADS
		glVertex2f 0, 0
		glVertex2f 0, h
		glVertex2f w, h
		glVertex2f w, 0
	glEnd
	
	fnt.pintaCentro "please wait$"
	
	'barra de progresso
	_notex_
	glBegin GL_QUADS
		glColor3f 0.5, 0.5, 1
		glVertex2f w * 0.35, h * 0.53
		
		glColor3f 0.2, 0, 0.8
		glVertex2f w * 0.35, h * 0.55
		
		glVertex2f w * (0.35 + (0.3 * (progresso / 100))), h * 0.55
		
		glColor3f 0.5, 0.5, 1
		glVertex2f w * (0.35 + (0.3 * (progresso / 100))), h * 0.53	
	glEnd	
	
	cam.restaura
	
	'fade na msica
	mus.setVolume 1 - (progresso / 100)
	
	fimLoop
end sub

sub menuContainer.escolheModo
	dim i as integer

	do
		inicioLoop
		
		'controles
		if kev.confereTecla(kev.K_ESCOLHE, kev.PRESSED) then exit do
		if kev.confereTecla(kev.K_RETORNA, kev.PRESSED) then modo_corrida = 6: exit do
		if kev.confereTecla(kev.K_UP, kev.PRESSED) and modo_corrida > 0 then modo_corrida = modo_corrida - 1
		if kev.confereTecla(kev.K_DOWN, kev.PRESSED) and modo_corrida < 6 then modo_corrida = modo_corrida + 1
		
		'grficos
		pintaFundo "main menu"
		
		'texto
		cam.orto
		
		for i = 0 to 6
			dim linha as integer
			linha = ((h / 2) / _dy_) - 5 + (2 * i)
			
			dim texto as string
			select case i
				case 0: texto = "free practice"
				case 1: texto = "quick race"
				case 2: texto = "grand prix race"
				case 3: texto = "endurance race"
				case 4: texto = "track editor"
				case 5: texto = "help"
				case 6: texto = "exit"
			end select
			
			if modo_corrida = i then
				fnt.setCor cor_hl
			else
				fnt.setCor cor_texto
			end if
			
			fnt.pintaCentro texto, linha
		next i
		
		cam.restaura
		
		fimLoop
	loop
end sub

sub menuContainer.escolheCarro
	do
		inicioLoop
		
		'animao
		escolha.spr.ang = wrap(escolha.spr.ang - (0.5 * fps.getFator), 0, 360)
		
		'controles
		if kev.confereTecla(kev.K_ESCOLHE, kev.PRESSED) or kev.confereTecla(kev.K_RETORNA, kev.PRESSED) then exit do
		if kev.confereTecla(kev.K_UP, kev.PRESSED) or kev.confereTecla(kev.K_DOWN, kev.PRESSED) then escolha.autogear = not escolha.autogear
		if kev.confereTecla(kev.K_LEFT, kev.PRESSED) then escolha.cor = escolha.cor - 1
		if kev.confereTecla(kev.K_RIGHT, kev.PRESSED) then escolha.cor = escolha.cor + 1
		if kev.confereTecla(kev.K_PAUSA, kev.PRESSED) then sleep 5000, 1 'apenas para screenshots
		
		if escolha.cor < 0 then escolha.cor = MAX_CORES - 1
		if escolha.cor >= MAX_CORES then escolha.cor = 0
		
		'grficos
		pintaFundo "please select your car"
		
		'plataforma 1
		dim i as integer
		dim c as double
		
		glPushMatrix
		glTranslatef 0, -12, 9
		glRotatef 12, 1, 0, 0		
		
		_notex_
		glBegin GL_TRIANGLE_FAN
			glVertex3f 0, 0, 0
		
			for i = 0 to 16 step 1
				c = 0.6 + (0.3 * tabs.seno((i * 2) mod 16))
				glColor3f c, c, c
				
				glVertex3f 5 * tabs.cosseno(i), 5 * tabs.seno(i), 0
			next
		glEnd
		
		'carro transparente
		glClear GL_DEPTH_BUFFER_BIT
		
		glPushMatrix
		glScalef 1, 1, -1
		
		escolha.pinta false
		
		glPopMatrix
		
		'plataforma 2
		_notex_
		glBegin GL_TRIANGLE_FAN
			glVertex3f 0, 0, 0
		
			for i = 0 to 16 step 1
				c = 0.6 + (0.3 * tabs.seno((i * 2) mod 16))
				glColor4f c, c, c, 0.75
				
				glVertex3f 5 * tabs.cosseno(i), 5 * tabs.seno(i), 0
			next
		glEnd
								
		glBegin GL_TRIANGLE_STRIP
			for i = 0 to 16 step 1
				c = 0.5 + (0.3 * tabs.seno((i * 2) mod 16))
				glColor4f c, c, c, 1
				
				glVertex3f 5 * tabs.cosseno(i), 5 * tabs.seno(i), 0
				glVertex3f 5.2 * tabs.cosseno(i), 5.2 * tabs.seno(i), -0.6
			next
		glEnd	
		
		'carro topo
		escolha.pinta false
		
		glPopMatrix
		
		'texto
		dim as string texto
		dim as integer y
		y = (h - (3 * _dy_)) / _dy_
		
		dim as double x, y1, y2
		x = (w / 2) - (4 * _dx_)
		y1 = h - (3 * _dy_)
		y2 = y1 + _dy_ 
		
		cam.orto
		
		fnt.pintaCentro "<                                      >"
		
		if escolha.autogear then
			texto = "(auto)"
		else
			texto = "(manual)"
		end if
		
		fnt.pintaCentro texto, y
		
		cam.restaura
		
		fimLoop
	loop
end sub

sub menuContainer.escolhePista
	dim as double ang = 0
	
	do
		inicioLoop
		
		'controles
		if kev.confereTecla(kev.K_ESCOLHE, kev.PRESSED) or kev.confereTecla(kev.K_RETORNA, kev.PRESSED) then exit do
		if kev.confereTecla(kev.K_UP, kev.PRESSED) then timeframe -= 1: if timeframe < 0 then timeframe = 4
		if kev.confereTecla(kev.K_DOWN, kev.PRESSED) then timeframe += 1: if timeframe > 4 then timeframe = 0
		if kev.confereTecla(kev.K_LEFT, kev.PRESSED) then trk.vl.load "trk/" + tracks.getArquivo(-1)
		if kev.confereTecla(kev.K_RIGHT, kev.PRESSED) then trk.vl.load "trk/" + tracks.proxArquivo
		if kev.confereTecla(kev.K_PAUSA, kev.PRESSED) then sleep 5000, 1
		
		'animao
		ang = wrap(ang - (0.5 * fps.getFator), 0, 360)
		
		'grficos
		pintaFundo "please select a race course"
		
		'pista
		glPushMatrix
		glTranslatef 0, -14, 9.5
		glScalef 0.009, 0.009, 0.009
		glRotatef 20, 1, 0, 0
		glRotatef ang, 0, 0, 1
		
		_notex_
		glColor3f 0.4, 0.4, 0.4 '0.6, 0.5, 0.3
		trk.vl.pintaSimples		
		
		glPopMatrix
		
		'texto
		dim as string texto
		dim as integer y
		y = (h - (3 * _dy_)) / _dy_
		
		cam.orto
		
		fnt.pintaCentro "<  " + tracks.getArquivo(0) + "  >"
		
		select case timeframe
			case 0:	texto = "(daytime)"
			case 1:	texto = "(nighttime)"
			case 2:	texto = "(current time)"
			case 3:	texto = "(daytime, frozen)"
			case 4: texto = "(current time, frozen)"
			case else: texto = "?"
		end select 
		
		fnt.pintaCentro texto, y
		
		cam.restaura
		
		fimLoop
	loop
end sub

sub menuContainer.pintaResultados
	dim i as integer

	do
		inicioLoop
		
		'controles
		if kev.confereTecla(kev.K_ESCOLHE, kev.PRESSED) or kev.confereTecla(kev.K_RETORNA, kev.PRESSED) then exit do
		
		'grficos
		pintaFundo "race results"
		
		'texto
		cam.orto
		
		fnt.pintaCentro tracks.getArquivo(0) + " - " + str(race.voltas) + " laps", 5
		
		dim lider as carro
		lider = grid.getCarro(0)
		
		for i = 0 to MAX_CARROS - 1
			dim linha as integer
			linha = 10 + (i * 2)
			
			dim atual as carro
			atual = grid.getCarro(i)
			
			if atual.piloto <> "No correu" then
				fnt.setCor cor_texto
				fnt.setSombra true
				fnt.pintaTopo "  " + str(i + 1) + ".", linha
				
				fnt.pintaCentro atual.piloto + " (" + atual.getInfo.nome + ")", linha
				
				if not atual.evt_dnf then
					if race.cron(atual.cron_id).volta = race.cron(lider.cron_id).volta then
						fnt.pintaTopoDir race.cron(atual.cron_id).getTotal + "  ", linha
					else
						fnt.pintaTopoDir "-" _
							+ str(race.cron(lider.cron_id).volta - race.cron(atual.cron_id).volta) + "  ", linha
					end if
				else
					fnt.pintaTopoDir "DNF  ", linha
				end if
				
				fnt.setCor palete(atual.cor)
				fnt.setSombra false
				fnt.pintaTopo "      >>>", linha
			end if
		next
		
		fnt.setCor cor_preto
		fnt.setSombra false
		fnt.pintaTopo "  pos color", 8
		fnt.pintaCentro "driver / car", 8
		fnt.pintaTopoDir "total time  ", 8
		
		cam.restaura
		
		fimLoop
	loop
end sub

sub menuContainer.pintaAjuda
	dim as integer res = 1
	if w >= 800 then res = 2
	
	dim as integer x, y
	x = (w / 2) - (200 * res)
	y = (h / 2) - (118 * res)

	do
		inicioLoop
		
		'controles
		if kev.confereTecla(kev.K_ESCOLHE, kev.PRESSED) or kev.confereTecla(kev.K_RETORNA, kev.PRESSED) then exit do
		
		'grficos
		pintaFundo "ingame controls"
		
		'imagem
		cam.orto
		
		'glColor4f 1, 1, 1, 0.9
		ajuda.aplica
		glBegin GL_QUADS
			glTexCoord2i 0, 0
			glVertex2f x, y
			
			glTexCoord2i 0, 1
			glVertex2f x, y + (236 * res)
			
			glTexCoord2i 1, 1
			glVertex2f x + (400 * res), y + (236 * res)
			
			glTexCoord2i 1, 0
			glVertex2f x + (400 * res), y
		glEnd
		
		cam.restaura
		
		fimLoop
	loop
end sub

sub menuContainer.configEditor
	'carrega
	dim as string arq, cur
	arq = ""
	cur = "-"
	
	dim as integer cursor = 1
	dim as double intervalo = 0
	
	do
		intervalo = fps.subtraiTempo(intervalo)
		inicioLoop intervalo
		
		'controles
		if kev.confereTecla(kev.K_ESCOLHE, kev.PRESSED) or kev.confereTecla(kev.K_RETORNA, kev.PRESSED) then exit do
		
		if intervalo = 0 then
			if kev.confereTecla(kev.K_UP, kev.DOWN) then
				if cur = "-" then
					cur = "a"
				elseif cur < "z" then
					cur = chr(asc(cur) + 1)
				else
					cur = "a"
				end if

			elseif kev.confereTecla(kev.K_DOWN, kev.DOWN) then
				if cur = "-" then
					cur = "z"
				elseif cur > "a" then
					cur = chr(asc(cur) - 1)
				else
					cur = "z"
				end if
				
			elseif kev.confereTecla(kev.K_LEFT, kev.DOWN) then
				if len(arq) > 0 then
					cur = right(arq, 1)
					arq = left(arq, len(arq) - 1)
				end if
				
			elseif kev.confereTecla(kev.K_RIGHT, kev.DOWN) then
				arq = arq + cur
				cur = "-"
			end if
			
			'configura intervalo entre updates
			if kev.confereTecla(kev.K_UP, kev.DOWN) or kev.confereTecla(kev.K_DOWN, kev.DOWN) _
				or kev.confereTecla(kev.K_LEFT, kev.DOWN) or kev.confereTecla(kev.K_RIGHT, kev.DOWN) then
					intervalo = 100
			end if
		end if
		
		'grficos
		pintaFundo "track editor"
		
		'texto
		cam.orto
		
		dim linha as integer
		linha = ((h / 2) / _dy_) - 1
		
		fnt.pintaCentro "enter filename to create or edit:", linha
		
		fnt.setCor cor_hl
		fnt.pintaCentro arq + cur, linha + 2
		
		fnt.setCor cor_titulo
		fnt.pintaCentro space(len(arq)) + "|", linha + 3
		
		fnt.setCor cor_preto
		fnt.setSombra false
		fnt.pintaCentro "track editor controls:                       ", linha + 6
		fnt.pintaCentro "mouse button 1 - place or drag control points", linha + 7
		fnt.pintaCentro "mouse button 2 - erase control point         ", linha + 8
		fnt.pintaCentro "(shift) up/down arrow - change road width    ", linha + 9
		fnt.pintaCentro "esc - quit the editor                        ", linha + 10
		
		
		cam.restaura
		
		fimLoop
	loop
	
	arq = arq + cur
	if kev.confereTecla(kev.K_RETORNA, kev.PRESSED) or arq = "-" then exit sub

	'passa fluxo para editor
	arq = "trk/" + arq
	editor.inicio arq		
	
	'salva?
	cursor = 0
	
	dim as string txt_cenario (0 to 2), txt_aclive (0 to 2)
	txt_cenario(0) = "summer"
	txt_cenario(1) = "autumn"
	txt_cenario(2) = "winter"
	
	txt_aclive(0) = "flat"
	txt_aclive(1) = "normal"
	txt_aclive(2) = "high"
	
	dim escolha (0 to 1) as integer
	escolha(0) = editor.getDefault(0)
	escolha(1) = editor.getDefault(1)
	
	do
		inicioLoop
		
		'controle
		if kev.confereTecla(kev.K_UP, kev.PRESSED) then
			if cursor > 0 then cursor = cursor - 1
		elseif kev.confereTecla(kev.K_DOWN, kev.PRESSED) then
			if cursor < 3 then cursor = cursor + 1
		end if
		
		if cursor = 0 or cursor = 1 then
			if kev.confereTecla(kev.K_LEFT, kev.PRESSED) then
				if escolha(cursor) > 0 then escolha(cursor) = escolha(cursor) - 1
			elseif kev.confereTecla(kev.K_RIGHT, kev.PRESSED) then
				if escolha(cursor) < 2 then escolha(cursor) = escolha(cursor) + 1
			end if
		else
			if kev.confereTecla(kev.K_ESCOLHE, kev.PRESSED) then exit do
		end if
		
		'grficos
		pintaFundo "track editor"
		
		'texto		
		cam.orto
		
		dim i as integer
		for i = 0 to 3
			dim linha as integer
			linha = ((h / 2) / _dy_) - 3 + (2 * i)
			
			dim texto as string
			select case i
				case 0: texto = "scenery: " + txt_cenario(escolha(0))
				case 1: texto = "hill steepness: " + txt_aclive(escolha(1))
				case 2: texto = "save changes to '" + right(arq, len(arq) - 4) + "'"
				case 3: texto = "exit without saving"
			end select
			
			if cursor = i then
				fnt.setCor cor_hl
				if i = 0 or i = 1 then texto = "< " + texto + " >"
			else
				fnt.setCor cor_texto
			end if
			
			fnt.pintaCentro texto, linha
		next i
		
		cam.restaura
		
		fimLoop
	loop
	
	if cursor = 2 then
		editor.salva arq, escolha(0), escolha(1)
		
		'recarrega fileList
		tracks.nova "trk/*"
		trk.vl.load "trk/" + tracks.getArquivo(0)
	end if
end sub

sub menuContainer.configCorrida
	dim as integer i, num_voltas, num_carros

	select case modo_corrida
		'treino livre
		case 0
			num_voltas = 9999
			num_carros = 1
		
		'quickrace
		case 1:
			num_voltas = 2
			num_carros = 5
		
		'grand prix
		case 2:
			num_voltas = 5
			num_carros = 8
			
		'endurance
		case else:
			num_voltas = 20
			num_carros = 8
	end select
	
	race.nova num_voltas
	
	grid.novo
	grid.add @gol, "The Player", escolha.cor, escolha.autogear
	
	for i = (10 - num_carros) to 8
		select case i
			case 2: grid.add @gol, "Miles Behind", 0.85
			case 3: grid.add @gol, "Willy Swerve", 0.88
			case 4: grid.add @gol, "Percy Veer", 0.91
			case 5: grid.add @gol, "Otto Partz", 0.93	
			case 6: grid.add @gol, "Slippy Ann Slidin'", 0.95	
			case 7: grid.add @gol, "Max Throttle", 0.97
			case 8: grid.add @gol, "Hugh Jengine", 1				
		end select
	next i
	grid.finaliza
	
	som.setVolume som.getPlayer, 0
	som.setVolume som.getPlayer + 1, 0
	
	replay.novo
	cam.nova
	cam.up
	
	select case timeframe
		case 0: env.setHora 12, 0.0005
		case 1: env.setHora 0, 0.0005
		case 2: env.setHora val(left(time, 2)), 0.0005
		case 3: env.setHora 12, 0
		case 4: env.setHora val(left(time, 2)), 0
	end select	
	
	jogo
end sub

sub menuContainer.inicio
	'l todas as pistas
	tracks.nova "trk/*" 'isso  platform independent?
	
	'carrega primeira
	trk.vl.load "trk/" + tracks.getArquivo(0)
	
	'configura carro
	dim origem as coord3d
	origem.nova

	escolha.novo @gol
	escolha.cor = 21 'tenta default branco
	escolha.spr.coord = origem

	'no mostra resultados no comeo
	race.evt_fim = false

	'fluxo de menus
	timeframe = 0
	modo_corrida = 1
	jukebox = 1
	
	do
		mus.carrega "mus/" + str(jukebox) + ".ogg"
		
		env.setHora 12, 0
		env.atualiza
		cam.nova
	
		if race.evt_fim then pintaResultados
	
		topo:
		escolheModo
		select case modo_corrida
			case 4:
				configEditor				
				goto topo
			case 5:
				pintaAjuda
				goto topo
			case 6:
				fim
		end select
		
		escolhePista
		if kev.confereTecla(kev.K_RETORNA, kev.PRESSED) then goto topo
			
		escolheCarro
		if kev.confereTecla(kev.K_RETORNA, kev.PRESSED) then goto topo
		
		trk.inicia
		
		mus.destroi
		jukebox = jukebox + 1
		if jukebox > MAX_MUSICAS then jukebox = 1
		
		configCorrida
	loop
end sub
